;================================================================================
;
;CONSOLE CONTROL MACROS
;
;	
;	The following macros control the display of the WYSE 60 family
;	of terminals, which includes the WY60, WY150, WY160, WY325 &
;	GPT units when operating in native mode.  Currently, the WY325
;	color control sequences are not implemented.
;
;	The extended box graphics characters are printed from the sec-
;	ondary character set, which is derived from font bank 1 after
;	it has been loaded with the multinational character set.
;	
;
;
;	clearing data...
;
bs       .macro                ;destructive backspace
         .byte a_bs,a_esc,"W"
         .endm
;
ce       .macro                ;clear to end of screen
         .byte a_esc,"Y"
         .endm
;
cf       .macro                ;clear foreground
         .byte a_esc,";"
         sf
         .endm
;
cl       .macro                ;clear to end of line 
         .byte a_esc,"T"
         .endm
;
cs       .macro                ;clear screen
         .byte a_esc,"+"
         .endm
;
;
;	cursor control...
;
cpos     .macro .x,.y          ;position cursor at col,row
         .byte a_esc,"="
         .byte .y+32,.x+32
         .endm

ch       .macro                ;home cursor
         .byte a_esc,"{"
         .endm
;
cn       .macro                ;cursor on
         .byte a_esc,"`1"
         .endm
;
co       .macro                ;cursor off
         .byte a_esc,"`0"
         .endm
;
cr       .macro                ;carriage return
         .byte a_cr
         .endm
;
cu       .macro                ;reverse linefeed
         .byte a_esc,"j"
         .endm
;
lf       .macro                ;carriage return/line feed
         cr
         .byte a_lf
         .endm
;
;
;	display attributes...
;
daleadin .macro                ;attribute change lead-in
         .byte a_esc,"G"
         .endm
;
bf       .macro                ;reverse foreground
         .byte a_esc,"("
         daleadin
         .byte "4"
         .endm
;
br       .macro                ;reverse background
         .byte a_esc,")"
         daleadin
         .byte "t"
         .endm
;
er       .macro                ;enable normal foreground
         .byte a_esc,"("
         daleadin
         .byte "0"
         .endm
;
sb       .macro                ;set background
         .byte a_esc,")"
         daleadin
         .byte "p"
         .endm

sf       .macro                ;set foreground
         .byte a_esc,"("
         daleadin
         .byte "0"
         .endm
;
;
;	display editing...
;
cd       .macro                ;delete column
         .byte a_esc,"cJ"
         .endm
;
ci       .macro                ;insert column
         .byte a_esc,"cM"
         .endm
;
dc       .macro                ;delete character
         .byte a_esc,"W"
         .endm
;
ic       .macro                ;insert character
         .byte a_esc,"Q"
         .endm
;
ld       .macro                ;delete line
         .byte a_esc,"R"
         .endm
;
li       .macro                ;insert line
         .byte a_esc,"E"
         .endm
;
;
;	font selection...
;
lscs     .macro                ;set up secondary character set
         .byte a_esc,"c@A"
         .byte a_esc,"cC1"
         .endm
;
spcs     .macro                ;select primary character set
         .byte a_esc,"cD"
         .endm
;
sscs     .macro                ;select secondary character set
         .byte a_esc,"cE"
         .endm
;
;
;	box graphics characters...
;
g0       .macro                ;horizontal line
         bg
         .byte ":"
         eg
         .endm
;
g1       .macro                ;vertical line
         bg
         .byte "6"
         eg
         .endm
;
g2       .macro                ;top left corner
         bg
         .byte "2"
         eg
         .endm
;
g3       .macro                ;top right corner
         bg
         .byte "3"
         eg
         .endm
;
g4       .macro                ;bottom left corner
         bg
         .byte "1"
         eg
         .endm
;
g5       .macro                ;bottom right corner
         bg
         .byte "5"
         eg
         .endm
;
g6       .macro                ;left tee |
         bg
         .byte "4"
         eg
         .endm
;
g7       .macro                ;right tee |
         bg
         .byte "9"
         eg
         .endm
;
g8       .macro                ;bottom tee
         bg
         .byte "="
         eg
         .endm
;
g9       .macro                ;top tee
         bg
         .byte "0"
         eg
         .endm
;
ga       .macro                ;four-way connect
         bg
         .byte "8"
         eg
         .endm
;
gb       .macro                ;solid block
         bg
         .byte "7"
         eg
         .endm
;
gc       .macro                ;dark hash block
         bg
         .byte ";"
         eg
         .endm
;
gd       .macro                ;light hash block
         bg
         .byte "?"
         eg
         .endm
;
ge       .macro                ;double vertical line
         bg
         .byte ">"
         eg
         .endm
;
gf       .macro                ;double horizontal line
         bg
         .byte "<"
         eg
         .endm
;
;
;	extended box graphics characters...
;
xg0      .macro                ;horizontal line
         spcs
         .byte "M"
         sscs
         .endm
;
xg1      .macro                ;vertical line
         spcs
         .byte ":"
         sscs
         .endm
;
xg2      .macro                ;top left corner
         spcs
         .byte "I"
         sscs
         .endm
;
xg3      .macro                ;top right corner
         spcs
         .byte ";"
         sscs
         .endm
;
xg4      .macro                ;bottom left corner
         spcs
         .byte "H"
         sscs
         .endm
;
xg5      .macro                ;bottom right corner
         spcs
         .byte "<"
         sscs
         .endm
;
xg6      .macro                ;left tee |
         spcs
         .byte "L"
         sscs
         .endm
;
xg7      .macro                ;right tee |
         spcs
         .byte "9"
         sscs
         .endm
;
xg8      .macro                ;bottom tee
         spcs
         .byte "J"
         sscs
         .endm
;
xg9      .macro                ;top tee
         spcs
         .byte "K"
         sscs
         .endm
;
xga      .macro                ;four-way connect
         spcs
         .byte "N"
         sscs
         .endm
;
xgb      .macro                ;left transition tee |
         spcs
         .byte "G"
         sscs
         .endm
;
xgc      .macro                ;right transition tee |
         spcs
         .byte "6"
         sscs
         .endm
;
xgd      .macro                ;bottom transition tee
         spcs
         .byte "O"
         sscs
         .endm
;
xge      .macro                ;top transition tee
         spcs
         .byte "Q"
         sscs
         .endm
;
xgf      .macro                ;half solid block
         spcs
         .byte "\"
         sscs
         .endm
;
;
;	miscellaneous control...
;
bg       .macro                ;enable graphics mode
         .byte a_esc,"H",a_stx
         .endm
;
dn       .macro                ;display on
         .byte a_esc,"`9"
         .endm
;
do       .macro                ;display off
         .byte a_esc,"`8"
         .endm
;
eg       .macro                ;enable text mode
         .byte a_esc,"H",a_etx
         .endm
;
rb       .macro                ;ring bell
         .byte a_bel
         .endm
;
	.end
