;
;
;EPHEMERAL DEFINITIONS
;
.s_byte  =1                    ;size of a byte
.s_word  =.s_byte*2            ;size of a word
;
;
;	65C816 register sizes...
;
.s_mpudb =.s_byte              ;data bank
.s_mpudp =.s_word              ;direct page
.s_mpupb =.s_byte              ;program bank
.s_mpupc =.s_word              ;program counter
.s_mpusp =.s_word              ;stack pointer
.s_mpusr =.s_byte              ;status
;
;
;	status register bits...
;
.sr_car  =@00000001            ;C  carry
.sr_bdm  =@00001000            ;D  decimal
.sr_irq  =@00000100            ;I  IRQ
.sr_neg  =@10000000            ;N  result negative
.sr_ovl  =@01000000            ;V  sign overflow
.sr_zer  =@00000010            ;Z  result zero
.sr_amw  =@00100000            ;m  accumulator/memory size
.sr_ixw  =@00010000            ;x  index sizes
;
;
;	stack definitions...
;
.sfbase  .= 1                  ;base stack index
.sfidx   .= .sfbase            ;workspace index
;
;> workspace stack frame start <
.ws1     =.sfidx               ;workspace element
.sfidx   .= .sfidx+.s_word
;> workspace stack frame end <
;
.s_wsf   =.sfidx-.sfbase       ;stack workspace size
.sfbase  .= .sfidx
;
;> register stack frame start <
.reg_c   =.sfidx               ;.C
.sfidx   .= .sfidx+.s_word
.reg_x   =.sfidx               ;.X
.sfidx   .= .sfidx+.s_word
.reg_y   =.sfidx               ;.Y
.sfidx   .= .sfidx+.s_word
.reg_db  =.sfidx               ;DB
.sfidx   .= .sfidx+.s_mpudb
.reg_sr  =.sfidx               ;SR
.sfidx   .= .sfidx+.s_mpusr
.reg_pc  =.sfidx               ;PC
.sfidx   .= .sfidx+.s_mpupc
;> register stack frame end <
;
.s_rsf   =.sfidx-.sfbase       ;register stack frame size
.sfbase  .= .sfidx
;
;> parameter stack frame start <
.parm1   =.sfidx               ;parameter 1
.sfidx   .= .sfidx+.s_word
;> parameter stack frame end <
;
.s_psf   =.sfidx-.sfbase       ;parameter stack frame size
;
;
;	error flags...
;
.er_ef1  =.sr_zer              ;flag 1
.er_ef2  =.sr_ovl              ;flag 2
.er_ef3  =.sr_neg              ;flag 3
.er_bits =.er_ef1|.er_ef2|.er_ef3 ;mask
;


;
.done    oras .reg_sr          ;modify SR according...
         stas .reg_sr          ;to exit status
         longa
         clc
         tsc
         adcw .s_wsf
         tcs                   ;discard stack workspace
         adcw .s_rsf
         tax
         adcw .s_psf
         tay
         ldaw .s_rsf-1
         mvp 0,0               ;realign stack
         tya
         tcs
         pla
         plx
         ply
         plb
         plp
         rts
