;================================================================================
;
;CONVENIENCE MACROS
;
break    .macro                ;set debugging break point
         brk
         nop
         .endm
;
breakspc .macro                ;set debugging break point...
         php                   ;resets DP to monitor default
         pea kerneldp
         pld
         plp
         break
         tsc
         tcd
         .endm
;
bytegen  .macro .op            ;generate BYTE value
         .byte .op
         .endm
;
const    .macro .s,.c          ;generate constant value .C of .S bytes (.S > 1)
         .if .s > 1 && .s < 5
             .word .c & $ffff
             .if .s == 3
                 .byte .c >> 16 & $ffff
             .endif
             .if .s == 4
                 .word .c >> 16
             .endif
         .else
             .error ""+0$+": invalid constant size: must be 2, 3 or 4"
         .endif
         .endm
;
cpxym32  .macro .s,.d          ;mem-to-mem 32 bit copy (uses .X & .Y)
         ldx .s
         ldy .s+2
         stx .d
         sty .d+2
         .endm
;
cursor   .macro .c,.r          ;set cursor position: col,row
         php
         shortr
         ldx #.c
         ldy #.r
         jsr plot
         plp
         .endm
;
decmemw  .macro .op            ;32 bit decrement memory (uses .C)
         lda .op
         bne .0000010
         dec .op+2
.0000010 dec .op
         lda .op
         ora .op+2
         .endm
;
dwordgen .macro .c             ;generate DWORD value
         .word .c & $ffff,.c >> 16
         .endm
;        
dxy      .macro                ;16 bit decrement .X/.Y (lo/hi)
         cpx #0
         bne .0000010
         dey
.0000010 dex
         .endm
;
dxyw     .macro                ;32 bit decrement .X/.Y (lo/hi)
         cpxw 0
         bne .0000010
         dey
.0000010 dex
         .endm
;
incmem   .macro .op            ;16 bit increment memory
         inc .op
         bne .0000010
         inc .op+1
.0000010 ;this line intentionally has no code
         .endm
;
incmemw  .macro .op            ;32 bit increment memory
         inc .op
         bne .0000010
         inc .op+2
.0000010 ;this line intentionally has no code
         .endm
;
ixy      .macro                ;increment .X/.Y (lo/hi)
         inx
         bne .0000010
         iny
.0000010 ;this line intentionally has no code
         .endm
;
laz      .macro                ;load .A with zero (8 bit)
         lda #0
         .endm
;
labz     .macro                ;load .A & .B with zero (8 bit)
         lda #0
         xba
         lda #0
         .endm
;
lxy      .macro .op            ;load .X/.Y with LSB/MSB
         ldx #<.op
         ldy #>.op
         .endm
;
lxyi32   .macro .op            ;load immediate .X/.Y with LSW/MSW
         ldxw .op & $ffff
         ldyw .op >> 16
         .endm
;
lxym32   .macro .op            ;load .X/.Y with LSW/MSW from memory
         ldx .op
         ldy .op+2
         .endm
;
lxz      .macro                ;load .X with zero (8 bit)
         ldx #0
         .endm
;
lyz      .macro                ;load .Y with zero (8 bit)
         ldy #0
         .endm
;
printchr .macro .s             ;print character to console
         php
         shorta
         lda #.s
         jsr putcha
         plp
         .endm
;
pullall  .macro                ;restore .Y, .X & .A
         ply
         plx
         pla
         .endm
;
pushall  .macro                ;preserve .A, .X & .Y
         pha
         phx
         phy
         .endm
;
reserve  .macro .s             ;reserve .S bytes of storage
         *=*+.s
         .endm
;
reverspc .macro .ct            ;reverse program counter CT times
         *=*-3*.ct
         .endm
;
shiftfac .macro .m,.r          ;convert power-of-2 to number of shifts...
;
;	
;	Invocation: shiftfac M,R, where M is an exact multiple of 2, such as 256
;	or 65536, & R is 'a', 'x' or 'y', representing the register into which
;	the result is to be loaded.  If the value passed in R is lower case an 8
;	bit load will occur.  If upper case, the load will be 16 bits.  Results
;	are undefined if M is not an exact multiple of 2.
;	
;
.s       .set 0
         .rept 31
           .if .m == 1 << .s
             .if .r == 'a'
               lda #.s
             .else
               .if .r == 'x'
                 ldx #.s
               .else
                 .if .r == 'y'
                   ldy #.s
                 .else
                   .if .r == 'A'
                     ldaw .s
                   .else
                     .if .r == 'X'
                       ldxw .s
                     .else
                       .if .r == 'Y'
                         ldyw .s
                       .else
                         .error ""+%0$+": REGISTER MUST BE 'a|A', 'x|X' or 'y|Y'"
                       .endif
                     .endif
                   .endif
                 .endif
               .endif
             .endif
             .exitm
           .else
.s           .= .s + 1
           .endif
         .endr
         .error ""+%0$+": INVALID OPERAND"
         .endm
;
sleep    .macro .s             ;pause execution .S seconds
         php
         longa
         pha
         ldaw .s
         jsr utdelay
         pla
         plp
         .endm
;
sxy      .macro .op            ;store .X/.Y LSB/MSB
         stx .op
         sty .op+1
         .endm
;
sxy32    .macro .op            ;store .X/.Y LSW/MSW
         stx .op
         sty .op+2
         .endm
;
wordgen  .macro .op            ;generate WORD value
         .word .op & $ffff
         .endm
;
xwordgen .macro .op            ;generate XWORD value
         .word .op & $ffff
         .byte .op >> 16 & $ffff
         .endm
;
	.end