;===============================================================================
;
;<funcname>: <SUBROUTINE TITLE>
;
;	
;	Calling syntax: 
;
;	Exit registers: .A: 
;	                .B: 
;	                .X: 
;	                .Y: 
;	                DB: 
;	                DP: 
;	                PB: 
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    |||||||+> 
;	                    ||||||+> 
;	                    |||||+> 
;	                    ||||+> 
;	                    |||+> 
;	                    ||+> 
;	                    |+> 
;	                    +> 
;
;	Notes: 
;
;	Examples: 
;	
;
funcname ;*** this line intentionally has no code ***;
;
;
;LOCAL DECLARATIONS
;
.s_byte  =1                    ;size of a byte
.s_word  =.s_byte*2            ;size of a word
.s_dword =.s_word*2            ;size of a double word
;
;
;	65C816 register sizes...
;
.s_mpudb =.s_byte              ;data bank
.s_mpudp =.s_word              ;direct page
.s_mpupb =.s_byte              ;program bank
.s_mpupc =.s_word              ;program counter
.s_mpusp =.s_word              ;stack pointer
.s_mpusr =.s_byte              ;status
;
;
;	status register bits...
;
.sr_car  =@00000001            ;C  carry
.sr_zer  =@00000010            ;Z  result zero
.sr_irq  =@00000100            ;I  IRQ
.sr_bdm  =@00001000            ;D  decimal
.sr_amw  =@00100000            ;m  accumulator/memory size
.sr_ixw  =@00010000            ;x  index sizes
.sr_ovl  =@01000000            ;V  sign overflow
.sr_neg  =@10000000            ;N  result negative
;
;
;	stack definitions...
;
.sfbase  .= 1                  ;base stack index
.sfidx   .= .sfbase            ;workspace index
;
;> workspace stack frame start <
;     *** enter workspace definitions here ***
;> workspace stack frame end <
;
.s_wsf   =.sfidx-.sfbase       ;workspace stack frame size
.sfbase  .= .sfidx
;
;> MPU register stack frame start <
.reg_c   =.sfidx               ;.C
.sfidx   .= .sfidx+.s_word
.reg_x   =.sfidx               ;.X
.sfidx   .= .sfidx+.s_word
.reg_y   =.sfidx               ;.Y
.sfidx   .= .sfidx+.s_word
.reg_db  =.sfidx               ;DB
.sfidx   .= .sfidx+.s_mpudb
.reg_dp  =.sfidx               ;DP
.sfidx   .= .sfidx+.s_mpudp
.reg_sr  =.sfidx               ;SR
.sfidx   .= .sfidx+.s_mpusr
.reg_pc  =.sfidx               ;PC
.sfidx   .= .sfidx+.s_mpupc
;> MPU register stack frame end <
;
.s_rsf   =.sfidx-.sfbase       ;register stack frame size
.sfbase  .= .sfidx
;
;> parameter stack frame start <
;  *** enter call parameter definitions here ***
;> parameter stack frame end <
;
.s_psf   =.sfidx-.sfbase       ;parameter stack frame size
;
;
         php                   ;save MPU state
         phd
         phb
         longr
         phy
         phx
         pha
         cld
         .if .s_wsf            ;create workspace if defined
             sec
             tsc
             sbcw .s_wsf
             tcs
         .endif
;
.main    ;*** code body goes here ***;
;
.done    longr                 ;common exit point
         .if .s_wsf || .s_psf  ;clean up stack if necessary
             clc
             tsc
             .if .s_wsf
                 adcw .s_wsf
                 tcs
             .endif
             .if .s_psf
                 adcw .s_rsf
                 tax
                 adcw .s_psf
                 tay
                 ldaw .s_rsf-1
                 mvp 0,0
                 tya
                 tcs
             .endif
         .endif
         pla                   ;restore MPU state
         plx
         ply
         plb
         pld
         plp
         rts                   ;return to caller
;
	.end