;===============================================================================
;
;BIOS API FUNCTION INDICES
;
;	
;	This file contains index numbers that must be used to call BIOS API
;	functions.  The call is made with COP <api_idx>, where <api_idx> is the
;	index number of the desired function.  For example, to read from serial
;	I/O channel 'C':
;
;	    ldx #nxpchanc         ;channel index
;	    cop kchin             ;set channel C as default
;	    bcs error             ;invalid channel number
;	    cop kchget            ;read from channel C
;	    bcs nodatum           ;no data available
;	    sta datum             ;store datum
;
;	Prior to making the call, be sure to load the registers as required for
;	the API function being called.  In some cases, it will be necessary to
;	push parameters to the stack before calling the API function.  A return
;	from the call will automatically perform stack housekeeping.
;
;	Upon return, carry will be set to indicate if a failure occurred, where
;	the meaning of "failure" varies from call to call.  If carry is set,
;	.C will be loaded with an 8-bit error code.  Error codes are defined in
;	in the next section of this file.
;
;	WARNING!!!  DO NOT edit this file!
;	
;
hrst     =$00                  ;soft-reset system
mon      =$01                  ;M/L monitor entry
;
;
;	serial I/O (SIO) functions...
;
kchget   =$02                  ;read SIO channel
kchput   =$03                  ;write SIO channel
kchin    =$04                  ;set SIO input channel
kchout   =$05                  ;set SIO output channel
kchset   =$06                  ;set default channels
kchrst   =$07                  ;reset SIO channel
kchcur   =$08                  ;get current SIO channels
;
;	
;	API numbers $09 & $0A are reserved.
;	
;
;
;	kernel timekeeping functions...
;
ktodget  =$0b                  ;get date & time
kuptget  =$0c                  ;get uptime
ktodput  =$0d                  ;set date & time
ktdygen  =$0e                  ;generate time delay
;
;	
;	API numbers $0F, $10 & $11 are reserved.
;	
;
;
;	SCSI subsystem functions...
;
kscsinit =$12                  ;initialize SCSI subsystem
kscsparm =$13                  ;get SCSI device parameters
kscshaid =$14                  ;get SCSI host adapter ID
kscscmd  =$15                  ;execute SCSI command
;
;	
;	API numbers $16, $17 & $18 are reserved.
;	
;
;
;	RTC/NVRAM functions...
;
krtcget  =$19                  ;read RTC date & time
knvrget  =$1a                  ;read NVRAM
kdtrput  =$1b                  ;write RTC date & time
knvrput  =$1c                  ;write NVRAM
;
;	
;	API numbers $0D, $1E & $1F are reserved.
;	
;
;
;	console functions...
;
kscget   =$20                  ;read char from console
kscput   =$21                  ;write char to console
kscplot  =$22                  ;position console cursor
kscprint =$23                  ;print char string to console
;
;	
;	API numbers $24 & $25 are reserved.
;	API numbers $80 & up are not used.
;	
;
;===============================================================================
;
;BIOS API FUNCTION ERROR CODES
;
;	
;	In the event a BIOS API function returns with carry set one of the fol-
;	lowing 8-bit codes may be returned in .C.  Otherwise, .C will contain
;	data or its entry value.
;
;	WARNING!!!  DO NOT edit this file!
;	
;
e_okay   =$0000                ;no error...
;
;	
;	E_OKAY is returned by functions that otherwise have no returns & are
;	not expected to preserve .C.
;	
;
e_sioss  =$0001                ;SIO subsystem not ready
e_scsiss =$0002                ;SCSI subsystem not ready
;
;
;	SIO-related errors...
;
e_sioch  =$0010                ;invalid SIO channel requested
e_stlen  =$0011                ;string too long
;
;
;	SCSI subsystem errors...
;
e_sstid  =$0020                ;target ID out of range
e_ssdne  =$0021                ;device not enumerated
e_ssdnp  =$0022                ;device not present
e_sscge  =$0023                ;controller general error
e_sscfe  =$0024                ;controller FIFO error
e_sscmd  =$0025                ;illegal controller command
e_ssubr  =$0040                ;unexpected bus reset
e_ssubp  =$0041                ;unsupported bus phase
e_ssudf  =$0042                ;unsupported driver function
e_ssucg  =$0043                ;unsupported command group
e_ssabt  =$0044                ;aborted transaction
e_sschk  =$0080                ;check condition
;
;
;	miscellaneous...
;
e_apiidx =$ffff                ;bad API index or undefined function
;
	.end