;================================================================================
;
;KERNEL DIRECT PAGE STORAGE
;
kerneldp =zeropage             ;kernel's notion of direct page
;
;
;	timekeeping workspace...
;
tiudctl  =kerneldp             ;time update control
titdflag =tiudctl+s_byte       ;sleep control
titimesl =titdflag+s_byte      ;sleep expiration time
tijiffux =titimesl+s_utcnt     ;jiffy counter
titimeup =tijiffux+s_byte      ;system uptime (seconds)
titimeux =titimeup+s_utcnt     ;816NIX time (seconds)
;
;
;	SIO driver workspace...
;
siozpptr =titimeux+s_time_t    ;start of pointer array
siosr    =siozpptr             ;channel status registers
siocr    =siosr+s_nxpchp       ;channel command registers
siofif   =siocr+s_nxpchp       ;channel FIFOs
siogetrx =siofif+s_nxpchp      ;RxQ 'get' pointers
sioputrx =siogetrx+s_nxpchp    ;RxQ 'put' pointers
siogettx =sioputrx+s_nxpchp    ;TxQ 'get' pointers
sioputtx =siogettx+s_nxpchp    ;TxQ 'put' pointers
siotxst  =sioputtx+s_nxpchp    ;TxD status bit field
siochcur =siotxst+s_byte       ;default channels...
;
;	0xxx0xxx
;	 ||| |||
;	 ||| +++> input
;	 +++> output
;
sioirqst =siochcur+s_byte      ;vQUART IRQ status workspace
;
;
;	SCSI driver workspace...
;
scsihba  =sioirqst+s_byte      ;$80 = HBA detected
scsididx =scsihba+s_byte       ;device table pointer (16-bit)
scsigidx =scsididx+s_ptr       ;CDB size table index
scsistat =scsigidx+s_byte      ;status byte
sccdbptr =scsistat+s_byte      ;CDB pointer (24-bit)
;
;
;	SCSI enumeration workspace...
;
scsiedev =sccdbptr+s_xptr      ;current device
scsierow =scsiedev+s_byte      ;display row
scsiztmp =scsierow+s_byte      ;temporary storage
;
;
;	miscellaneous...
;
gpptr    =scsiztmp+s_byte      ;multipurpose extended pointer
;
zeropage .=gpptr+s_xptr        ;new start of zero page
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR DIRECT PAGE STORAGE
;
reg_pbx  =zeropage             ;PB shadow
reg_pcx  =reg_pbx+s_mpupbx     ;PC shadow
reg_srx  =reg_pcx+s_mpupcx     ;SR shadow
reg_ax   =reg_srx+s_mpusrx     ;.C shadow
reg_xx   =reg_ax+s_word        ;.X shadow
reg_yx   =reg_xx+s_word        ;.Y shadow
reg_spx  =reg_yx+s_word        ;SP shadow
reg_dpx  =reg_spx+s_mpuspx     ;DP shadow
reg_dbx  =reg_dpx+s_mpudpx     ;DB shadow
addra    =reg_dbx+s_mpudbx     ;address #1
addrb    =addra+s_addr         ;address #2
faca     =addrb+s_addr         ;primary accumulator
facax    =faca+s_pfac          ;extended primary accumulator
facb     =facax+s_pfac         ;secondary accumulator
facc     =facb+s_sfac          ;tertiary accumulator
operand  =facc+s_sfac          ;instruction operand
auxbufix =operand+s_oper       ;auxiliary buffer index
ibufidx  =auxbufix+s_byte      ;input buffer index
bitsdig  =ibufidx+s_byte       ;bits per numeral
numeral  =bitsdig+s_byte       ;numeral buffer
radix    =numeral+s_byte       ;radix index
admodidx =radix+s_byte         ;addressing mode index
charcnt  =admodidx+s_byte      ;character counter
instsize =charcnt+s_word       ;instruction size
mnepck   =instsize+s_word      ;encoded mnemonic
opcode   =mnepck+s_mnepck      ;current opcode
status   =opcode+s_byte        ;I/O status flag
xrtemp   =status+s_byte        ;temp .X storage
eopsize  =xrtemp+s_byte        ;entered operand size
flimflag =eopsize+s_byte       ;forced long immediate...
;
;	xx000000
;	||
;	|+> 0: .X/.Y = 8 bits
;	|            1: .X/.Y = 18 bits
;	+> 0: .A = 8 bits
;	             1: .A = 16 bits
;
;	
;	During assembly, FLIMFLAG indicates the operand size used with an immed-
;	iate mode instruction, thus causing the following disassembly to display
;	the assembled  operand size.   During disassembly,  FLIMFLAG will mirror
;	the effect of the most recent REP or SEP instruction.
;	
;
iopsize  =flimflag+s_byte      ;operand size
range    =iopsize+s_byte       ;allowable radix range
vopsflag =range+s_byte         ;VOPS & ROPS mode bits
;
;
;	copy/fill workspace (overlaps some of the above)...
;
mcftwork =faca                 ;start of copy/fill code
mcftopc  =mcftwork+s_byte      ;instruction opcode
mcftbnk  =mcftopc+s_byte       ;banks
;
;
;	S-record load workspace (overlaps some of the above)...
;
endaddr  =addrb                ;load ending address
loadaddr =facc                 ;current record load address
loadbank =loadaddr+s_word      ;current record load bank
strtaddr =addra                ;load starting address
;
datax    =auxbufix             ;datum holding area
cksumcmp =mnepck               ;computed record checksum
nrecs    =eopsize              ;processed S1 record count
pageoff  =iopsize              ;record load page offset
reclen   =range                ;record length
rectype  =vopsflag             ;record type
;
;
;	end of monitor direct page storage...
;
zeropage .= vopsflag+s_byte
;
	.end