;================================================================================
;
;NCR 53CF94 SCSI CONTROLLER REGISTER DEFINITIONS
;
nr_srsi  =16                   ;total registers
fd_srsi  =16                   ;FIFO depth
;
;
;	register offsets...
;
sr_ctcl  =$00                  ;transfer count LSB (read)
sr_stcl  =sr_ctcl              ;start transfer count LSB (write)
sr_ctch  =$01                  ;transfer count MSB (read)
sr_stch  =sr_ctch              ;start transfer count MSB (write)
sr_fifo  =$02                  ;data FIFO
sr_cmd   =$03                  ;controller command...
;
;	xxxxxxxx
;	||||||||
;	|+++++++> command
;	+> 1: DMA
;
sr_stat  =$04                  ;status (read)...
;
;	xxxxxxxx
;	||||||||
;	|||||+++> SCSI bus status
;	||||+> 1: group code valid
;	|||+> 1: current transfer count = 0
;	||+> 1: SCSI bus parity error
;	|+> 1: illegal operation
;	+> 1: interrupt pending
;
sr_sdid  =sr_stat              ;target's SCSI ID (write)
;
;	xxxxxxxx
;	||||||||
;	|||||+++> target device SCSI ID
;	+++++> reserved
;
sr_irqst =$05                  ;interrupt status (read)...
;
;	xxxxxxxx
;	||||||||
;	|||||||+> 1: selected
;	||||||+> 1: selected w/attention
;	|||||+> 1: reselected
;	||||+> 1: successful operation
;	|||+> 1: service request
;	||+> 1: disconnected
;	|+> 1: invalid command
;	+> 1: bus reset
;
sr_tout  =sr_irqst             ;selection timeout period (write)
sr_isr   =$06                  ;internal state (read)...
;
;	0000xxxx
;	    ||||
;	    |+++> internal state
;	    +> 0: sync offset register at maximum
;
sr_stp   =sr_isr               ;sync transfer period (write)
sr_isrf  =$07                  ;internal/FIFO status (read)...
;
;	xxxxxxxx
;	||||||||
;	|||+++++> FIFO byte count
;	+++> internal state (see SR_ISR bits 0-2)
;
sr_synco =sr_isrf              ;sync offset (write)
sr_cr1   =$08                  ;control register #1...
;
;	xxxxxxxx
;	||||||||
;	|||||+++> 1: controller's SCSI ID
;	||||+> 1: enable self-test
;	|||+> 1: report parity errors
;	||+> 1: enable parity testing
;	|+> 1: no IRQ on bus reset
;	+> 1: enable extended timing
;
sr_clkf  =$09                  ;clock factor (write)...
;
;	00000xxx
;	     |||
;	     +++> 010: 10.00      MHz
;	             011: 10.01 - 15 MHz
;	             100: 15.01 - 20 MHz
;	             101: 20.01 - 25 MHz
;
sr_ftm   =$0a                  ;forced test mode (write)
sr_cr2   =$0b                  ;control register #2...
;
;	xxxxxxxx
;	||||||||
;	|||||||+> 1: pass through/generate data parity
;	||||||+> 1: pass through/generate register parity
;	|||||+> 1: abort on parity error
;	||||+> 1: enable SCSI-2 features
;	|||+> 1: tri-state DREQ & ignore /DACK
;	||+> 1: select byte order
;	|+> 1: latch SCSI phase
;	+> 1: data alignment enable
;
sr_cr3   =$0c                  ;control register #3...
;
;	00000xxx
;	     |||
;	     ||+> burst size
;	     |+> modify DMA mode
;	     +> last byte transfer mode
;
sr_dar   =$0f                  ;data alignment
;
;================================================================================
;
;53CF94 SCSI CONTROLLER CONFIGURATION CONSTANTS
;
sr_clkhz =20000000             ;controller clock frequency (Hz)...
;
;	
;	SR_CLKHZ may range from 10,000,000 to 40,000,000 & must match the
;	frequency of the clock generator installed in the host adapter.
;	
;
;
;	clock conversion factors...
;
         .if sr_clkhz < 10000000 || sr_clkhz > 40000000
           .error "Invalid 53CF94 clock frequency"
         .endif
         .if sr_clkhz < 40000001 && sr_clkhz > 35000000
sr_clkfp =@000
         .endif
         .if sr_clkhz < 35000001 && sr_clkhz > 30000000
sr_clkfp =@111
         .endif
         .if sr_clkhz < 30000001 && sr_clkhz > 25000000
sr_clkfp =@110
         .endif
         .if sr_clkhz < 25000001 && sr_clkhz > 20000000
sr_clkfp =@101
         .endif
         .if sr_clkhz < 20000001 && sr_clkhz > 15000000
sr_clkfp =@100
         .endif
         .if sr_clkhz < 15000001 && sr_clkhz > 10000000
sr_clkfp =@011
         .endif
         .if sr_clkhz < 10000001         
sr_clkfp =@010         
         .endif
k_temp   =sr_clkhz/2000
n_temp   =k_temp*scsiselt
d_temp   =sr_clkfp*8192
sr_toutp =n_temp/d_temp        ;selection timeout factor
sr_cr1p  =@01010000 | scsiid94 ;control register #1...
;
;	01010111
;	 | | |||
;	 | | +++> HBA SCSI ID (determined by SCSIID94)
;	 | +> parity error reporting enabled
;	 +> disable bus reset IRQ
;
sr_cr2p  =@00001100            ;control register #2...
;
;	00000011
;	      ||
;	      |+> abort on parity error
;	      +> enable SCSI-2 features
;
sr_cr3p  =@00000000            ;control register #3
sr_ftmp  =@00000000            ;forced test modes off
;
;================================================================================
;
;53CF94 SCSI CONTROLLER REGISTER & CONTROL MASKS
;
c94isfct =@00011111            ;FIFO count
c94issqc =@00000100            ;command sequence completed
c94isrst =@00000111            ;state bits
;
;
;	interrupt status...
;
c94sel   =@00000001            ;selected
c94sela  =@00000010            ;selected w/ATN
c94resel =@00000100            ;reselected
c94so    =@00001000            ;successful operation
c94sr    =@00010000            ;service request
c94dis   =@00100000            ;disconnected
c94icmd  =@01000000            ;invalid command
c94reset =@10000000            ;bus reset
c94srso  =c94sr | c94so        ;service request/success
;
;
;	general status...
;
c94phasd =@00000110            ;data-in/out phase test
c94phase =@00000111            ;bus phase
c94gcv   =@00001000            ;group code valid
c94ctz   =@00010000            ;transfer count = 0
c94pe    =@00100000            ;bus parity error
c94ioe   =@01000000            ;illegal operation
c94int   =@10000000            ;'C94 interrupting
c94srall =c94int ^ @11111111   ;all bits but IRQ
c94srsts =@01111000            ;status bits only
;
;
;	miscellaneous...
;
c94irqge =@01000000            ;gross error flag bit
c94irqsf =@10000000            ;general purpose flag bit
;
;================================================================================
;
;53CF94 SCSI CONTROLLER COMMANDS
;
_flagdma =@10000000            ;invoke DMA operation
;
sccfifc  =$01                  ;clear FIFO
sccnop   =$00                  ;no operation (PIO)
sccnopd  =sccnop | _flagdma    ;no operation (DMA)
sccrstc  =$02                  ;reset controller
sccrstb  =$03                  ;reset bus
;
;
;	idle state...
;
sccselda =$45                  ;disable selection/reselection
sccselea =$44                  ;enable selection/reselection
sccrsel  =$40                  ;reselect steps
sccsel   =$41                  ;select
sccsela  =$42                  ;select w/ATN
sccselas =$43                  ;select w/ATN & stop
sccsela3 =$46                  ;select w/ATN3
;
;
;	initiator state...
;
sccicc   =$11                  ;command complete steps
sccinfot =$10                  ;information transfer (PIO)
sccinfod =sccinfot | _flagdma  ;information transfer (DMA)
sccmsga  =$12                  ;message accepted
sccatnr  =$1b                  ;reset ATN
sccatns  =$1a                  ;set ATN
scctpb   =$18                  ;transfer pad bytes
;
;
;	target state...
;
scctcc   =$25                  ;command complete steps
sccdisc  =$27                  ;disconnect
sccdiscs =$23                  ;disconnect steps
sccdmaa  =$04                  ;DMA abort
scccmdr  =$29                  ;receive command
scccmdrs =$2b                  ;receive command steps
sccdatar =$2a                  ;receive data
sccmsgrs =$28                  ;receive message steps
sccdatas =$22                  ;send data
sccmsgs  =$20                  ;send message
sccstss  =$21                  ;send status
sccterm  =$24                  ;terminate steps
;
	.end