;================================================================================
;
;MULTIPLE UART COMMON DEFINITIONS
;
n_nxpblk =2                    ;atomic blocks in vQUART
n_nxpcpb =2                    ;atomic channels per block
n_nxpchn  =n_nxpcpb*n_nxpblk   ;channels in vQUART
s_nxpchn  =8                   ;channel spacing in block
s_nxpblk =s_nxpchn*n_nxpcpb    ;size of block
s_nxpchp =s_ptr*n_nxpchn       ;channel pointer space
s_nxpbkp =s_ptr*n_nxpblk       ;block pointer space
;
;================================================================================
;
;PHILLIPS/NXP 26C92/28L92 DUART REGISTERS
;
;	Abbreviations used in comments are as follows:
;
;	    -------------
;	    C/T  Counter/timer
;	    IP   Input port
;	    LSB  Least significant byte
;	    MSB  Most significant byte
;	    OP   Output port
;	    (R)  Read only
;	    (RW) Read/write
;	    RxD  Receiver
;	    TxD  Transmitter
;	    (W)  Write only
;	    -------------
;
;
;	DUART channel register offsets...
;
nx_mr    =0                    ;mode select
nx_sr    =1                    ;status (R)
nx_csr   =1                    ;clock select (W)
nx_cr    =2                    ;command (W)
nx_rhr   =3                    ;RxD FIFO (R)
nx_thr   =3                    ;TxD FIFO (W)
nx_fif   =3                    ;communication FIFO (RW)
;
;
;	DUART block register offsets...
;
nx_ipcr  =4                    ;IP change (R)
nx_acr   =4                    ;auxiliary control (W)
nx_isr   =5                    ;interrupt status (R)
nx_imr   =5                    ;interrupt enable (W)
nx_ctu   =6                    ;C/T MSB value (R)
nx_ctl   =7                    ;C/T LSB value (R)
nx_crur  =6                    ;C/T MSB preset (W)
nx_crlr  =7                    ;C/T LSB preset (W)
nx_ureg  =12                   ;user-defined (RW)
nx_ipsr  =13                   ;IP status (R)
nx_opcr  =13                   ;OP configuration (W)
nx_sct   =14                   ;start C/T (R)
nx_sopb  =14                   ;set OP bits (W)
nx_rct   =15                   ;stop C/T (R)
nx_ropb  =15                   ;reset OP bits (W)
;
;================================================================================
;
;vQUART REGISTER OFFSETS
;
;
;	channel A...
;
nx_mra   =nx_mr                ;mode select
nx_sra   =nx_sr                ;status (R)
nx_csra  =nx_csr               ;clock select (W)
nx_cra   =nx_cr                ;command (W)
nx_rhra  =nx_rhr               ;RxD FIFO (R)
nx_thra  =nx_thr               ;TxD FIFO (W)
nx_fifa  =nx_fif               ;communication FIFO (RW)
;
;
;	channel B...
;
nx_mrb   =nx_mra+s_nxpchn      ;mode select
nx_srb   =nx_sra+s_nxpchn      ;status (R)
nx_csrb  =nx_csra+s_nxpchn     ;clock select (W)
nx_crb   =nx_cra+s_nxpchn      ;command (W)
nx_rhrb  =nx_rhra+s_nxpchn     ;RxD FIFO (R)
nx_thrb  =nx_thra+s_nxpchn     ;TxD FIFO (W)
nx_fifb  =nx_fifa+s_nxpchn     ;communication FIFO (RW)
;
;
;	channel C...
;
nx_mrc   =nx_mra+s_iodev       ;mode select
nx_src   =nx_sra+s_iodev       ;status (R)
nx_csrc  =nx_csra+s_iodev      ;clock select (W)
nx_crc   =nx_cra+s_iodev       ;command (W)
nx_rhrc  =nx_rhra+s_iodev      ;RxD FIFO (R)
nx_thrc  =nx_thra+s_iodev      ;TxD FIFO (W)
nx_fifc  =nx_fifa+s_iodev      ;communication FIFO (RW)
;
;
;	channel D...
;
nx_mrd   =nx_mrc+s_nxpchn      ;mode select
nx_srd   =nx_src+s_nxpchn      ;status (R)
nx_csrd  =nx_csrc+s_nxpchn     ;clock select (W)
nx_crd   =nx_crc+s_nxpchn      ;command (W)
nx_rhrd  =nx_rhrc+s_nxpchn     ;RxD FIFO (R)
nx_thrd  =nx_thrc+s_nxpchn     ;TxD FIFO (W)
nx_fifd  =nx_fifc+s_nxpchn     ;communication FIFO (RW)
;
;
;	block A...
;
nx_ipcra =nx_ipcr              ;IP change (R)
nx_acra  =nx_acr               ;auxiliary control (W)
nx_isra  =nx_isr               ;interrupt status (R)
nx_imra  =nx_imr               ;interrupt enable (W)
nx_ctua  =nx_ctu               ;C/T MSB value (R)
nx_ctla  =nx_ctl               ;C/T LSB value (R)
nx_crura =nx_crur              ;C/T MSB preset (W)
nx_crlra =nx_crlr              ;C/T LSB preset (W)
nx_urega =nx_ureg              ;user-defined (RW)
nx_ipsra =nx_ipsr              ;IP status (R)
nx_opcra =nx_opcr              ;OP configuration (W)
nx_scta  =nx_sct               ;start C/T (R)
nx_sopba =nx_sopb              ;set OP bits (W)
nx_rcta  =nx_rct               ;stop C/T (R)
nx_ropba =nx_ropb              ;reset OP bits (W)
;
;
;	block B...
;
nx_ipcrb =nx_ipcr+s_iodev      ;IP change (R)
nx_acrb  =nx_acr+s_iodev       ;auxiliary control (W)
nx_isrb  =nx_isr+s_iodev       ;interrupt status (R)
nx_imrb  =nx_imr+s_iodev       ;interrupt enable (W)
nx_ctub  =nx_ctu+s_iodev       ;C/T MSB value (R)
nx_ctlb  =nx_ctl+s_iodev       ;C/T LSB value (R)
nx_crurb =nx_crur+s_iodev      ;C/T MSB preset (W)
nx_crlrb =nx_crlr+s_iodev      ;C/T LSB preset (W)
nx_uregb =nx_ureg+s_iodev      ;user-defined (RW)
nx_ipsrb =nx_ipsr+s_iodev      ;IP status (R)
nx_opcrb =nx_opcr+s_iodev      ;OP configuration (W)
nx_sctb  =nx_sct+s_iodev       ;start C/T (R)
nx_sopbb =nx_sopb+s_iodev      ;set OP bits (W)
nx_rctb  =nx_rct+s_iodev       ;stop C/T (R)
nx_ropbb =nx_ropb+s_iodev      ;reset OP bits (W)
;
;================================================================================
;
;PHILIPS/NXP 28L92 DUART CONFIGURATION & CONTROL CONSTANTS
;
nxx1freq =3686400              ;X1 clock frequency in Hz
nxctscal =nxx1freq/2           ;C/T scaled clock
;
;
;	ACR  auxiliary control register...
;
nxparbrt =@01100000            ;...
;
;	a) Use baud rate table #1  see data sheet.
;	b) C/T acts as a timer.
;	c) C/T clock source is scaled to X1  1.
;
;
;	CR  command register...
;
nxpcrrxe =@00000001            ;enable receiver
nxpcrrxd =@00000010            ;disable receiver
nxpcrtxe =@00000100            ;enable transmitter
nxpcrtxd =@00001000            ;disable transmitter
nxpcrmr1 =@00010000            ;select MR1
nxpcrrxr =@00100000            ;reset receiver
nxpcrtxr =@00110000            ;reset transmitter
nxpcresr =@01000000            ;reset error status
nxpcrbir =@01010000            ;reset received break change IRQ
nxpcrbks =@01100000            ;start break
nxpcrbke =@01110000            ;stop break
nxpcrrsa =@10000000            ;assert request to send
nxpcrrsd =@10010000            ;deassert request to send
nxpcrtme =@10100000            ;select C/T timer mode
nxpcrmr0 =@10110000            ;select MR0
nxpcrtmd =@11000000            ;select C/T counter mode
;
;
;	CSR  clock select register...
;
nxpcsdef =@01100110            ;RxD & TxD at 115.2 Kbps...
;
;	
;	The above data rate is based upon the value of NXPARBRT (above).
;	
;
;
;	CT  counter/timer...
;
nxpctdlo =<[nxctscal/hz]       ;underflows/sec LSB
nxpctdhi =>[nxctscal/hz]       ;underflows/sec MSB
;
;
;	IMR  interrupt mask register...
;
nxpairq  =nxpatirq|nxparirq    ;enable ch A RxD & TxD IRQs
nxpbirq  =nxpbtirq|nxpbrirq    ;enable ch B RxD & TxD IRQs
nxpiqmsk =nxpairq|nxpbirq|nxpctirq ;set IRQ sources...
;
;
;	MR0  mode 0 register...
;
nxpm0def =@11001100            ;...
;
;	11001100
;	||||||||
;	|||||+++> extended baud rate #2
;	||||+> 16-deep FIFO
;	||++> TxD interrupts only when FIFO is empty
;	|+> RxD interrupts only when FIFO is full (see also MR1:6)
;	+> enable RxD watchdog timer
;
;
;	MR1  mode 1 register...
;
nxpm1def =@11010011            ;...
;
;	11010011
;	||||||||
;	||||||++> 8 bit char size
;	|||||+> parity type (ignored)
;	|||++> no parity generated or checked
;	||+> character error mode
;	|+> RxD interrupts only when FIFO is full (see also MR0:6)
;	+> RxD controls RTS
;
;
;	MR2  mode 2 register...
;
nxpm2def =@00010111            ;normal mode, auto RTS
;
;
;	OPCR  output port configuration register...
;
nxpopdef =@11110000            ;OP4-7 as IRQ outputs
;
;
;	combined setup commands...
;
nxpcrrtd =nxpcrtxd|nxpcrrxd    ;disable transmitter & receiver
nxpcrrte =nxpcrtxe|nxpcrrxe    ;enable transmitter & receiver
;
;
;	channel status analysis masks...
;
nxprxdr  =@00000001            ;RxD: FIFO not empty
nxprxdf  =@00000010            ;RxD: FIFO full
nxptxdr  =@00000100            ;TxD: FIFO not full
nxptxdur =@00001000            ;TxD: FIFO empty
nxprxdor =@00010000            ;RxD: FIFO overrun
nxpparer =@00100000            ;RxD: parity error
nxpfrmer =@01000000            ;RxD: framing error
nxpbreak =@10000000            ;RxD: break
;
;
;	interrupt status analysis masks...
;
nxpatirq =@00000001            ;ch A: TxD
nxparirq =@00000010            ;ch A: RxD...
nxpbtirq =@00010000            ;ch B: TxD
nxpbrirq =@00100000            ;ch B: RxD
;
;	
;	The RxD & TxD masks are of historical interest.  Determination of which
;	source is interrupting is accomplished by reading & processing the
;	vQUART IRQ status byte (see below).
;	
;	
nxpabirq =@00000100            ;ch A: break change
nxpbbirq =@01000000            ;ch B: break change
nxpctirq =@00001000            ;C/T: underflow
nxpipirq =@10000000            ;input port change
;
;
;	vQUART IRQ status...
;
m_irqrx  =@00001111            ;RxD IRQ mask
m_irqtx  =@11110000            ;TxD IRQ mask
;
	.end