;================================================================================
;
;SCSI SUBSYSTEM ATOMIC CONSTANTS
;
;
;	"number of" constants...
;
n_grpshf =5                    ;opcode group field shifts
n_lbabg0 =21                   ;bits in group 0 LBA
n_lbabg2 =32                   ;bits in group 2 LBA
n_lunshf =5                    ;LUN field shifts
n_phases =8                    ;SCSI bus phases
n_scsiid =8                    ;recognized SCSI IDs
n_scsilu =8                    ;recognized SCSI LUNs
;
;
;	"size of" constants...
;
s_cdbg0  =6                    ;group 0 CDB
s_cdbg1  =10                   ;group 1 CDB
s_cdbg2  =10                   ;group 2 CDB
s_cdbg4  =16                   ;group 4 CDB
s_cdbg5  =12                   ;group 5 CDB
s_mesg   =1                    ;message
s_mesgws =8                    ;message workspace
s_sspid  =16                   ;product ID field
s_reqsen =18                   ;request sense data
s_revsn  =4                    ;product revision field
s_sernm  =8                    ;product serial number field
s_stat   =1                    ;status
s_vendr  =8                    ;vendor ID field
s_inqry  =s_vendr+s_sspid+s_revsn+s_sernm
s_miswrk =s_inqry              ;enumeration & inquiry workspace
;
;
;	SCSI bus phases...
;
ssphasdo =@000                 ;data out
ssphasdi =@001                 ;data in
ssphascm =@010                 ;command
ssphasst =@011                 ;status
ssphasra =@100                 ;reserved
ssphasrb =@101                 ;reserved
ssphasmo =@110                 ;message out
ssphasmi =@111                 ;message in
;
;
;	bus control constants...
;
scsiid94 =7                    ;host adapter SCSI ID
scsirdly =5                    ;bus reset recovery time (sec)
scsiselt =250                  ;selection timeout (msec)
;
;
;	miscellaneous...
;
lbamsw32 =@1000000000000000000000 >> s_bword ;...
;
;	
;	LBA32MSW is used during SCSI enumeration to determine if a SCSI device
;	recognizes 32 bit block addressing.  If the MSW of the device's capac-
;	ity as returned by the "read capacity" command is greater than 21 bits
;	then the device will be flagged as 32 bit LBA capable.
;	
;
lun      =0                    ;default logical unit
scsitrct =10                   ;enumeration test ready attempts
scsitrtd =2                    ;test ready attempt spacing (secs)
;
;================================================================================
;
;SCSI ENUMERATION DISPLAY CONSTANTS
;
colid    =2                    ;ID column
colvendr =6                    ;vendor column
colprod  =16                   ;product column
colvers  =34                   ;version column
colstat  =41                   ;status column
rowtitle =9                    ;column titles row
rowenum  =rowtitle+2           ;enumeration starting row
seghllen =48                   ;enumeration horiz line length
;
;================================================================================
;
;SCSI SUBSYSTEM MASKS
;
grpmask  =@11100000            ;command group
haidmask =@00000111            ;HBA bus ID
iqsynctr =@00010000            ;synchronous transfer flag bit
iqvendr  =@00000111            ;ANSI version mask
lbafmask =@10000000            ;32 bit LBA flag
lunmask  =@11100000            ;LUN field mask
lunmaskr =lunmask ^ @11111111  ;LBA MSB field mask
scbdimm  =@00000001            ;immediate return bit
sdtmask  =@00011111            ;device type flag mask
sg2mask  =2 << s_bnybbl        ;group 2 field mask
;
;================================================================================
;
;SCSI DEVICE COMMAND CODES
;
sc_tur   =$00                  ;test unit ready
sc_recal =$01                  ;recalibrate/rewind...
;
;	
;	SC_RECAL is obsoleted for block devices, & may cause
;	a "check condition" status if issued to one.
;	
;
sc_getba =$02                  ;get LBA...
;
;	
;	SC_GETBA is obsoleted for block devices, & may cause
;	a "check condition" status if issued to one.
;	
;
sc_getsd =$03                  ;get sense data
sc_fmt   =$04                  ;format unit
sc_getbl =$05                  ;get block limits
sc_read  =$08                  ;read from medium
sc_writ  =$0a                  ;write to medium
sc_seek  =$0b                  ;seek to block (block device)
sc_seekb =$0c                  ;seek to block (stream device)
sc_wrtfm =$10                  ;write filemarks
sc_space =$11                  ;space on medium
sc_inqry =$12                  ;get device parameters
sc_mode  =$15                  ;mode select
sc_resrv =$16                  ;reserve device
sc_reles =$17                  ;release device
;
;	
;	SC_RELES & SC_RESRV are obsoleted but retained
;	for compatibility with older devices.
;	
;
sc_copy  =$18                  ;copy data
sc_erase =$19                  ;erase data
sc_modes =$1a                  ;mode sense
sc_ssdev =$1b                  ;start/stop or load/unload medium
sc_lock  =$1e                  ;lock/unlock medium
sc_getdc =$25                  ;get device capacity
sc_readx =$28                  ;read from medium extended
sc_writx =$2a                  ;write to medium extended
sc_seekx =$2b                  ;seek to block extended
sc_locat =$2b                  ;set position in medium
sc_getp  =$34                  ;get logical position
;
;================================================================================
;
;LOGGED SCSI DEVICES TABLE
;
;                  
;  slot offset > $00  $01  $02  $03  $04  $05  $06  $07
;                  
;                   |    |    ^^    ^^
;                   |    |        capacity         block
;                   |    |                         size
;                   |    +> type
;                   +> flag
;
;
;	field sizes (description order)...
;
s_sdbsiz =s_word               ;block size
s_sdflag =s_byte               ;flag
s_sdcap  =s_dword              ;capacity in blocks
s_sdtype =s_byte               ;type
;
;
;	slot & table sizes...
;
sdslotex =ex2_x3               ;slot size computation factor...
;
;	
;	SDSLOTEX is used to atomically define the size of the device table slot
;	so bit shifts can be used to compute a slot address for a SCSI ID.  The
;	above definition should be changed with caution.  See additional notes
;	in the SSLOTADR function.
;	
;
s_sdslot =1 << sdslotex        ;slot
s_sd_tab =s_sdslot*n_scsiid    ;table
;
;
;	field offsets...
;
o_sdflag =0                    ;device flag...
;
;	xx000xxx
;	||   |||
;	||   +++> ANSI version
;	|+> 1: supports sync transfer
;	+> 1: device enumerated
;
o_sdtype =o_sdflag+s_sdflag    ;device type...
;
;	x00xxxxx
;	|  |||||
;	|  +++++> ANSI device type
;	+> 1: device recognizes 32 bit LBAs
;
o_sdcap  =o_sdtype+s_sdtype    ;capacity in blocks
o_sdbsiz =o_sdcap+s_sdcap      ;block size
;
;================================================================================
;
;SCSI MESSAGES (description order)
;
sm_abort =$06                  ;abort transaction
sm_cmdcp =$00                  ;command completed
sm_disc  =$04                  ;disconnect pending
sm_mesgx =$01                  ;extended (2 byte) message
sm_reset =$0c                  ;forced reset
sm_mesgr =$07                  ;message rejected
sm_nop   =$08                  ;no operation
sm_dptrr =$03                  ;restore data pointers
sm_dptrs =$02                  ;save data pointers
sm_sreq  =$01                  ;synchronous transfer request
;
;================================================================================
;
;SCSI STATUS CODES (description order)
;
ss_check =@00000010            ;check condition
ss_ccomp =@00000000            ;command completed
ss_cterm =@00100010            ;command terminated
ss_cmet  =@00000100            ;condition met/good
ss_goodi =@00010000            ;intermediate good
ss_resrv =@00011000            ;reservation conflict
ss_busy  =@00001000            ;target busy
;
;================================================================================
;
;SCSI COMMAND DESCRIPTOR BLOCK OFFSETS
;
so_opcod =0                    ;SCSI operation code...
;
;	xxxxxxxx
;	||||||||
;	|||+++++> command
;	+++> group
;
so_lunla =1                    ;LUN/LBA MSB...
;
;	xxxxxxxx
;	||||||||
;	|||+++++> LBA bits 16-21 (group 0 only)
;	+++> LUN
;
;
;	group 0 block device offsets...
;
so_lbam  =2                    ;LBA bits 8-15
so_lbal  =3                    ;LBA bits 0-7
so_xfrl  =4                    ;transfer length
;
;
;	group 0 stream device offsets...
;
so_xfrmb =2                    ;transfer length bits 16-23
so_xfrib =3                    ;transfer length bits 8-15
so_xfrlb =4                    ;transfer length bits 0-7
;
;
;	group 0 common offsets...
;
so_ctrl  =5                    ;control byte
;
;
;	groups 1, 2 & 5 common offsets...
;
so_lbamb =2                    ;LBA bits 24-31
so_lbamu =3                    ;LBA bits 16-23
so_lbaml =4                    ;LBA bits 8-15
so_lball =5                    ;LBA bits 0-7
;
;
;	group 1 & 2 offsets...
;
so_xfrlh =7                    ;transfer length MSB
so_xfrll =8                    ;transfer length LSB
so_ctrll =9                    ;control byte
;
;
;	group 5 offsets...
;
so_xfrxh =8                    ;transfer length MSB
so_xfrxl =9                    ;transfer length LSB
so_ctrxl =11                   ;control byte
;
;================================================================================
;
;SCSI MISCELLANEOUS OFFSETS
;
;
;	read capacity...
;
rc_lbahi =0                    ;LBA MSW
rc_lbalo =2                    ;LBA LSW
rc_bszhi =4                    ;block size MSW
rc_bszlo =6                    ;block size LSW
;
;
;	basic inquiry...
;
iq_pdt   =0                    ;ANSI device type
iq_ansiv =2                    ;ANSI version
iq_flags =7                    ;flags
iq_vendr =8                    ;vendor ID
iq_pid   =iq_vendr+s_vendr     ;product ID
iq_revsn =iq_pid+s_sspid       ;product revision
;
;
;	serial number inquiry...
;
iq_page  =1                    ;page code ($80)
iq_pglen =3                    ;page length
iq_sernm =4                    ;product serial number
;
;================================================================================
;
;SCSI DEVICE TYPES
;
sd_da    =$00                  ;direct access
sd_seq   =$01                  ;sequential access
sd_prtr  =$02                  ;printer
sd_proc  =$03                  ;processor
sd_worm  =$04                  ;WORM
sd_cdrom =$05                  ;CD/DVD ROM
sd_scanr =$06                  ;scanner
sd_optcl =$07                  ;optical storage
sd_juke  =$08                  ;medium changer
sd_comm  =$09                  ;miscellaneous communications
sd_sarry =$0c                  ;storage array
sd_das   =$0e                  ;simplified direct access
sd_ocrw  =$0f                  ;optical card reader/writer
;
;================================================================================
;
;SCSI VITAL PRODUCT DATA PAGE CODES
;
vpdpages =$00                  ;supported VPD pages
vpdserno =$80                  ;device serial number
vpddevid =$83                  ;device identification
;
	.end