;===============================================================================
;
;MASS STORAGE CONSTANTS & DEFINITIONS
;
s_fsflag =s_byte               ;filesystem flag size
s_fsmag  =s_dword              ;filesystem magic number size
s_fsnam  =8                    ;filesystem name padded size
s_islmag =s_dword              ;ISL magic number size
s_lba    =s_dword              ;logical block address size
s_pblks  =s_dword              ;block count field size
s_volnam =8                    ;volume name size
islmagic =$c3c4d4A2            ;ISL magic number...
;
;	
;	ISLMAGIC must be in the boot block of each disk on which 1 or more file-
;	systems have been defined.  See the narratives for more detail.
;	
;
;
;	default device block sizes...
;
s_cdblk  =2048                 ;CDA
s_hdblk  =512                  ;HDA
s_tdblk  =512                  ;TDA
s_cdblkb =s_cdblk*s_bbyte      ;CDA block size in bits
s_hdblkb =s_hdblk*s_bbyte      ;HDA block size in bits
s_tdblkb =s_tdblk*s_bbyte      ;TDA block size in bits
;
;
;	disk layout...
;
;	          FIELD          LOGICAL BLOCK ADDRESS
;	+=======================+
;	|    1st filesystem     |
;	+=======================+ $00000100 (pb_fs001)
;	| reserved (222 blocks) |
;	++ $00000022 (pb_rsrvd)
;	|     stage 2 boot      |
;	++ $00000002 (pb_stag2)
;	|     master  boot      |
;	+=======================+ $00000000 (pb_mboot)
;
pb_mboot =$00000000            ;master boot origin LBA
pb_stag2 =$00000002            ;stage 2 boot origin LBA
pb_rsrvd =$00000022            ;reserved blocks origin LBA
pb_fs001 =$00000100            ;1st filesystem origin LBA
m_fsdisk =4                    ;max filesystems per disk
n_mboot  =pb_stag2 -pb_mboot   ;master boot blocks
n_stag2  =pb_rsrvd -pb_stag2   ;stage 2 boot blocks
n_rsrvd  =pb_fs001 -pb_rsrvd   ;reserved blocks
s_mboot  =n_mboot  *s_hdblk    ;master boot bytes
s_s2boot =n_stag2  *s_hdblk    ;stage 2 boot bytes
s_rsrvd  =n_rsrvd  *s_hdblk    ;reserved bytes
;
;
;	boot block layout...
;
;	               FIELD               Offset in Block
;	+================================+ 
;	|  master boot code (914 bytes)  |
;	++ $006E (mb_code)
;	|  filesystem table  (90 bytes)  |
;	++ $0014 (mb_fstab)
;	|          volume  name          |
;	++ $000C (mb_magic)
;	|  master boot record timestamp  |
;	++ $0006 (mb_utime)
;	|    boot block magic number     |
;	++ $0002 (mb_magic)
;	| master boot code start address |
;	+================================+ $0000 (mb_jaddr)
;
;
;	filesystem table slot structure...
;
ft_flag =0                    ;status flag...
;
;	xx000000
;	||
;	|+> 0: filesystem is read only
;	|            1: filesystem is read/write
;	+> 0: slot is unused
;	             1: slot points to active filesystem
;
ft_nused =ft_flag  +s_fsflag   ;unused field
ft_magic =ft_nused +s_byte     ;magic number
ft_fsnam =ft_magic +s_fsmag    ;name
ft_sblba =ft_fsnam +s_fsnam    ;starting LBA
ft_pblks =ft_sblba +s_lba      ;total physical blocks
;
s_fsslot =ft_pblks +s_pblks    ;table slot size
;
;
;	filesystem table structure...
;
ft_nfs   =0                    ;number of active entries
ft_slotx =ft_nfs   +s_byte     ;lowest unused slot index
ft_slota =ft_slotx +s_byte     ;slot array
ft_slotp =s_fsslot *m_fsdisk   ;slot array size
s_fstab  =ft_slota +ft_slotp   ;table size
;
;
;	boot block structure...
;
mb_jaddr =0                    ;MB code JuMP address
mb_magic =mb_jaddr +s_ptr      ;MBR magic number
mb_utime =mb_magic +s_islmag   ;MBR timestamp
mb_volum =mb_utime +s_time_t   ;volume name
mb_fstab =mb_volum +s_volnam   ;filesystem table
mb_code  =mb_fstab +s_fstab    ;master boot code
s_mbcode =s_mboot  -mb_code    ;master boot code space
;
	.end
