;================================================================================
;
;POC V1 BIOS API INDICES
;
;	
;	This file lists the API index numbers for all user-accessible BIOS func-
;	tions.  It must be INCLUDEd in any program that makes BIOS API calls.
;	DO NOT hardcode these values into any application.
;
;	BIOS API functions are called as follows:
;
;	         COP API_NUM           ;call BIOS function
;	         BCS EXCEP             ;handle processing exception
;
;	If the macro TRAP has been defined it may be substituted for COP.  For
;	example, to write a character to TIA-232 channel A the following code
;	would be used:
;
;	         SEP #%00100000        ;8 bit accumulator
;	         LDA #CHAR             ;load character to write
;	         TRAP GETCHA           ;call function
;	         BCS ERROR
;
;	If the API index is out of range, carry will be set upon return & the 8
;	bit accumulator will be loaded with $FF.  Such an error will not occur
;	if this table is used to define API index numbers.
;
;	WARNING: DO NOT EDIT THESE DEFINITIONS!  Do not hard-code any API index
;	numbers, as they are subject to change in future BIOS revisions.
;	
;
hrst     =0                    ;restart system (soft reset)
mon      =1                    ;enter machine language monitor
getcha   =2                    ;read from TIA-232 channel A
getchb   =3                    ;read from TIA-232 channel B
putcha   =4                    ;write to TIA-232 channel A
putchb   =5                    ;write to TIA-232 channel B
chanbctl =6                    ;disable/enable TIA-232 channel B
getdtr   =7                    ;read date & time from RTC
getnvr   =8                    ;read data from NVRAM
getuxtim =9                    ;get UNIX time
getsutim =10                   ;get system up time
plot     =11                   ;position console cursor
putdtr   =12                   ;write date & time to RTC
putnvr   =13                   ;write data to NVRAM
setuxtim =14                   ;set UNIX time
utdelay  =15                   ;generate time delay
scsicmd  =16                   ;execute SCSI command
scsihaid =17                   ;get SCSI host adapter bus ID
scsiparm =18                   ;get SCSI device parameters
scsiinit =19                   ;initialize SCSI subsystem
;
;================================================================================
;
;DEFAULT DIRECT PAGE BASE ADDRESS
;
;	
;	WARNING: DO NOT EDIT THIS DEFINITION!
;	
;
dzpage   =$80
;
;================================================================================
;
;DEFAULT HARDWARE STACK LOCATION
;
;	
;	Applications wishing to reset the stack pointer to the  default
;	should use this definition.  Future architectural revisions may
;	result in this location being moved.   The top of the stack may
;	also be changed when an operating system is loaded.  User-writ-
;	ten applications  should  not set the stack pointer higher than
;	this definition, as doing so will overwrite the TIA-232 buffers
;	or I/O hardware.
;
;	WARNING: DO NOT EDIT THIS DEFINITION!
;	
;
dstkptr  =$00cbff
;
;================================================================================
;
;DEFAULT APPLICATION LOAD ADDRESS
;
;	
;	WARNING: DO NOT EDIT THIS DEFINITION!
;	
;
dloadadr =$000400
;
	.end
