;===============================================================================
;
;	     816NIX FILESYSTEM LAYOUT       LBO (note 4)
;	+=================================+ ====================
;	|         data block array        |
;	++ note 3    (fs_dba)
;	|           inode array           |
;	++ note 2    (fs_inode)
;	|    block allocation map (BAM)   |
;	++ note 1    (fs_bam)
;	|    inode allocation map (IAM)   |
;	++ $00000002 (fs_iam)
;	|   BAM tag bitmap logical block  |
;	++ $00000001 (fs_tag)
;	|           superblock            |
;	+=================================+ $00000000 (fs_super)
;	 1) Determined by size of IAM.
;	 2) Determined by size of BAM.
;	 3) Determined by size of inode array.
;	 4) Logical block offset in filesystem.
;
;
;	         SUPERBLOCK LAYOUT          Byte Offset in Block
;	+=================================+ ====================
;	|    IAM tag bitmap (256 bytes)   |
;	++ $0100 (sb_iamta)
;	|    reserved space (166 bytes)   |
;	++ $005A (sb_rsrvd)
;	|      data block origin LBA      |
;	++ $0056 (sb_dbsa)
;	|      inode array origin LBA     |
;	++ $0052 (sb_iasa)
;	|          BAM origin LBA         |
;	++ $004E (sb_bamsa)
;	|          IAM origin LBA         |
;	++ $004A (sb_iamsa)
;	|        BAM tag bitmap LBA       |
;	++ $0046 (sb_bamta)
;	|           free inodes           |
;	++ $0042 (sb_dabf)
;	|           total inodes          |
;	++ $003E (sb_inodf)
;	|         free data blocks        |
;	++ $003A (sb_inod)
;	|        total data blocks        |
;	++ $0036 (sb_dablk)
;	|        total inode blocks       |
;	++ $0032 (sb_inblk)
;	|         total BAM blocks        |
;	++ $002E (sb_dmblk)
;	|         total IAM blocks        |
;	++ $002A (sb_imblk)
;	|   logical blocks in filesystem  |
;	++ $0026 (sb_lblks)
;	|  physical blocks in filesystem  |
;	++ $0022 (sb_pblks)
;	|     filesystem sentinel LBA     |
;	++ $001E (sb_fssen)
;	|      filesystem origin LBA      |
;	++ $001A (sb_orig)
;	|     last modification time      |
;	++ $0014 (sb_mtime)
;	|    filesystem creation time     |
;	++ $000E (sb_ktime)
;	|          magic number           |
;	++ $000A (sb_magic)
;	|        filesystem name          |
;	++ $0002 (sb_fsnam)
;	|          status flag            |
;	++ $0001 (sb_stat)
;	|           lock flag             |
;	+=================================+ $0000 (sb_lock)
;
;
;	         DIRECTORY LAYOUT           Byte Offset in Directory
;	+=================================+ ========================
;	| INUM | ...etc...                |
;	++ s_dsslot*5
;	| INUM | <filename3>              | ...etc...
;	++ s_dsslot*4
;	| INUM | <filename2>              |
;	++ s_dsslot*3
;	| INUM | <filename1>              |
;	++ s_dsslot*2
;	| INUM | .. (parent)              |
;	++ s_dsslot
;	| INUM | .  (CWD)                 |
;	+=================================+ $00000000
;	  CWD: current working directory
;	 INUM: 32 bit inode number
;
;
;	
;	Filesystem Atomic Definitions
;	
;
mag816nx =$fd187e21            ;filesystem magic number
n_ldblk  =2                    ;physical blocks in a logical block
n_supblk =1                    ;logical blocks in superblock...
;
;	
;	Although the superblock is defined to be the size of a logical block all
;	of its content is confined to the 1st 512 bytes.  Hence the core copy is
;	contained in a single physical disk block.  The remainder of the logical
;	block is dead space.
;	
;
s_filnam =60                   ;maximum size of a filename
s_inum   =s_dword              ;size of an inode number...
;
;	
;	The sum of S_FILNAM & S_INUM should be 2^n.  Hence the filename size 
;	should be 28, 60, 124 or 252.  Other values could be used but would re-
;	sult in wasted space, since the number of directory slots that will fit
;	into a logical block will not fully utilize the space in the block.
;	As the filename size is increased fewer directory slots will fit in a
;	logical block, causing an increase in filesystem overhead & directory
;	search times due to increased disk activity.
;
;	DO NOT change the definition of S_INUM!
;	
;
s_ldblk  =s_hdblk  *n_ldblk    ;bytes in a logical block
s_ldblkb =s_hdblkb *n_ldblk    ;bits in a logical block
s_lboadr =s_dword              ;bytes in a logical block offset
;
;	
;	File Modes & Permissions
;	
;
fp_oexec =1     << 0           ;other execute
fp_owrit =1     << 1           ;other write
fp_oread =1     << 2           ;other read
fp_gexec =1     << 3           ;group execute
fp_gwrit =1     << 4           ;group write
fp_gread =1     << 5           ;group read
fp_uexec =1     << 6           ;owner execute
fp_uwrit =1     << 7           ;owner write
fp_uread =1     << 8           ;owner read
fp_stiky =1     << 9           ;sticky
fp_sgid  =1     << 10          ;set GID on execution
fp_suid  =1     << 11          ;set UID on execution
ft_notaf =@0000 << 12          ;not a file  invalid
ft_fifo  =@0001 << 12          ;named pipe
ft_specc =@0010 << 12          ;character special
ft_dir   =@0100 << 12          ;directory
ft_specb =@0110 << 12          ;block special
ft_user  =@1000 << 12          ;regular
ft_symlk =@1010 << 12          ;symbolic link
ft_sockt =@1100 << 12          ;socket
ft_pipe  =@1111 << 12          ;anonymous pipe
;
;	
;	Inode Structure Definitions
;	
;
n_tocadr =24                   ;LBAs in table of contents
n_tocind =3                    ;indirect LBAs in TOC
n_dirblk =n_tocadr -n_tocind   ;direct LBAs in TOC
s_filmod =s_word               ;mode & permissions
s_filsiz =s_dword              ;bytes in file
s_lnkcnt =s_word               ;link count
s_toc    =s_lba    *n_tocadr   ;size of TOC
;
;	
;	Data Block Addressing Rules
;	
;
i_indbk1 =n_dirblk             ;TOC single indirection slots
i_indbk2 =i_indbk1 +1          ;TOC double indirection slots
i_indbk3 =i_indbk2 +1          ;TOC triple indirection slots
n_idbadr =s_ldblk  /s_lba      ;LBAs per indirect block
n_iadrl1 =n_idbadr             ;total 1st indirect level LBAs
n_iadrl2 =n_idbadr *n_iadrl1   ;total 2nd indirect level LBAs
n_iadrl3 =n_idbadr *n_iadrl2   ;total 3rd indirect level LBAs
th_lbosi =n_dirblk             ;single indirection threshold
th_lbodi =th_lbosi +n_iadrl1   ;double indirection threshold
th_lboti =th_lbodi +n_iadrl2   ;triple indirection threshold
th_ovrfl =th_lboti +n_iadrl3   ;out-of-range offset sentinel
;
;	
;	On-Disk Inode Structure
;	
;
di_mode  =0                    ;file type & mode
di_nlink =di_mode  +s_filmod   ;link count
di_uid   =di_nlink +s_lnkcnt   ;user ID
di_gid   =di_uid   +s_uid      ;group ID
di_size  =di_gid   +s_gid      ;file size in bytes
di_toc   =di_size  +s_filsiz   ;table of contents
di_atime =di_toc   +s_toc      ;atime: file last accessed
di_mtime =di_atime +s_time_t   ;mtime: file last modified
di_ctime =di_mtime +s_time_t   ;ctime: inode last modified
;
di_devsf =di_toc               ;device special fields
di_dmin  =di_devsf             ;minor device number
di_dmaj  =di_dmin  +s_byte     ;major device number
;
;	
;	In-Core Inode Structure
;	
;
ci_inum  =0                    ;inode number
ci_sdid  =ci_inum  +s_inum     ;device ID
ci_iflag =ci_sdid  +s_fsflag   ;control & status flag...
;
;	xxxxx00x
;	|||||  |
;	|||||  +> 1: inode is a mount point
;	||||+> 1: process waiting for access
;	|||+> 1: perform synchronous write
;	||+> 1: update CTIME
;	|+> 1: update MTIME
;	+> 1: update ATIME
;
ci_lflag =ci_iflag +s_fsflag   ;lock flag...
;
;	+> bit 15
;	|
;	V
;	xxxxxxxxxxxxxxxx
;	||||||||||||||||
;	|+++++++++++++++> controlling PID
;	+> 1: inode locked
;
ci_opnct =ci_lflag +s_word     ;number of references
ci_llba  =ci_opnct +s_word     ;last LBA accessed
ci_offst =ci_llba  +s_lba      ;current byte offset
ci_rptr  =ci_offst             ;FIFO read offset
ci_wptr  =ci_rptr  +s_dptr     ;FIFO write offset
ci_mode  =ci_wptr  +s_dptr     ;file type & mode
ci_nlink =ci_mode  +s_filmod   ;link count
ci_uid   =ci_nlink +s_lnkcnt   ;user ID
ci_gid   =ci_uid   +s_uid      ;group ID
ci_size  =ci_gid   +s_gid      ;file size
ci_toc   =ci_size  +s_filsiz   ;table of contents
ci_atime =ci_toc   +s_toc      ;ATIME: file last accessed
ci_mtime =ci_atime +s_time_t   ;MTIME: file last modified
ci_ctime =ci_mtime +s_time_t   ;CTIME: inode last modified
;
ci_devsf =ci_toc               ;device special fields
ci_dmin  =ci_devsf             ;minor device number
ci_dmaj  =ci_dmin  +s_byte     ;major device number
;
;	
;	Inode Sizes
;	
;
s_inodec =ci_ctime +s_time_t   ;in-core
s_inode  =s_ldblk  /ex2_x8     ;on-disk...
;
         .if di_ctime+s_time_t > s_inode
             .error "!!! on-disk inode size exceeds S_INODE definition !!!"
         .endif
;
n_inoldb =s_ldblk  /s_inode    ;inodes per logical disk block
;
;	
;	Directory Structure
;	
;
s_dsslot =s_inum   +s_filnam   ;slot size in bytes
n_ddslot =s_ldblk  /s_dsslot   ;slots per logical block
;
;
;	directory offsets...
;
do_cwd   =0                    ;CWD
do_par   =do_cwd   +s_dsslot   ;parent
do_start =do_par   +s_dsslot   ;1st user filename
;
;
;	slot offsets...
;
ds_inum  =0                    ;inode number
ds_fnam  =ds_inum  +s_inum     ;filename
;
;	
;	Filesystem Structural Constants
;	
;
n_bamta  =1                    ;BAM tag logical blocks
n_bamtab =n_bamta  *s_ldblkb   ;BAM tag bits
s_iamta  =s_rampag             ;IAM tag bytes
s_supblk =s_ldblk  *n_supblk   ;superblock bytes
s_tagblk =n_bamta  *s_ldblk    ;BAM tag bytes
;
;	
;	Superblock Structure
;	
;
sb_lock =0                     ;lock flag (in-core)...
;
;	xx000000
;	||
;	|+> 1: IAM being processed
;	+> 1: BAM being processed
;
sb_stat  =sb_lock  +s_fsflag   ;status flag...
;
;	xx000000
;	||
;	|+> 0: filesystem mounted read-only (in-core)
;	|            1: filesystem mounted read-write (in-core)
;	+> 1: filesystem mounted (on-disk) or...
;	                superblock dirty (in-core)
;
sb_fsnam =sb_stat  +s_fsflag   ;filesystem name
sb_magic =sb_fsnam +s_fsnam    ;magic number
sb_ktime =sb_magic +s_fsmag    ;filesystem creation time
sb_mtime =sb_ktime +s_time_t   ;last modification time
sb_orig  =sb_mtime +s_time_t   ;filesystem origin LBA
sb_fssen =sb_orig  +s_lba      ;filesystem sentinel LBA
sb_pblks =sb_fssen +s_lba      ;filesystem physical blocks
sb_lblks =sb_pblks +s_pblks    ;filesystem logical blocks
sb_imblk =sb_lblks +s_pblks    ;total IAM blocks
sb_dmblk =sb_imblk +s_pblks    ;total BAM blocks
sb_inblk =sb_dmblk +s_pblks    ;total inode blocks
sb_dablk =sb_inblk +s_pblks    ;total data blocks
sb_dabf  =sb_dablk +s_pblks    ;free data blocks
sb_inod  =sb_dabf  +s_pblks    ;total inodes
sb_inodf =sb_inod  +s_pblks    ;free inodes
sb_bamta =sb_inodf +s_pblks    ;BAM tag bitmap LBA
sb_iamsa =sb_bamta +s_lba      ;IAM origin LBA
sb_bamsa =sb_iamsa +s_lba      ;BAM origin LBA
sb_iasa  =sb_bamsa +s_lba      ;inode array origin LBA
sb_dbsa  =sb_iasa  +s_lba      ;data array origin LBA
sb_rsrvd =sb_dbsa  +s_lba      ;reserved space
sb_iamta =sb_lock  +s_rampag   ;inode tag bit field
s_sbcore =sb_iamta +s_iamta    ;in-core superblock size
;     
;	
;	Static Logical Block Offsets
;	
;
fs_super =0                    ;superblock
fs_tag   =fs_super +n_supblk   ;start of BAM tag block
fs_iam   =fs_tag   +n_bamta    ;start of IAM
;
;	
;	Filesystem Creation Constants
;	
;	The below table describes the constants & limits that apply to new file-
;	system generation.  "Blocks" in the units column are 1KB logical blocks.
;
;	    Symbol     Decimal     Hex    Description                    Units
;	    ---------------------------------
;	    M_INODE   16777216  01000000  maximum inode array size       inodes
;	    M_INODEP  16777215  00FFFFFF  usable inode array size        inodes
;	    M_INODEM      8191  00001FFF  minimum inode array size       inodes
;	    r             8192  00002000  logical block dimension        bits
;	    FOH816X          2  00000002  immutable filesystem overhead  blocks
;	    M_BAMBLK      8192  00002000  maximum BAM size               blocks
;	    M_DBLK    67108864  04000000  maximum data array size        blocks
;	    M_IBLK     2097152  00200000  maximum inode array size       blocks
;	    M_IAMBLK      2048  00000800  maximum IAM size               blocks
;	    M_FSSIZ   69216258  04202802  maximum filesystem size        blocks
;	    M_FSSIZM   2107395  00202803  minimum filesystem size        blocks
;	    ---------------------------------
;
;	 M_INODE is fixed by the number of inodes that can be mapped by the
;	  IAM.
;
;	    M_INODE = M_IAMBLK  r
;
;	 During filesystem generation inode zero will be permanently allocated,
;	  as a directory entry whose inode is zero marks a deleted file.  Hence
;	  inode zero can't be used to index a file.
;
;	    M_INODEP = M_INODE - 1
;
;	 M_INODEM reflects the number of inodes that can be mapped in a logical
;	  disk block.  This value is arbitrary in the sense that it could be as
;	  small as 3.  However, 3 inodes would result in a filesystem that would
;	  be incapable of storing user data.  Hence it was decided that since at
;	  least one IAM block has to be defined the minimum should be set to the
;	  number of inodes that 1 IAM block can map, minus 1.  During filesystem
;	  generation, an inode will be added to the requested inodes to account
;	  for the unused inode zero.
;
;	    M_INODEM = S_LDBLKB - 1
;
;	 r is the number of bits in a logical block & hence can be considered
;	  an atomic constant, since the size of a logical block (S_LDBLK) is at-
;	  omically defined by N_LDBLK & the size of a physical disk block, as
;	  well as the number bits in a byte.
;
;	    r = S_LDBLK  8
;
;	 FOH816X is immutable filesystem overhead comprised of the superblock &
;	  BAM tag block.
;
;	    FOH816X = N_SUPBLK + N_BAMTA
;
;	 M_BAMBLK is fixed by the number of BAM blocks that can be mapped by
;	  the BAM tag bitmap.
;
;	    M_BAMBLK = N_BAMTA  r
;
;	 M_DBLK is fixed by the number of data blocks that can be mapped by the
;	  BAM.
;
;	    M_DBLK = M_BAMBLK  r
;
;	 M_IAMBLK is fixed by the number of IAM blocks that can be mapped by
;	  the IAM tag bitmap, which is a structure within the superblock.  This
;	  bitmap's size is atomically defined as 256 bytes.
;
;	    M_IAMBLK = 256 * 8
;
;	 M_IBLK is fixed by the maximum number of inodes that can be mapped in
;	  the IAM.
;
;	    M_IBLK = M_INODE  S_INODE + (MOD(M_INODE,S_INODE) != 0)
;
;	  Note: MOD() is the modulo or remainder of division.
;
;	 M_FSSIZ is the maximum allowable size of a single filesystem.
;
;	    M_FSSIZ = FOH816X + M_BAMBLK + M_IAMBLK + M_DBLK
;
;	 M_FSSIZM is the smallest filesystem size that may be created, a limit
;	  that is based upon other filesystem minimums.
;
;	
;	Filesystem Geometry Calculation
;	
;	In the geometry calculations that follow, FSSIZ represents the filesys-
;	tem size in logical blocks & INODES represents the desired number of
;	inodes.  Both of these parameters are determined by the system administ-
;	rator when the filesystem is created.  Algebraic precedence rules apply.
;
;	   NINODE = INODES + 1
;
;	    NINODE is the number of inodes that will be created in the inode ar-
;	    ray & will always be 1 more than the number of inodes requested by
;	    the system administrator.  The extra inode accounts for unused inode
;	    zero.
;
;	   IBLK = NINODE  N_INOLDB + (MOD(NINODE,N_INOLDB) != 0)
;
;	    IBLK is the number of logical blocks needed to store the inode ar-
;	    ray.  The modulo operation compensates for when NINODE is not evenly
;	    divisible by S_INODE, in which case an extra block will be required
;	    to accomodate the final inodes.  N_INOLDB is the number of inodes
;	    that will fit into a logical block.
;
;	   IAMBLK = NINODE  r + (MOD(NINODE,r) != 0)
;
;	    IAMBLK is the number of logical blocks needed for the inode allocat-
;	    ion map (IAM).  The modulo operation compensates for when NINODE is
;	    not evenly divisible by r, in which case an extra block will be req-
;	    uired to accomodate the final bitmap flags.
;
;	    IAMBLK implicitly defines the number of active flags in the IAM tag
;	    bitmap, as there is 1 IAM tag flag per IAM block.
;
;	   UBLK = FSSIZ - (FOH816X + IAMBLK + IBLK)
;
;	    UBLK is the number of logical blocks that will be available for the
;	    filesystem's BAM & data blocks.
;
;	   DBLK = UBLK  r  (r + 1)
;
;	    DBLK is the number of logical blocks into which data can be written.
;	    "Data" refers to actual data & to the indirect blocks needed to ac-
;	    cess large files.
;
;	   BAMBLK = DBLK  r + (MOD(DBLK,r) != 0)
;
;	    BAMBLK is the number of logical blocks needed for the BAM.  The mod-
;	    ulo operation compensates for when DBLK is not evenly divisible by
;	    r, in which case an extra block will be required to accomodate the
;	    final bitmap flags.
;
;	    BAMBLK implicitly defines the number of active flags in the BAM tag
;	    bitmap, as there is 1 BAM tag flag per BAM block.
;
;	   (BAMBLK + DBLK) == UBLK
;	    (FOH816X + IAMBLK + BAMBLK + IBLK + DBLK) == FSSIZ
;
;	    These steps are consistency checks.  If either expression evaluates
;	    to false then an error occurred during a previous computation.
;
;	All results are positive numbers.  Division produces an integer result
;	like that of the trunc() function in ANSI C.
;	
;
foh816x  = n_supblk + n_bamta
m_bamblk = n_bamta  * s_ldblkb
m_dblk   = m_bamblk * s_ldblkb
m_inode  = s_iamta  * s_bbyte  * s_ldblkb
m_inodem = s_ldblkb - 1
m_inodep = m_inode  - 1
m_iamblk = m_inode  / s_ldblkb
m_iblk   = m_inode  / s_ldblk  * s_inode
m_fssiz  = foh816x  + m_bamblk + m_iamblk + m_iblk   + m_dblk
m_fssizm = foh816x  + m_iblk   + m_iamblk + s_ldblkb + 1
;
	.end