;===============================================================================
;
;	     816NIX FILESYSTEM LAYOUT       LBO (note 4)
;	+=================================+ ====================
;	|         data block array        |
;	++ note 3    (fs_dba)
;	|           inode array           |
;	++ note 2    (fs_inode)
;	|    block allocation map (BAM)   |
;	++ note 1    (fs_bam)
;	|    inode allocation map (IAM)   |
;	++ $00000002 (fs_iam)
;	|   BAM tag bitmap logical block  |
;	++ $00000001 (fs_tag)
;	|           superblock            |
;	+=================================+ $00000000 (fs_super)
;	 1) Determined by size of IAM.
;	 2) Determined by size of BAM.
;	 3) Determined by size of inode array.
;	 4) Logical block offset in filesystem.
;
;
;	         SUPERBLOCK LAYOUT          Byte Offset in Block
;	+=================================+ ====================
;	|    IAM tag bitmap (256 bytes)   |
;	++ $0100 (sb_iamta)
;	|    reserved space (172 bytes)   |
;	++ $0054 (sb_rsrvd)
;	|     filesystem sentinel LBA     |
;	++ $0050 (sb_fssen)
;	|    data storage starting LBA    |
;	++ $004C (sb_dbsa)
;	|        BAM starting LBA         |
;	++ $0048 (sb_bamsa)
;	|        BAM tag block LBA        |
;	++ $0044 (sb_bamta)
;	|      inode array start LBA      |
;	++ $0040 (sb_iasa)
;	|         IAM starting LBA        |
;	++ $003C (sb_iamsa)
;	|         free inode count        |
;	++ $0038 (sb_inodf)
;	|           total inodes          |
;	++ $0034 (sb_inod)
;	|      free data block count      |
;	++ $0030 (sb_dabf)
;	|        total data blocks        |
;	++ $002C (sb_dablk)
;	|        total inode blocks       |
;	++ $0028 (sb_inblk)
;	|         total BAM blocks        |
;	++ $0024 (sb_dmblk)
;	|         total IAM blocks        |
;	++ $0020 (sb_imblk)
;	|   logical blocks in filesystem  |
;	++ $001C (sb_lblks)
;	|  physical blocks in filesystem  |
;	++ $0018 (sb_pblks)
;	|     last modification time      |
;	++ $0012 (sb_mtime)
;	|     filesystem origin LBA       |
;	++ $000E (sb_orig)
;	|          magic number           |
;	++ $000A (sb_magic)
;	|        filesystem name          |
;	++ $0002 (sb_fsnam)
;	|          status flag            |
;	++ $0001 (sb_mode)
;	|           lock flag             |
;	+=================================+ $0000 (sb_lock)
;
;
;	         DIRECTORY LAYOUT           Byte Offset in Directory
;	+=================================+ ========================
;	| INUM | ...etc...                |
;	++ s_dsslot*5
;	| INUM | <filename3>              | ...etc...
;	++ s_dsslot*4
;	| INUM | <filename2>              |
;	++ s_dsslot*3
;	| INUM | <filename1>              |
;	++ s_dsslot*2
;	| INUM | .. (parent)              |
;	++ s_dsslot
;	| INUM | .  (CWD)                 |
;	+=================================+ $00000000
;	  CWD: current working directory
;	 INUM: 32 bit inode number
;
;
;	
;	Filesystem Atomic Definitions
;	
;
mag816nx =$fd187e21            ;filesystem magic number
n_ldblk  =2                    ;physical blocks in a logical block
n_supblk =1                    ;logical blocks in superblock...
;
;	
;	Although the superblock is defined to be the size of a logical block all
;	of its content is confined to the 1st 512 bytes.  Hence the core copy is
;	contained in a single physical disk block.  The remainder of the logical
;	block is dead space.
;	
;
s_filnam =60                   ;maximum size of a filename
s_inum   =s_dword              ;size of an inode number...
;
;	
;	The sum of S_FILNAM & S_INUM should be 2^n.  Hence the filename size 
;	should be 28, 60, 124 or 252.  Other values could be used but would re-
;	sult in wasted space, since the number of directory slots that will fit
;	into a logical block will not fully utilize the space in the block.
;	As the filename size is increased fewer directory slots will fit in a
;	logical block, causing an increase in filesystem overhead & directory
;	search times due to increased disk activity.
;
;	DO NOT change the definition of S_INUM!
;	
;
s_ldblk  =s_hdblk*n_ldblk      ;bytes in a logical block
s_ldblkb =s_hdblkb*n_ldblk     ;bits in a logical block
s_lboadr =s_dword              ;bytes in a logical block offset
;
;	
;	File Modes & Permissions
;	
;
fp_oexec =1 << 0               ;other execute
fp_owrit =1 << 1               ;other write
fp_oread =1 << 2               ;other read
fp_gexec =1 << 3               ;group execute
fp_gwrit =1 << 4               ;group write
fp_gread =1 << 5               ;group read
fp_uexec =1 << 6               ;owner execute
fp_uwrit =1 << 7               ;owner write
fp_uread =1 << 8               ;owner read
fp_stiky =1 << 9               ;sticky
fp_sgid  =1 << 10              ;set GID on execution
fp_suid  =1 << 11              ;set UID on execution
ft_notaf =@0000 << 12          ;not a file  invalid
ft_fifo  =@0001 << 12          ;named pipe
ft_specc =@0010 << 12          ;character special
ft_dir   =@0100 << 12          ;directory
ft_specb =@0110 << 12          ;block special
ft_user  =@1000 << 12          ;regular
ft_symlk =@1010 << 12          ;symbolic link
ft_sockt =@1100 << 12          ;socket
ft_pipe  =@1111 << 12          ;anonymous pipe
;
;	
;	Inode Structure Definitions
;	
;
n_tocadr =24                   ;LBAs in table of contents
n_tocind =3                    ;indirect LBAs in TOC
n_dirblk =n_tocadr-n_tocind    ;direct LBAs in TOC
s_filmod =s_word               ;mode & permissions
s_filsiz =s_dword              ;bytes in file
s_lnkcnt =s_word               ;link count
s_toc    =s_lba*n_tocadr       ;size of TOC
;
;	
;	Data Block Addressing Rules
;	
;
i_indbk1 =n_dirblk             ;TOC single indirection slots
i_indbk2 =i_indbk1+1           ;TOC double indirection slots
i_indbk3 =i_indbk2+1           ;TOC triple indirection slots
n_idbadr =s_ldblk/s_lba        ;LBAs per indirect block
n_iadrl1 =n_idbadr             ;total 1st indirect level LBAs
n_iadrl2 =n_idbadr*n_iadrl1    ;total 2nd indirect level LBAs
n_iadrl3 =n_idbadr*n_iadrl2    ;total 3rd indirect level LBAs
th_lbosi =n_dirblk             ;single indirection threshold
th_lbodi =th_lbosi+n_iadrl1    ;double indirection threshold
th_lboti =th_lbodi+n_iadrl2    ;triple indirection threshold
th_ovrfl =th_lboti+n_iadrl3    ;out-of-range offset sentinel
;
;	
;	On-Disk Inode Structure
;	
;
di_mode  =0                    ;file type & mode
di_nlink =di_mode+s_filmod     ;link count
di_uid   =di_nlink+s_lnkcnt    ;user ID
di_gid   =di_uid+s_uid         ;group ID
di_size  =di_gid+s_gid         ;file size in bytes
di_toc   =di_size+s_filsiz     ;table of contents
di_atime =di_toc+s_toc         ;atime: file last accessed
di_mtime =di_atime+s_time_t    ;mtime: file last modified
di_ctime =di_mtime+s_time_t    ;ctime: inode last modified
;
di_devsf =di_toc               ;device special fields
di_dmin  =di_devsf             ;minor device number
di_dmaj  =di_dmin+s_byte       ;major device number
;
;	
;	In-Core Inode Structure
;	
;
ci_inum  =0                    ;inode number
ci_sdid  =ci_inum+s_inum       ;device ID
ci_iflag =ci_sdid+s_byte       ;control & status flag...
;
;	xxxxx00x
;	|||||  |
;	|||||  +> 1: inode is a mount point
;	||||+> 1: process waiting for access
;	|||+> 1: perform synchronous write
;	||+> 1: update CTIME
;	|+> 1: update MTIME
;	+> 1: update ATIME
;
ci_lflag =ci_iflag+s_byte      ;lock flag...
;
;	+> bit 15
;	|
;	V
;	xxxxxxxxxxxxxxxx
;	||||||||||||||||
;	|+++++++++++++++> controlling PID
;	+> 1: inode locked
;
ci_opnct =ci_lflag+s_word      ;number of references
ci_llba  =ci_opnct+s_word      ;last LBA accessed
ci_offst =ci_llba+s_lba        ;current byte offset
ci_rptr  =ci_offst             ;FIFO read offset
ci_wptr  =ci_rptr+s_dptr       ;FIFO write offset
ci_mode  =ci_wptr+s_dptr       ;file type & mode
ci_nlink =ci_mode+s_filmod     ;link count
ci_uid   =ci_nlink+s_lnkcnt    ;user ID
ci_gid   =ci_uid+s_uid         ;group ID
ci_size  =ci_gid+s_gid         ;file size
ci_toc   =ci_size+s_filsiz     ;table of contents
ci_atime =ci_toc+s_toc         ;ATIME: file last accessed
ci_mtime =ci_atime+s_time_t    ;MTIME: file last modified
ci_ctime =ci_mtime+s_time_t    ;CTIME: inode last modified
;
ci_devsf =ci_toc               ;device special fields
ci_dmin  =ci_devsf             ;minor device number
ci_dmaj  =ci_dmin+s_byte       ;major device number
;
;	
;	Inode Sizes
;	
;
s_inodec =ci_ctime+s_time_t    ;in-core
s_inode  =s_ldblk/ex2_x8       ;on-disk...
;
         .if di_ctime+s_time_t > s_inode
             .error "!!! on-disk inode size exceeds S_INODE definition !!!"
         .endif
;
;	
;	If structural changes are made to the inode the on-disk size may need to
;	to be adjusted.  The on-disk size must be exactly S_LDBLK/(2^n), where n
;	is any non-negative integer.   A logical  block is defined as 1024 bytes
;	in 816NIX, hence 0 =< n <11.
;	
;
n_inoldb =s_ldblk/s_inode      ;inodes per logical disk block
;
;	
;	Directory Structure
;	
;
s_dsslot =s_inum+s_filnam      ;slot size in bytes
n_ddslot =s_ldblk/s_dsslot     ;slots per logical block
;
;
;	directory offsets...
;
do_cwd   =0                    ;CWD
do_par   =do_cwd+s_dsslot      ;parent
do_start =do_par+s_dsslot      ;1st user filename
;
;
;	slot offsets...
;
ds_inum  =0                    ;inode number
ds_fnam  =ds_inum+s_inum       ;filename
;
;	
;	Filesystem Structural Constants
;	
;
n_bamta  =1                    ;BAM tag logical blocks
n_bamtab =n_bamta*s_ldblkb     ;BAM tag bits
s_iamta  =s_rampag             ;IAM tag bytes
s_supblk =s_ldblk*n_supblk     ;superblock bytes...
;
;	
;	See above comments about the superblock's on-disk size.
;	
;
s_tagblk =n_bamta*s_ldblk      ;BAM tag bytes
;
;	
;	Superblock Definitions
;	
;
sb_lock =0                     ;lock flag (in-core)...
;
;	xx000000
;	||
;	|+> 1: IAM being processed
;	+> 1: BAM being processed
;
sb_mode  =sb_lock+s_byte       ;status flag...
;
;	xx000000
;	||
;	|+> 0: filesystem mounted read-only (in-core)
;	|            1: filesystem mounted read-write (in-core)
;	+> 1: filesystem mounted (on-disk) or...
;	                superblock dirty (in-core)
;
sb_fsnam =sb_mode+s_byte       ;filesystem name
sb_magic =sb_fsnam+s_fsnam     ;magic number
sb_orig  =sb_magic+s_fsmag     ;filesystem origin LBA
sb_mtime =sb_orig+s_lba        ;last modification time
sb_pblks =sb_mtime+s_time_t    ;physical filesystem blocks
sb_lblks =sb_pblks+s_dword     ;logical filesystem blocks
sb_imblk =sb_lblks+s_dword     ;total IAM blocks
sb_dmblk =sb_imblk+s_dword     ;total BAM blocks
sb_inblk =sb_dmblk+s_dword     ;total inode blocks
sb_dablk =sb_inblk+s_dword     ;total data blocks
sb_dabf  =sb_dablk+s_dword     ;free data block count
sb_inod  =sb_dabf+s_dword      ;total inodes
sb_inodf =sb_inod+s_dword      ;free inode count
sb_iamsa =sb_inodf+s_dword     ;IAM start LBA
sb_iasa  =sb_iamsa+s_lba       ;inode array start LBA
sb_bamta =sb_iasa+s_lba        ;BAM tag block LBA
sb_bamsa =sb_bamta+s_lba       ;BAM start LBA
sb_dbsa  =sb_bamsa+s_lba       ;data block array start LBA
sb_fssen =sb_dbsa+s_lba        ;filesystem sentinel LBA...
;
;	
;	SB_FSSEN will be the LBA of the physical block that immediately follows
;	the last physical block in the filesystem.  When a request is made to
;	seek within a file & the equivalent LBA is computed, it will be compared
;	to SB_FSSEN in order to verify that the seek will not violate the data
;	array's boundaries.
;	
;
sb_rsrvd =sb_fssen+s_lba       ;reserved space
sb_iamta =sb_lock+s_rampag     ;inode tag bit field
;
s_sbcore =sb_iamta+s_iamta     ;in-core superblock size
;     
;	
;	Fixed Logical Block Offsets
;	
;
fs_super =0                    ;superblock
fs_tag   =fs_super+n_supblk    ;start of BAM tag block
fs_iam   =fs_tag+n_bamta       ;start of IAM
;
;	
;	Filesystem Creation Constants
;	
;	The below table describes the constants & limits that apply to new file-
;	system generation.  "Blocks" in the units column are 1KB logical blocks.
;
;	    Symbol     Decimal     Hex    Description                    Units
;	    ---------------------------------
;	    M_INODE   16777216  01000000  maximum inode array size       inodes
;	    M_INODEP  16777215  00FFFFFF  usable inode array size        inodes
;	    M_INODEM         8  00000008  minimum inode array size       inodes
;	    r             8192  00002000  logical block dimension        bits
;	    FOH816X          2  00000002  immutable filesystem overhead  blocks
;	    M_BAMBLK      8192  00002000  maximum BAM size               blocks
;	    M_DBLK    67108864  04000000  maximum data array size        blocks
;	    M_IBLK     2097152  00200000  maximum inode array size       blocks
;	    M_IAMBLK      2048  00000800  maximum IAM size               blocks
;	    M_FSSIZ   69216258  04202802  maximum filesystem size        blocks
;	    M_FSSIZM   2107395  00202803  minimum filesystem size        blocks
;	    ---------------------------------
;
;	 M_INODE is fixed by the number of inodes that can be mapped by the
;	  IAM.
;
;	    M_INODE = M_IAMBLK  r
;
;	 During filesystem generation inode zero will be permanently allocated,
;	  as a directory entry whose inode is zero marks a deleted file.  Hence
;	  inode zero can't be used to index a file.
;
;	    M_INODEP = M_INODE - 1
;
;	 M_INODEM reflects the number of inodes that will fit into a logical
;	  block, with an inode added to account for inode 0.  This value is arb-
;	  itrary in the sense that it could be as small as 3.  However, 3 inodes
;	  would result in a filesystem incapable of storing user data.  Hence it
;	  was decided that since at least one inode has to be allocated just to
;	  create the '.' & '..' entries in the root directory of the filesystem,
;	  the minimum should be set to the number of inodes that will fit in a
;	  logical block.
;
;	    M_INODEM = N_INOLDB
;
;	 r is the number of bits in a logical block & hence can be considered
;	  an atomic constant, since the size of a logical block (S_LDBLK) is at-
;	  omically defined by N_LDBLK & the size of a physical disk block, as
;	  well as the number bits in a byte.
;
;	    r = S_LDBLK  8
;
;	 FOH816X is immutable filesystem overhead comprised of the superblock &
;	  BAM tag block.
;
;	    FOH816X = N_SUPBLK + N_BAMTA
;
;	 M_BAMBLK is fixed by the number of BAM blocks that can be mapped by
;	  the BAM tag bitmap.
;
;	    M_BAMBLK = N_BAMTA  r
;
;	 M_DBLK is fixed by the number of data blocks that can be mapped by the
;	  BAM.
;
;	    M_DBLK = M_BAMBLK  r
;
;	 M_IAMBLK is fixed by the number of IAM blocks that can be mapped by
;	  the IAM tag bitmap, which is a structure within the superblock.  This
;	  bitmap's size is atomically defined as 256 bytes.
;
;	    M_IAMBLK = 256 * 8
;
;	 M_IBLK is fixed by the maximum number of inodes that can be mapped in
;	  the IAM.
;
;	    M_IBLK = M_INODE  S_INODE + (MOD(M_INODE,S_INODE) != 0)
;
;	  Note: MOD() is the modulo or remainder of division.
;
;	 M_FSSIZ is the maximum allowable size of a single filesystem.
;
;	    M_FSSIZ = FOH816X + M_BAMBLK + M_IAMBLK + M_DBLK
;
;
;	Geometry Calculation
;	
;	In the geometry calculations that follow, FSSIZ represents the filesys-
;	tem size in logical blocks & INODES represents the desired number of
;	inodes.  Algebraic precedence rules apply.
;
;	   NINODE = INODES + 1
;
;	    NINODE is the number of inodes that will be created in the inode ar-
;	    ray.  An inode will be added to the INODES value so as to account
;	    for inode zero.
;
;	   IBLK = NINODE  N_INOLDB + (MOD(NINODE,N_INOLDB) != 0)
;
;	    IBLK is the number of logical blocks needed to store the inode ar-
;	    ray.  The modulo operation compensates for when NINODE is not evenly
;	    divisible by S_INODE, in which case an extra block will be required
;	    to accomodate the excess inodes.  N_INOLDB is the number of inodes
;	    that will fit into a logical block.
;
;	   IAMBLK = NINODE  r + (MOD(NINODE,r) != 0)
;
;	    IAMBLK is the number of logical blocks needed for the inode allocat-
;	    ion map (IAM).  The modulo operation compensates for when NINODE is
;	    not evenly divisible by r, in which case an extra block will be req-
;	    uired to accomodate the excess bitmap flags.
;
;	    IAMBLK implicitly defines the number of active flags in the IAM tag
;	    bitmap.
;
;	   UBLK = FSSIZ - (FOH816X + IAMBLK + IBLK)
;
;	    UBLK is the number of logical blocks that will be available for the
;	    data area of the filesystem.  This space will be split between the
;	    BAM & user- or system-generated data in a way that results in the
;	    least amount of wasted (aka "slack") space.
;
;	   DBLK = UBLK  r  (r + 1)
;
;	    DBLK is the number of logical blocks into which data can be written.
;	    "Data" refers to actual data & to the indirect blocks needed to ac-
;	    cess large files.
;
;	   BAMBLK = DBLK  r + (MOD(DBLK,r) != 0)
;
;	    BAMBLK is the number of blocks needed for the BAM.  The modulo oper-
;	    ation compensates for when DBLK is not evenly divisible by r, in
;	    which case an extra block will be required to accomodate the excess
;	    bitmap flags.
;
;	    BAMBLK implicitly defines the number of active flags in the BAM tag
;	    bitmap.
;
;	   (BAMBLK + DBLK) == UBLK
;	    (FOH816X + IAMBLK + BAMBLK + IBLK + DBLK) == FSSIZ
;
;	    These steps are consistency checks.  If either expression evaluates
;	    to false then an error occurred during a previous computation, im-
;	    plying that a bug exists somewhere in the code.
;
;	All results are positive numbers.  Division produces an integer result
;	like that of the trunc() function in ANSI C.
;	
;
foh816x  = n_supblk + n_bamta
m_bamblk = n_bamta  * s_ldblkb
m_dblk   = m_bamblk * s_ldblkb
m_inode  = s_iamta  * s_bbyte  * s_ldblkb
m_inodem = n_inoldb
m_inodep = m_inode  - 1
m_iamblk = m_inode  / s_ldblkb
m_iblk   = m_inode  / s_ldblk  * s_inode
m_fssiz  = foh816x  + m_bamblk + m_iamblk + m_iblk   + m_dblk
m_fssizm = foh816x  + m_iblk   + m_iamblk + s_ldblkb + 1
;
	.end