;================================================================================
;
;INODE DEFINITIONS
;
n_tocadr =24                   ;TOC: total LBAs
n_tocind =3                    ;TOC: indirect LBAs
n_dirblk =n_tocadr-n_tocind    ;TOC: direct LBAs
s_filmod =s_word               ;mode & permissions
s_filsiz =s_dword              ;bytes in file
s_inum   =s_word               ;inode number
s_lnkcnt =s_word               ;link count
s_toc    =s_dbadr*n_tocadr     ;size of TOC
;
;
;	on-disk structure...
;
di_mode  =0                    ;mode...
;
;	
;	A mode value of $0000 indicates that the inode is unused &
;	therefore available for allocation to a new file.   Active
;	inodes have the following bit assignments:
;
;	+> bit 15
;	|
;	V
;	xxxxxxxxxxxxxxxx
;	||||||||||||||||
;	|||||||||||||||+> 1: other execute
;	||||||||||||||+> 1: other write
;	|||||||||||||+> 1: other read
;	||||||||||||+> 1: group execute
;	|||||||||||+> 1: group write
;	||||||||||+> 1: group read
;	|||||||||+> 1: user execute
;	||||||||+> 1: user write
;	|||||||+> 1: user read
;	||||||+> 1: sticky
;	|||||+> 1: set GID on execution
;	||||+> 1: set UID on execution
;	++++> type: 0001: named pipe (FIFO)
;	                           0010: character device
;	                           0110: block device
;	                           1000: regular file
;	                           1010: symbolic link
;	                           1100: socket
;	                           1111: anonymous pipe
;	
;
di_nlink =di_mode+s_filmod     ;link count
di_uid   =di_nlink+s_lnkcnt    ;user ID
di_gid   =di_uid+s_uid         ;group ID
di_size  =di_gid+s_gid         ;file size in bytes
di_toc   =di_size+s_filsiz     ;data block table of contents
di_atime =di_toc+s_toc         ;atime: file last accessed
di_mtime =di_atime+s_time_t    ;mtime: file last modified
di_ctime =di_mtime+s_time_t    ;ctime: inode last modified
;
;	
;	The following fields apply to device special files.
;	
;
di_devsf =di_toc               ;device special fields
di_dmin  =di_devsf             ;minor device number
di_dmaj  =di_dmin+s_byte       ;major device number
;
;
;	in-core structure...
;
ci_inum  =0                    ;inode number
ci_sdid  =ci_inum+s_inum       ;device ID
ci_iflag =ci_sdid+s_byte       ;control & status flag...
;
;	xxxxx00x
;	|||||  |
;	|||||  +> 1: inode is a mount point
;	||||+> 1: process waiting for access
;	|||+> 1: perform synchronous write
;	||+> 1: update ATIME (file accessed)
;	|+> 1: update MTIME (file modified)
;	+> 1: update CTIME (inode modified)
;
ci_lflag =ci_iflag+s_byte      ;lock flag...
;
;	+> bit 15
;	|
;	V
;	xxxxxxxxxxxxxxxx
;	||||||||||||||||
;	|+++++++++++++++> controlling PID
;	+> 1: inode locked
;
ci_opnct =ci_lflag+s_pid       ;number of references
ci_llba  =ci_opnct+s_word      ;last LBA accessed
ci_offst =ci_llba+s_dbadr      ;current byte offset
ci_rptr  =ci_offst             ;FIFO read offset
ci_wptr  =ci_rptr+s_dptr       ;FIFO write offset
;
;	
;	The following fields are the in-core analogs of the on-disk
;	copy.  Refer to the on-disk definitions for field details.
;	
;
ci_mode  =ci_wptr+s_dptr       ;mode
ci_nlink =ci_mode+s_filmod     ;link count
ci_uid   =ci_nlink+s_lnkcnt    ;user ID
ci_gid   =ci_uid+s_uid         ;group ID
ci_size  =ci_gid+s_gid         ;file size
ci_toc   =ci_size+s_filsiz     ;data block table of contents
ci_atime =ci_toc+s_toc         ;file last accessed
ci_mtime =ci_atime+s_time_t    ;file last modified
ci_ctime =ci_mtime+s_time_t    ;inode last modified
;
ci_devsf =ci_toc               ;device special fields
ci_dmin  =ci_devsf             ;minor device number
ci_dmaj  =ci_dmin+s_byte       ;major device number
;
;
;	table of contents indirect block offsets...
;
i_indbk1 =n_dirblk             ;single indirection
i_indbk2 =n_dirblk+1           ;double indirection
i_indbk3 =i_indbk2+1           ;triple indirection
;
;
;	inode sizes...
;
s_inodec =ci_ctime+s_time_t    ;in-core
s_inode  =s_ldblk/ex2_x8       ;on-disk...
;
         .if di_ctime+s_time_t > s_inode
             .error "!!! on-disk inode size exceeds S_INODE definition !!!"
         .endif
;
;	
;	If structural changes are made to the inode the on-disk size may need to
;	to be adjusted.  The on-disk size must be exactly S_LDBLK/(2^n), where n
;	is any non-negative integer.  In the S51K filesystem, a logical block is
;	1024 bytes, hence valid values for n are 0 through 10 inclusive.
;	
;
n_inoldb =s_ldblk/s_inode      ;inodes per logical disk block
;
	.end
