;===============================================================================
;
;816NIX SUPERBLOCK STRUCTURE
;
;The superblock is the 1st block in a filesystem & contains the basic administr-
;ative information required for successful access.  Corruption to the superblock
;will generally render the filesystem inaccessible to the operating system.
;
;Pictorially, the superblock layout is as follows (read from bottom up):
;
;	        Superblock Layout         Offset in Block
;	+===============================+ ================
;	|       IAM tag bit field       |
;	++ $0036 (sb_iamta)
;	|  data block array start LBA   |
;	++ $0032 (sb_dbsa)
;	|    inode array start LBA      |
;	++ $002E (sb_iasa)
;	|       BAM starting LBA        |
;	++ $002A (sb_bamsa)
;	|      BAM tag block LBA        |
;	++ $0026 (sb_bamsa)
;	|       IAM starting LBA        |
;	++ $0022 (sb_iamsa)
;	|       free data blocks        |
;	++ $001E (sb_dmbf)
;	|       free inode blocks*      |
;	++ $001C (sb_imbf)
;	|         blocks in BAM         |
;	++ $0018 (sb_dmblk)
;	|         blocks in IAM*        |
;	++ $0016 (sb_imblk)
;	| physical blocks in filesystem |
;	++ $0012 (sb_nblks)
;	|    last modification time     |
;	++ $000E (sb_time)
;	|         magic number          |
;	++ $000A (sb_magic)
;	|             name              |
;	++ $0002 (sb_fsnam)
;	|         status flag           |
;	++ $0001 (sb_mode)
;	|          lock flag            |
;	+===============================+ $0000 (sb_lock)
;
;Fields marked with an asterisk contain static values in the current  implement-
;ation.  A future implementation may allow the administrator to configure an ar-
;bitrary number of inodes to better utilize filesystem space.
;
;During filesystem creation the mkfs program will calculate the LBAs  associated
;with fields such as SB_IAMSA & SD_DBSA, which implies that a filesystem  cannot
;be relocated as a unit.
;
;
sb_lock =0                     ;lock flag...
;
;	xx000000
;	||
;	|+> 1: IAM being processed
;	+> 1: BAM being processed
;
sb_mode  =sb_lock+s_byte       ;status flag...
;
;	xx000000
;	||
;	|+> 0: filesystem mounted read-only
;	|            1: filesystem mounted read-write
;	+> 1: filesystem mounted (on-disk)
;	                superblock dirty (in-core)
;
sb_fsnam =sb_mode+s_byte       ;filesystem name
sb_magic =sb_fsnam+s_fsnam     ;magic number
sb_mtime =sb_magic+s_fsmag     ;last modification time
sb_nblks =sb_mtime+s_time_t    ;physical blocks in filesystem
sb_imblk =sb_nblks+s_dword     ;total blocks in IAM
sb_dmblk =sb_imblk+s_word      ;total blocks in BAM
sb_imbf  =sb_dmblk+s_dword     ;number of free inode blocks
sb_dmbf  =sb_imbf+s_word       ;number of free data blocks
sb_iamsa =sb_dmbf+s_dword      ;IAM start LBA
sb_bamta =sb_iamsa+s_dbadr     ;BAM tag block LBA
sb_bamsa =sb_bamta+s_dbadr     ;BAM start LBA
sb_iasa  =sb_bamsa+s_dbadr     ;inode array start LBA
sb_dbsa  =sb_iasa+s_dbadr      ;data block array start LBA
sb_iamta =sb_dbsa+s_dbadr      ;inode tag bit field
;
s_sbcore =sb_iamta+s_iamta     ;in-core superblock size
;
	.end
