;================================================================================
;
;PHILIPS/NXP MULTIPLE CHANNEL UART CONFIGURATION & CONTROL CONSTANTS
;
nxx1freq =3686400              ;X1 clock frequency in Hz
nxctscal =nxx1freq/2           ;C/T scaled clock
;
;
;	ACR  auxiliary control register...
;
nxparbrt =%01100000            ;...
;
;	a) Use baud rate table #1  see data sheet.
;	b) C/T acts as a timer.
;	c) C/T clock source is scaled to X1  1.
;
;
;	CR  command register...
;
nxpcrrxe =%00000001            ;enable receiver
nxpcrrxd =%00000010            ;disable receiver
nxpcrtxe =%00000100            ;enable transmitter
nxpcrtxd =%00001000            ;disable transmitter
nxpcrmr1 =%00010000            ;select MR1
nxpcrrxr =%00100000            ;reset receiver
nxpcrtxr =%00110000            ;reset transmitter
nxpcresr =%01000000            ;reset error status
nxpcrbir =%01010000            ;reset received break change IRQ
nxpcrbks =%01100000            ;start break
nxpcrbke =%01110000            ;stop break
nxpcrrsa =%10000000            ;assert request to send
nxpcrrsd =%10010000            ;deassert request to send
nxpcrtme =%10100000            ;select C/T timer mode
nxpcrmr0 =%10110000            ;select MR0
nxpcrtmd =%11000000            ;select C/T counter mode
;
;
;	CSR  clock select register...
;
nxpcsdef =%01100110            ;RxD & TxD at 115.2 Kbps...
;
;	
;	The above data rate is based upon the value of NXPARBRT (above).
;	
;
;
;	CT  counter/timer...
;
nxpctdlo =<{nxctscal/hz}       ;underflows/sec LSB
nxpctdhi =>{nxctscal/hz}       ;underflows/sec MSB
;
;
;	IMR  interrupt mask register...
;
nxpairq  =nxpatirq|nxparirq    ;enable ch A RxD & TxD IRQs
nxpbirq  =nxpbtirq|nxpbrirq    ;enable ch B RxD & TxD IRQs
nxpiqmsk =nxpairq|nxpbirq|nxpctirq ;set IRQ sources...
;
;	
;	nxpiqmsk enables all channel IRQs plus timer IRQs.
;	
;
;
;	MR0  mode 0 register...
;
nxpm0def =%11001100            ;...
;
;	11001100
;	||||||||
;	|||||+++> extended baud rate #2
;	||||+> 16-deep FIFO
;	||++> TxD interrupts only when FIFO is empty
;	|+> RxD interrupts only when FIFO is full (see also MR1:6)
;	+> enable RxD watchdog timer
;
;
;	MR1  mode 1 register...
;
nxpm1def =%11010011            ;...
;
;	11010011
;	||||||||
;	||||||++> 8 bit char size
;	|||||+> parity type (ignored)
;	|||++> no parity generated or checked
;	||+> character error mode
;	|+> RxD interrupts only when FIFO is full (see also MR0:6)
;	+> RxD controls RTS
;
;
;	MR2  mode 2 register...
;
nxpm2def =%00010111            ;normal mode, auto RTS
;
;
;	OPCR  output port configuration register...
;
nxpopdef =%11110000            ;OP4-7 as IRQ outputs
;
;
;	combined setup commands...
;
nxpcrrtd =nxpcrtxd|nxpcrrxd    ;disable transmitter & receiver
nxpcrrte =nxpcrtxe|nxpcrrxe    ;enable transmitter & receiver
;
	.end