;================================================================================
;
;PHILLIPS/NXP DUART REGISTERS
;
;	Abbreviations used in comments are as follows:
;
;	    -------------
;	    C/T  Counter/timer
;	    IP   Input port
;	    LSB  Least significant byte
;	    MSB  Most significant byte
;	    OP   Output port
;	    (R)  Read only
;	    (RW) Read/write
;	    RxD  Receiver
;	    TxD  Transmitter
;	    (W)  Write only
;	    -------------
;
;
;	DUART channel register offsets...
;
nx_mr    =0                    ;mode select
nx_sr    =1                    ;status (R)
nx_csr   =1                    ;clock select (W)
nx_cr    =2                    ;command (W)
nx_rhr   =3                    ;RxD FIFO (R)
nx_thr   =3                    ;TxD FIFO (W)
nx_fif   =3                    ;communication FIFO (RW)
;
;
;	DUART block register offsets...
;
nx_ipcr  =4                    ;IP change (R)
nx_acr   =4                    ;auxiliary control (W)
nx_isr   =5                    ;interrupt status (R)
nx_imr   =5                    ;interrupt enable (W)
nx_ctu   =6                    ;C/T MSB value (R)
nx_ctl   =7                    ;C/T LSB value (R)
nx_crur  =6                    ;C/T MSB preset (W)
nx_crlr  =7                    ;C/T LSB preset (W)
nx_ureg  =12                   ;user-defined (RW)
nx_ipsr  =13                   ;IP status (R)
nx_opcr  =13                   ;OP configuration (W)
nx_sct   =14                   ;start C/T (R)
nx_sopb  =14                   ;set OP bits (W)
nx_rct   =15                   ;stop C/T (R)
nx_ropb  =15                   ;reset OP bits (W)
;
;================================================================================
;
;vQUART REGISTER OFFSETS
;
;
;	channel A...
;
nx_mra   =nx_mr                ;mode select
nx_sra   =nx_sr                ;status (R)
nx_csra  =nx_csr               ;clock select (W)
nx_cra   =nx_cr                ;command (W)
nx_rhra  =nx_rhr               ;RxD FIFO (R)
nx_thra  =nx_thr               ;TxD FIFO (W)
nx_fifa  =nx_fif               ;communication FIFO (RW)
;
;
;	channel B...
;
nx_mrb   =nx_mr+s_nxpchn       ;mode select
nx_srb   =nx_sr+s_nxpchn       ;status (R)
nx_csrb  =nx_csr+s_nxpchn      ;clock select (W)
nx_crb   =nx_cr+s_nxpchn       ;command (W)
nx_rhrb  =nx_rhr+s_nxpchn      ;RxD FIFO (R)
nx_thrb  =nx_thr+s_nxpchn      ;TxD FIFO (W)
nx_fifb  =nx_fif+s_nxpchn      ;communication FIFO (RW)
;
;
;	channel C...
;
nx_mrc   =nx_mra+s_iodev       ;mode select
nx_src   =nx_sra+s_iodev       ;status (R)
nx_csrc  =nx_csra+s_iodev      ;clock select (W)
nx_crc   =nx_cra+s_iodev       ;command (W)
nx_rhrc  =nx_rhra+s_iodev      ;RxD FIFO (R)
nx_thrc  =nx_thra+s_iodev      ;TxD FIFO (W)
nx_fifc  =nx_fifa+s_iodev      ;communication FIFO (RW)
;
;
;	channel D...
;
nx_mrd   =nx_mrc+s_nxpchn      ;mode select
nx_srd   =nx_src+s_nxpchn      ;status (R)
nx_csrd  =nx_csrc+s_nxpchn     ;clock select (W)
nx_crd   =nx_crc+s_nxpchn      ;command (W)
nx_rhrd  =nx_rhrc+s_nxpchn     ;RxD FIFO (R)
nx_thrd  =nx_thrc+s_nxpchn     ;TxD FIFO (W)
nx_fifd  =nx_fifc+s_nxpchn     ;communication FIFO (RW)
;
;
;	block A...
;
nx_ipcra =nx_ipcr              ;IP change (R)
nx_acra  =nx_acr               ;auxiliary control (W)
nx_isra  =nx_isr               ;interrupt status (R)
nx_imra  =nx_imr               ;interrupt enable (W)
nx_ctua  =nx_ctu               ;C/T MSB value (R)
nx_ctla  =nx_ctl               ;C/T LSB value (R)
nx_crura =nx_crur              ;C/T MSB preset (W)
nx_crlra =nx_crlr              ;C/T LSB preset (W)
nx_urega =nx_ureg              ;user-defined (RW)
nx_ipsra =nx_ipsr              ;IP status (R)
nx_opcra =nx_opcr              ;OP configuration (W)
nx_scta  =nx_sct               ;start C/T (R)
nx_sopba =nx_sopb              ;set OP bits (W)
nx_rcta  =nx_rct               ;stop C/T (R)
nx_ropba =nx_ropb              ;reset OP bits (W)
;
;
;	block B...
;
nx_ipcrb =nx_ipcr+s_iodev      ;IP change (R)
nx_acrb  =nx_acr+s_iodev       ;auxiliary control (W)
nx_isrb  =nx_isr+s_iodev       ;interrupt status (R)
nx_imrb  =nx_imr+s_iodev       ;interrupt enable (W)
nx_ctub  =nx_ctu+s_iodev       ;C/T MSB value (R)
nx_ctlb  =nx_ctl+s_iodev       ;C/T LSB value (R)
nx_crurb =nx_crur+s_iodev      ;C/T MSB preset (W)
nx_crlrb =nx_crlr+s_iodev      ;C/T LSB preset (W)
nx_uregb =nx_ureg+s_iodev      ;user-defined (RW)
nx_ipsrb =nx_ipsr+s_iodev      ;IP status (R)
nx_opcrb =nx_opcr+s_iodev      ;OP configuration (W)
nx_sctb  =nx_sct+s_iodev       ;start C/T (R)
nx_sopbb =nx_sopb+s_iodev      ;set OP bits (W)
nx_rctb  =nx_rct+s_iodev       ;stop C/T (R)
nx_ropbb =nx_ropb+s_iodev      ;reset OP bits (W)
;
	.end