;================================================================================
;
;BUFFER HEADER DEFINITIONS
;
;             
;  offset > 00  01  02  03  04  05  06  07  08  09  0A  0B  0C  0D  0E  0F
;             
;              |   |  ^^  ^^   |  ^^  ^^  
;              |   |      BH_TLA      BH_TMW   |      BH_LBA       BH_ADDR
;              |   |                           |
;              |   +> BH_LOCK               +> BH_DEV
;              +> BH_STAT
;
;	
;	  00     BH_STAT  xxxxxxxx
;	                  ||||||||
;	                  |||+++++> extent
;	                  ||+> 1: process waiting for buffer
;	                  |+> 1: buffer content modified
;	                  +> 1: buffer content valid
;
;	  01     BH_LOCK  xxxxxxxx
;	                  ||||||||
;	                  |+++++++> PID of locking process
;	                  +> 1: buffer locked
;
;	  02-05  BH_TLA   time since last access
;	  06-07  BH_TMW   time until must write
;	  08     BH_DEV   0xxxxxxx
;	                   |||||||
;	                   |||++++> SCSI ID
;	                   +++> SCSI LUN
;	
;	  09-0C  BH_LBA   logical block address
;	  0D-0F  BH_ADDR  buffer address
;
;	---------
;	Field Descriptions
;	------------------------------------
;	BH_STAT  This field indicates the status of the buffer:
;
;	     7:  Set when buffer content is valid.
;	     6:  Set when buffer content is dirty.
;	     5:  Set when a process is waiting for access to this buffer.
;	   0-4:  Meanings are as follows:
;
;	         %00000  Buffer is chained but isn't the primary buffer.
;	         %00001  Buffer is "standalone," the default.
;	         %xxxxx  Buffer is chained & is the chain's primary buffer; the
;	                 binary-encoded value in this bit field is the number of
;	                 buffers in the chain, including this one; will always be
;	                 at least %00010 in this case.
;
;       BH_LOCK  This field is used to prevent process contention for the buffer
;	         or the header:
;
;	     7:  Set when header or buffer has been locked.
;	   0-6:  Binary-encoded process ID of the process that has set the lock.
;
;	BH_TLA   This field is the time in seconds since the most recent buffer
;	         access; meaningless unless BH_STAT:7 is set.  This field is re-
;	         set to zero each time the buffer is read or modified.
;
;	BH_TMW   This field is the time remaining until this buffer must written
;	         to stable storage; reset to $0000 when the buffer is flushed.
;
;	BH_DEV   This field is the encoded SCSI device ID & logical unit number
;	         from which this buffer was loaded or to which it is to be writ-
;	         ten; meaningless unless BH_STAT:7 is set.
;
;	BH_LBA   This field is the 32 bit, big-endian logical block address from
;	         which this buffer was loaded or to which it is to be written;
;	         meaningless unless BH_STAT:7 is set.
;
;	BH_ADDR  This field is the 24 bit, little-endian address of this buffer.
;	         It is set at boot time when the kernel is initialized & is not
;	         subsequently changed.
;
;	Excepting BH_ADDR, all of the above fields will be cleared if the buffer
;	is not a standalone or primary buffer.
;	
;
bh_stat  =0                    ;status
bh_lock  =bh_stat+s_byte       ;lock
bh_tla   =bh_addr+s_byte       ;time since last access
bh_tmw   =bh_tla+s_dword       ;time until "must flush"
bh_dev   =bh_tmw+s_word        ;SCSI device/LUN
bh_lba   =bh_dev+s_byte        ;logical block address
bh_addr  =bh_lock+s_dbadr      ;buffer address
s_bhslot =bh_lba+s_xword       ;slot size
;
	.end