;POC SERIES MASS STORAGE LAYOUT & SYSTEM START-UP INFORMATION
;
;The POC series  includes a SCSI-2 subsystem  equipped  with an 8 bit bus, which
;means up to 7 devices of various types may be attached.  It is only possible to
;create a filesystem on a random  access  device,  typically a disk or a CD/DVD.
;CD/DVD filesystems are not discussed herein.
;
;Each disk can support up to 4 filesystems, which have no relationship to each
;other, other than being ensconced on the same disk.  In order for an operating
;system to mount a filesystem, it must consult a filesystem table, which is loc-
;ated in the 1st addressable block of the disk.  As this same block is the one
;usually read by a computer's BIOS in an attempt to load & execute an operating
;system, this block is traditionally referred to as the "boot block" (or master
;boot block), even if the disk is not bootable.
;
;If a disk is bootable, the boot block will also contain the master boot machine
;code that is the 1st step in bringing the system up to full operation.  The 1st
;filesystem on this disk will be the boot filesystem & will contain the operat-
;ing system kernel.
;
;Pictorially, the layout of a bootable disk with 2 filesystems is as follows:
;
;	           DISK LAYOUT            Starting Physical Block LBA
;	+===============================+ ======================================
;	|                               | The starting LBA of this filesystem is
;	|                               | defined by pb_fs002, which is equal to
;	|         2nd filesystem        | the 1st filesystem's size (physical
;	|                               | blocks) added to the 1st filesystem's
;	|                               | starting LBA (defined by pb_fs001).
;	+===============================+ $???????? (pb_fs002)
;	|                               |
;	|         1st filesystem        |
;	|                               |
;	+===============================+ $00000112 (pb_fs001)
;	|     reserved (256 blocks)     |
;	++ $00000012 (pb_rsrvd)
;	|          stage 2 boot         |
;	++ $00000002 (pb_stag2)
;	|          master  boot         |
;	+===============================+ $00000000 (pb_mboot)
;
;The above layout also applies to a non-boot disk.  However, physical blocks
;$00000002 through $00000011 inclusive aren't used & no master boot code is
;present in the master boot block.  Refer to the boot.65s & stage2.65s source
;files for more information.
;
;
;SYSTEM STARTUP
;
;At power-on or reset, the MPU jumps through the hardware reset vector, which is
;atomically defined as $00FFFC-$00FFFD.  The reset code in the BIOS ROM starts
;by disabling IRQs, setting binary arithmetic mode & switching the MPU to 65C02
;emulation & then back to 65C816 native mode.  The disabling of IRQs & resetting
;to binary mode is necessary to deal with the case when software calls the reset
;routinethese actions are automatic when a hardware reset occurs.
;
;The effect of switching MPU modes is to default the direct page, data bank &
;program bank registers, reset the stack pointer to $000100, & set the accumula-
;tor & index registers sizes to 8 bits.  Hence the MPU will start off in a known
;state before executing the balance of the reset code.
;
;With these preliminary operations completed, normal system startup will proceed
;as follows:
;
;1)  The BIOS will start by testing zero page & stack memory.  Any failure will
;    cause an immediate system halt.  The BIOS will not be able to report that
;    the test has failed, as no hardware intialization will have been performed
;    at this stage of the boot process & thus no output to the console will be
;    possible.
;
;2)  Next, the timekeeping & I/O hardware will initialized & jiffy IRQs will be
;    started.  The power-on self-test (POST) banner will be displayed & the bal-
;    ance of memory will tested.  As testing proceeds, a memory count will be
;    displayed.  If defective memory is encountered, a terse error message will
;    be displayed & the system will be halted.
;
;3)  Following POST, the SCSI subsystem will reset & configured.  If this step
;    fails, a terse error message will be displayed & control will be given to
;    the ROM-resident machine code monitor.
;
;4)  Upon completion of SCSI subsystem configuration, SCSI device enumeration
;    will take place.  Each detected device will be issued a "start unit" com-
;    mand, which will spin up disks & cause tape drives to load a tape if pres-
;    ent.  During enumeration, a detected device summary will be displayed.
;
;5)  Next, the BIOS will attempt to load the boot block from the designated boot
;    device (usually SCSI ID 0) into RAM at $000400two physical blocks will be
;    loaded.  Note that the BIOS knows nothing about filesystems & can only load
;    the boot block & execute the master boot code therein.  If the boot device
;    fails to respond or doesn't have a valid boot block, control will be given
;    to the machine code monitor.  The status message "no operating system" will
;    be displayed if the boot block can be loaded but doesn't contain a bootable
;    image.
;
;6)  If step 5 was successful, the master boot code will be executed.  As space
;    in the boot block for executable code is constrained, master boot will load
;    the stage 2 boot code, which is stored at physical block 2 on the disk, in-
;    to RAM at $000600.  A total of 16 physical blocks will be loaded.  If stage
;    2 code cannot be loaded, an error message will be displayed & control will
;    be given to the machine code monitor.
;
;7)  Master boot will examine the stage 2 boot image to verify that it contains
;    valid code.  If stage 2 is invalid, an error message will be displayed &
;    control will be given to the machine code monitor.  Otherwise, control will
;    be given to the stage 2 boot code.
;
;8)  The stage 2 boot code will examine the filesystem table located in the boot
;    block (which will remain RAM-resident at $000400) to find the root filesys-
;    tem on the disk.  If it is not found, an error message will be displayed &
;    control will be given to the machine code monitor.
;
;9)  If the root filesystem is found, "Stage 2 boot..." will be displayed & the
;    root directory will be searched for the file "bootos", which contains the
;    code required to load & start a kernel.  As bootos is loaded it will be
;    checked for corruption to avoid the potential for root filesystem damage.
;    If bootos cannot be found, is found to be corrupted or if the root direct-
;    ory is unreadable, an error message will be displayed & control will be
;    given to the machine code monitor.
;
;10) If found, bootos will be loaded into RAM at $00A000 & executed.  bootos is
;    the 1st program executed during the boot sequence that is interactive.  It
;    will start by clearing the console screen, displaying a banner & searching
;    the root filesystem for the file 816nix, which is the default kernel.  If
;    816nix is not present, bootos will issue a warning message but will contin-
;    ue to the next step.
;
;11) bootos will display the boot prompt & wait for up to 15 seconds for user
;    input.  If there is no user response, bootos will time out & attempt to
;    continue the boot process.
;
;12) If the user presses [CR] without typing anything, bootos will continue the
;    boot process in the same fashion as if it had timed out.
;
;13) If the user enters a recognizable command it will be executed & then bootos
;    will await another command.  If an alternate kernel is to be loaded its
;    path & filename will be entered & [CR] pressed to start the boot.
;
;14) Upon being commanded to boot or having timed out at the boot prompt, bootos
;    will load the main kernel into RAM at $010100 & interrupt handler code into
;    RAM at $00E000.  As the kernel is read in it will be checked for possible
;    corruption.  Should corruption be detected, or if the kernel (816nix or an
;    alternate) cannot be found, bootos will display an error message & give
;    control to the machine code monitor.
;
;15) Upon successfully loading the kernel, bootos will go to the kernel's start
;    vector.  The kernel, in turn, will complete the system initialization proc-
;    ess & start the 1st daemon, which is schedd.  schedd will complete the
;    start-up & start initd.  initd will read /etc/inittab to bring the system
;    up to multiuser mode, at which time it will be possible to log in.
;
;
;MASTER BOOT BLOCK LAYOUT
;
;The boot block is defined in hardware as physical block $00 on any  device cap-
;able of supporting random access.  A valid boot block will contain a filesystem
;table & a "magic number" that the BIOS will check to verify that the boot block
;is, in fact, valid.  If the disk is bootable, the boot block will also  contain
;machine code that the BIOS can execute to perform an initial system load (ISL).
;
;Pictorially, the boot block structure is as follows:
;
;	         BOOT BLOCK LAYOUT         Offset in Block
;	+================================+ ===============
;	|  master boot code (928 bytes)  |
;	++ $0060 (mb_code)
;	|  filesystem table  (90 bytes)  |
;	++ $0006 (mb_fstab)
;	|    boot block magic number     |
;	++ $0002 (mb_magic)
;	| master boot code start address |
;	+================================+ $0000 (mb_jaddr)
;
;If the master boot code start address (mb_jaddr) is $0000 the disk is not boot-
;able & the master boot code segment may contain undefined content.   Note  that
;a complete boot block is N_MBOOT contiguous physical blocks
;
;Bootable or not, the magic number must be valid in order for the  remainder  of
;the block's content to be considered valid.  If the boot block is corrupted the
;rest of the disk will be inaccessible by ordinary means.   Recovery may be pos-
;sible in some cases by using the machine code monitor to repair damaged fields.
;
;As space available in the boot block for executable code is limited, the master
;boot code is limited to loading & executing the stage 2 boot block.
;
;
;FILESYSTEM TABLE
;
;The filesystem table is an array of 4 equal-sized slots, each holding data that
;identifies the location & extent of each defined filesystem.   Pictorially, the
;filesystem table is organized as follows:
;
;	  FILESYSTEM TABLE LAYOUT   Offset in Table
;	+=========================+ ===============
;	|   4th filesystem slot   |
;	++ $38
;	|   3rd filesystem slot   |
;	++ $26
;	|   2nd filesystem slot   |
;	++ $14
;	|   1st filesystem slot   |
;	++ $02
;	|      1st open slot      |
;	++ $01 (ft_slota)
;	|   active entries (0-4)  |
;	+=========================+ $00 (ft_nfs)
;
;If the active entries field is $00 no filesystems are accessible, even if valid
;data exist in any slot.
;
;
;FILESYSTEM TABLE SLOT LAYOUT
;
;For each defined filesystem, a slot will be populated with the  data  needed to
;located & mount the filesystem.  Pictorially each slot is organized as follows:
;
;	  FILESYSTEM TABLE SLOT LAYOUT    Offset in Slot
;	+===============================+ ==============
;	| physical blocks in filesystem |
;	++ $12 (ft_pblks)
;	|    filesystem starting LBA    |
;	++ $0E (ft_addr)
;	|             name              |
;	++ $06 (ft_fsnam)
;	|         magic number          |
;	++ $02 (ft_magic)
;	|           not used            |
;	++ $01 (ft_unusd)
;	|             flag              |
;	+===============================+ $00 (ft_flag)
;
;Filesystem names must be unique.   A filesystem is considered  valid  only if a
;valid magic number is present, name is not null,  the starting LBA is not $00 &
;the physical block count is not zero.  However, even if the filesystem is valid
;it cannot be mounted unless its flag is set to active.  If, during the mounting
;process, a discrepancy is discovered between the filesystem's name in the table
;& the name in the superblock (1st block of the filesystem), a warning  will  be
;issued & the filesystem will be mounted read only to allow the administrator to
;make a correction.   This will not apply if the name discrepancy exists for the
;filesystem from which the system is booted.  In such a case,  a correction will
;have to be made via an external repair program started from the resident  mach-
;ine language monitor.
;
;
;STAGE 2 BOOT BLOCK LAYOUT
;
;As previously described, the master boot code will load & execute the  stage  2
;boot code.  Stage 2 is able to search for & execute a file called bootos in the
;root filesystem on the boot device.   bootos is stage 3 of the  boot  process &
;the first program in the boot sequence that is able to interact with the user.
;
;Pictorially, the stage 2 boot block structure is as follows:
;
;	     STAGE 2 BOOT BLOCK LAYOUT      Offset in Block
;	+=================================+ ================
;	| stage 2 boot code  (8186 bytes) |
;	++ $0006 (s2_code)
;	| stage 2 boot block magic number |
;	++ $0002 (s2_magic)
;	| stage 2 boot code start address |
;	+=================================+ $0000 (s2_jaddr)
;
;Offsets are relative to the beginning of the block.
;
;In order for stage 2 boot to be considered valid, the magic number must be val-
;id & the start address must point to the boot code.   If any part of stage 2 is
;corrupted a correction  will  have to be made  via an external  repair  program
;started from the resident machine language monitor.
;
	.end
