;===============================================================================
;
;drawrect: draw GRAPHIC RECTANGLE ON CONSOLE
;
;	
;	Calling syntax: pea #width            ;width in columns (1)
;	                pea #height           ;height in rows (1)
;	                pea #tlcol            ;top left corner column (2)
;	                pea #tlrow            ;top left corner row (2)
;	                jsr drawrect          ;draw rectangle (3)
;	                bcs error             ;parameter error
;
;	Exit registers: .A: used
;	                .B: used
;	                .X: used
;	                .Y: used
;	                DB: entry value
;	                DP: entry value
;	                PB: entry value
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    |||||||+> 0: okay
;	                    |||||||      1: parameter error
;	                    +++++++> entry value
;
;	Notes: 1) Width & height parameters include the corners & must be at
;	          least 2.
;
;	       2) Column/row coordinates are zero-based.
;
;	       3) Upon successfully drawing the rectangle, this function will
;	          position the cursor at 1,1 relative to the rectangle's top
;	          left corner.
;	
;
drawrect clc                   ;no initial error
         php
;
;
;STACK DEFINITIONS
;
.sfbase  .set 1                ;base stack index
.sfidx   .set .sfbase          ;workspace index
;
;> workspace stack frame start <
.hlinlen =.sfidx               ;horiz line length
.sfidx   .= .sfidx+s_byte
.vlinlen =.sfidx               ;vert line length
.sfidx   .= .sfidx+s_byte
.rcol    =.sfidx               ;right column
.sfidx   .= .sfidx+s_byte
.brow    =.sfidx               ;bottom row
.sfidx   .= .sfidx+s_byte
;> workspace stack frame end <
;
.s_wsf   =.sfidx-.sfbase       ;workspace stack frame size
.sfbase  .= .sfidx
;
;> MPU register stack frame start <
.reg_sr  =.sfidx               ;SR
.sfidx   .= .sfidx+s_mpusrx
.reg_pc  =.sfidx               ;PC
.sfidx   .= .sfidx+s_mpupcx
;> MPU register stack frame end <
;
.s_rsf   =.sfidx-.sfbase       ;register stack frame size
.sfbase  .= .sfidx
;
;> parameter stack frame start <
.tlrow   =.sfidx               ;TL corner row
.sfidx   .= .sfidx+s_word
.tlcol   =.sfidx               ;TL corner column
.sfidx   .= .sfidx+s_word
.height  =.sfidx               ;overall height
.sfidx   .= .sfidx+s_word
.width   =.sfidx               ;overall width
.sfidx   .= .sfidx+s_word
;> parameter stack frame end <
;
.s_psf   =.sfidx-.sfbase       ;parameter stack frame size
;
;
         longa
         sec
         tsc
         sbc !#.s_wsf           ;create fugacious workspace
         tcs
         shortr
;
;
;	compute rectangle details...
;
.setup   sec
         lda .width,s          ;rectangle width
         sbc #2                ;account for corners
         bcc .setup01          ;width < 2, error
;
         sta .hlinlen,s        ;set horiz line length
         adc .tlcol,s          ;TL corner column
         sta .rcol,s           ;set right column
         sec
         lda .height,s         ;rectangle height
         sbc #2                ;account for corners
         bcs .setup02          ;height > 1, okay
;
.setup01 lda .reg_sr,s         ;SR return value
         ora #sr_car           ;set carry, ...
         sta .reg_sr,s         ;rewrite SR &...
         bra .done             ;exit
;
.setup02 sta .vlinlen,s        ;set vert line length
         adc .tlrow,s          ;top row
         sta .brow,s           ;set bottom row
;
;	
;	draw Rectangle
;	
;
.main    lda .tlcol,s          ;TL corner column
         tax
         lda .tlrow,s          ;TL corner row
         tay
         jsr plot              ;position cursor
;
;
;	draw top border...
;
         printf dc_g2          ;print TL corner
         lda .hlinlen,s
         jsr linehg            ;draw horizontal line
         printf dc_g3          ;print TR corner
;
;
;	draw left border...
;
         lda .tlcol,s          ;TL corner column
         tax
         lda .tlrow,s          ;TL corner row
         tay
         iny                   ;down a row
         jsr plot              ;position cursor
         lda .vlinlen,s        ;vert line length
         jsr linevg            ;draw vertical line
;
;
;	draw bottom border...
;
         lda .tlcol,s          ;left column
         tax
         lda .brow,s           ;bottom row
         tay
         jsr plot              ;position to BL corner
         printf dc_g4          ;print BL corner
         lda .hlinlen,s
         jsr linehg            ;draw horizontal line
         printf dc_g5          ;print BR corner
;
;
;	draw right border...
;
         lda .rcol,s           ;right column
         tax
         lda .tlrow,s          ;top row
         tay
         iny                   ;down a row
         jsr plot              ;position cursor
         lda .vlinlen,s        ;vert line length
         jsr linevg            ;draw vertical line
;
;
;	position to 1,1 relative to rectangle...
;
         lda .tlcol,s          ;TL corner column
         tax
         lda .tlrow,s          ;TL corner row
         tay
         inx                   ;over a column &...
         iny                   ;down a row
         jsr plot              ;position cursor
;
;	
;	Common Return
;	
;
.done    longr
         clc
         tsc
         adc !#.s_wsf
         tcs
         adc !#.s_rsf
         tax
         adc !#.s_psf
         tay
         lda !#.s_rsf-1
         mvp #0,#0
         tya
         tcs
         plp
         rts
;
;================================================================================
;
;scsihdrd: DISPLAY SCSI ENUMERATION HEADER
;
scsihdrd printf sd_bannr       ;enumeration titles
         lda #seghllen
;
;================================================================================
;
;linehg: PRINT GRAPHIC HORIZONTAL LINE
;
;	
;	Preparatory Ops: .A: 8 bits: line length
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry values
;	
;
linehg   php
         longr
         pha                   ;save MPU state
         phx
         phy
         shortr
         ora #0
         beq .0000020          ;zero length, just exit
;
         pha                   ;save length again
         printf dc_bg          ;select graphics mode
         plx                   ;get length
         lda #t_hline          ;horizontal line char
;
.0000010 jsr putcha            ;print
         dex
         bne .0000010
;
         printf dc_eg          ;restore text mode
;
.0000020 longr
         ply                   ;restore MPU state
         plx
         pla
         plp
         rts
;
;================================================================================
;
;linevg: PRINT GRAPHIC VERTICAL LINE
;
;	
;	Preparatory Ops: .A: 8 bits: line length
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry value
;	
;
linevg   php                   ;save MPU state
         longr
         phy
         phx
         pha
         shortr
         tax
         beq .done             ;zero length, just exit
;
         phx
         printf dc_bg          ;select graphics mode
         plx
         lda #0
;
.0000010 tay                   ;starting string index
;
.0000020 lda si_vline,y        ;vertical line sequence
         beq .0000030          ;done with it
;
         jsr putcha            ;print vertical line
         iny
         bpl .0000020          ;next
;
.0000030 dex                   ;repeat?
         bne .0000010          ;yes
; 
         printf dc_eg          ;select text mode
;
.done    longr
         pla                   ;restore MPU state
         plx
         ply
         plp
         rts
;
;================================================================================
;
;SET CURSOR POSITION
;
;	
;	Preparatory Ops: .X: absolute column
;	                 .Y: absolute row
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry values
;
;	Examples:  LDX #COL
;	           LDY #ROW
;	           JSR PLOT
;	
;
plot     php                   ;save MPU state
         longr
         phd
         pha                   ;protect registers
         phx
         phy
         lda !#kerneldp         ;set default...
         tcd                   ;direct page
         shortr
         printf dc_plot        ;positioning preamble
         longx
         ply                   ;recover row
         plx                   ;recover col
         shortx
         tya
         clc
         adc #t_cpbase         ;convert &...
         jsr putcha            ;send to console
         txa                   ;row
         adc #t_cpbase
         jsr putcha
         longr
         pla                   ;restore
         pld
         plp                   ;restore MPU state
         rts
;
;===============================================================================
;
;sprint: PRINT NULL-TERMINATED CHARACTER STRING
;
;	
;	Calling syntax: pea #<string_addr>    ;pointer to string (1)
;	                jsr sprint
;	                bcs error             ;string too long
;
;	Exit registers: .A: entry value
;	                .B: entry value
;	                .X: entry value
;	                .Y: entry value
;	                DB: entry value
;	                DP: entry value
;	                PB: entry value
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    |||||||+> 0: okay
;	                    |||||||      1: string too long
;	                    +++++++> entry value
;
;	Notes: 1) Any method that pushes a word may be used to set the string's
;	          address  PEA is illustrative.
;	       2) Maximum permissible string length including the terminator is
;	          32,767 bytes.
;	       3) This function will block as long as the console's SIO queue
;	          is full.
;	       5) The entry value of DB determines from which bank the string
;	          will sourced.
;	       6) DP must point to the kernel's direct page shadow area.
;	       7) DO NOT JUMP OR BRANCH INTO THIS FUNCTION!
;	
;
sprint   clc                   ;no initial error
         php                   ;save MPU state
         longr
         phb
         phy
         phx
         pha
;
;
;STACK DEFINITIONS
;
;
.sfbase  .set 1                ;base stack index
.sfidx   .set .sfbase          ;workspace index
;
;> MPU register stack frame start <
.reg_c   =.sfidx               ;.C
.sfidx   .= .sfidx+s_word
.reg_x   =.sfidx               ;.X
.sfidx   .= .sfidx+s_word
.reg_y   =.sfidx               ;.Y
.sfidx   .= .sfidx+s_word
.reg_db  =.sfidx               ;DB
.sfidx   .= .sfidx+s_mpudbx
.reg_sr  =.sfidx               ;SR
.sfidx   .= .sfidx+s_mpusrx
.reg_pc  =.sfidx               ;PC
.sfidx   .= .sfidx+s_mpupcx
;> MPU register stack frame end <
;
.s_rsf   =.sfidx-.sfbase       ;register stack frame size
.sfbase  .= .sfidx
;
;> parameter stack frame start <
.src     =.sfidx               ;string address
.sfidx   .= .sfidx+s_word
;> parameter stack frame end <
;
.s_psf   =.sfidx-.sfbase       ;parameter stack frame size
;
;
         cld
         shortr
         longx
         ldy !#0
;
.main    lda (.src,s),y        ;get from string
         beq .done             ;end of string
;
         jsr putcha            ;output datum
         iny
         bpl .main             ;get next
;
.main010 lda .reg_sr,s         ;set carry...
         ora #sr_car           ;to flag...
         sta .reg_sr,s         ;an error
;
.done    longr
         clc
         tsc
         adc !#.s_rsf
         tax
         adc !#.s_psf
         tay
         lda !#.s_rsf-1
         mvp #0,#0             ;realign stack
         tya
         tcs
         pla                   ;restore MPU state
         plx
         ply
         plb
         plp
         rts
;
	.end