;================================================================================
;
;iabort: ABORT INTERRUPT SERVICE ROUTINE
;
iabort   longr                 ;16 bit registers
         phb                   ;save DB
         phd                   ;save DP
         pha
         phx
         phy
;
;
;ABORT REGISTER STACK FRAME
;
abt_yrx  =1                    ;.Y
abt_xrx  =abt_yrx+s_word       ;.X
abt_arx  =abt_xrx+s_word       ;.C
abt_dpx  =abt_arx+s_word       ;DP
abt_dbx  =abt_dpx+s_mpudpx     ;DB
abt_srx  =abt_dbx+s_mpudbx     ;SR
abt_pcx  =abt_srx+s_mpusrx     ;PC
abt_pbx  =abt_pcx+s_mpupcx     ;PB
;
;
         jmp (ivabort)         ;abort indirect vector...
;
;	
;	Unless the IVABORT vector is modified to point elsewhere routing will be
;	to the CRTI common interrupt return.
;	
;
;================================================================================
;
;ibrk: SOFTWARE INTERRUPT REQUEST SERVICE ROUTINE
;
ibrk     longr                 ;16 bit registers
         phb                   ;save DB
         phd                   ;save DP
         pha
         phx
         phy
;
;
;BRK REGISTER STACK FRAME
;
brk_yrx  =1                    ;.Y
brk_xrx  =brk_yrx+s_word       ;.X
brk_arx  =brk_xrx+s_word       ;.A
brk_dpx  =brk_arx+s_word       ;DP
brk_dbx  =brk_dpx+s_mpudpx     ;DB
brk_srx  =brk_dbx+s_mpudbx     ;SR
brk_pcx  =brk_srx+s_mpusrx     ;PC
brk_pbx  =brk_pcx+s_mpupcx     ;PB
;
;
         jmp (ivbrk)           ;BRK indirect vector
;
;	
;	IVBRK points to the BRK entry point of the M/L monitor.
;	
;
;================================================================================
;
;icop: COPROCESSOR INTERRUPT SERVICE ROUTINE
;
icop     longr                 ;16 bit registers
         phb                   ;save DB
         phd                   ;save DP
         pha
         phx
         phy
;
;
;COP REGISTER STACK FRAME
;
cop_yrx  =1                    ;.Y
cop_xrx  =cop_yrx+s_word       ;.X
cop_arx  =cop_xrx+s_word       ;.A
cop_dpx  =cop_arx+s_word       ;DP
cop_dbx  =cop_dpx+s_mpudpx     ;DB
cop_srx  =cop_dbx+s_mpudbx     ;SR
cop_pcx  =cop_srx+s_mpusrx     ;PC
cop_pbx  =cop_pcx+s_mpupcx     ;PB
;
;
         jmp (ivcop)           ;COP indirect vector...
;
;	
;	IVCOP points to the BRK entry point of the M/L monitor.
;	
;
;================================================================================
;
;iirq: HARDWARE INTERRUPT REQUEST SERVICE ROUTINE
;
iirq     longr                 ;16 bit registers
         phb                   ;save DB
         phd                   ;save DP
         pha
         phx
         phy
;
;
;IRQ REGISTER STACK FRAME
;
irq_yrx  =1                    ;.Y
irq_xrx  =irq_yrx+s_word       ;.X
irq_arx  =irq_xrx+s_word       ;.C
irq_dpx  =irq_arx+s_word       ;DP
irq_dbx  =irq_dpx+s_mpudpx     ;DB
irq_srx  =irq_dbx+s_mpudbx     ;SR
irq_pcx  =irq_srx+s_mpusrx     ;PC
irq_pbx  =irq_pcx+s_mpupcx     ;PB
;
;
         jmp (ivirq)           ;take IRQ indirect vector
;
iirqa    longa
         lda !#kerneldp
         tcd                   ;set kernel DP
;
;---------------------------------------
;
;vQUART IRQ PROCESSING
;
         shortr
         lda io_quart+nx_isra  ;DUART #1 interrupting?
         bne iirq0100          ;yes
;
         lda io_quart+nx_isrb  ;DUART #2 interrupting?
         bne iirq0200          ;yes
;
         jmp iirq0400          ;no, skip vQUART
;
;---------------------------------------
;
;DUART C/T IRQ PROCESSING
;
iirq0100 bit #nxpctirq         ;C/T interrupting?
         beq iirq0200          ;no
;
         bit io_quart+nx_rcta  ;yes, clear interrupt
         longa
;
;
;	process time fields...
;
         ldx jiffct            ;clock jiffy counter
         dex                   ;decrement
         bne .0000020          ;not time to update
;
         ldx #hz               ;reset jiffy count
         inc uptime            ;bump uptime LSW
         bne .0000010          ;done with uptime
;
         inc uptime+s_word     ;bump uptime MSW
;
.0000010 inc uxtime            ;bump TOD LSW
         bne .0000020          ;done with TOD
;
         inc uxtime+s_word     ;bump TOD MID
         bne .0000020          ;done with TOD
;
         inc uxtime+s_dword    ;bump TOD MSW
;
.0000020 stx jiffct            ;set new jiffy count
;
;
;	process foreground sleep...
;
         shortr
         bit tdflag            ;sleep in process?
         bpl iirq0200          ;no
;
         longx
         ldx uptime+s_word     ;uptime MSW
         cpx slptime+s_word    ;sleep expire time MSW
         bcc .0000040          ;has not expired
;
         bne .0000030          ;has expired
;
         ldx uptime            ;uptime LSW
         cpx slptime           ;sleep expire time LSW
         bcc .0000040          ;has not expired
;
.0000030 stz tdflag            ;has expired
;
.0000040 shortr
;
;---------------------------------------
;
;vQUART RECEIVER IRQ PROCESSING
;
iirq0200 lda io_sioq           ;get vQUART channel IRQ status
         beq iirq0400          ;nothing to process
;
         bit #m_irqrx          ;receiver IRQ?
         beq iirq0310          ;no
;
         sta tiairqst          ;save IRQ status
         ldy #0                ;starting channel index
;
;
;	channel processing loop...
;
.0000010 lsr tiairqst          ;channel receiver interrupting?
         bcc .0000030          ;no, skip it
;
         tya                   ;yes, copy channel index &...
         asl                   ;make channel pointer...
         tax                   ;offset
         lda #nxpcresr         ;clear any...
         sta (tiacr,x)         ;RxD overrun error
;
;
;	RHR processing loop...
;
.0000020 lda (tiasr,x)         ;get channel status
         bit #nxprxdr          ;RHR empty?
         beq .0000030          ;yes, done with channel
;
         lda (tiafif,x)        ;get & hold datum from...
         xba                   ;RHR
         lda tiaputrx,x        ;get queue 'put' pointer
         inc                   ;bump it &...
         and #m_tiaptr         ;mask to queue size
         ora tiaqpmsk,x        ;align to queue base addr
         cmp tiagetrx,x        ;any room in queue?
         beq .0000020          ;no, discard datum
;
         xba                   ;store datum...
         sta (tiaputrx,x)      ;in queue
         xba                   ;'put' pointer
         sta tiaputrx,x        ;set it &...
         bra .0000020          ;get next datum
;
;	...end of RHR processing loop
;
;
.0000030 iny                   ;next channel
         cpy #n_nxpchn         ;all channels processed?
         bcc .0000010          ;no
;
;	...end of channel processing loop
;
;---------------------------------------
;
;vQUART TRANSMITTER IRQ PROCESSING
;
iirq0300 lda io_sioq           ;IRQ status
;
iirq0310 bit #m_irqtx          ;transmitter IRQ?
         beq iirq0400          ;no, done with vQUART
;
         sta tiairqst          ;save IRQ status
         ldy #n_nxpchn-1       ;starting channel
;
;
;	channel processing loop...
;
.0000010 asl tiairqst          ;transmitter interrupting?
         bcc .0000040          ;no, skip it
;
         tya                   ;copy channel index &...
         asl                   ;make channel pointer...
         tax                   ;offset
;
;
;	THR processing loop...
;
.0000020 lda tiagettx,x        ;queue 'get' pointer
         cmp tiaputtx,x        ;queue 'put' pointer
         beq .0000030          ;nothing to transmit
;
         xba                   ;save 'get' pointer
         lda (tiasr,x)         ;get channel status
         bit #nxptxdr          ;TFIFO full?
         beq .0000040          ;yes, done for now
;
         lda (tiagettx,x)      ;read from queue &...
         sta (tiafif,x)        ;write to TFIFO
         xba                   ;recover 'get' pointer
         inc                   ;bump it &...
         and #m_tiaptr         ;mask to queue size
         ora tiaqpmsk,x        ;align to queue base addr
         sta tiagettx,x        ;save it &...
         bra .0000020          ;get next datum
;
;	...end of THR processing loop
;
.0000030 lda #nxpcrtxd         ;disable...
         sta (tiacr,x)         ;TxD
         lda tiatstab,x        ;tell foreground...
         tsb tiatxst           ;about it
;
.0000040 dey                   ;all channels serviced?
         bpl .0000010          ;no
;
;	...end of channel processing loop
;
;---------------------------------------
;
;SCSI IRQ PROCESSING
;
iirq0400 bit scsihba           ;HBA present?
         bpl iirq0500          ;no
;
         ldy io_scsi+sr_stat   ;yes, get general status
         bpl iirq0500          ;HBA not interrupting
;
         ldx io_scsi+sr_isr    ;get command status
         lda io_scsi+sr_irqst  ;get interrupt status
         longa
         and !#%11111111       ;squelch noise in .B
;
;	
;	The following code modifies the stack frame that was pushed by the ISR
;	preamble,  thus affecting the behavior of the foreground code that was
;	interrupted.  The changes are as follows:
;
;	    Frame       MPU
;	    Offset    Register   Data or operation
;	    --------------------------
;	    irq_arx      .C      SCSI controller interrupt status
;	    irq_xrx      .X      SCSI controller command status
;	    irq_yrx      .Y      SCSI controller general status
;	    irq_pcx      PC      SCSI foreground execution vector
;	    irq_srx      SR      C & D cleared, m & x set
;	    --------------------------
;
;	No analysis of status is made here; the foreground sees to that.
;	
;
         sta irq_arx,s         ;.A = interrupt status
         txa
         sta irq_xrx,s         ;.X = command status
         tya
         sta irq_yrx,s         ;.Y = general status
         lda ivscsi            ;get alternate driver vector
         bne .0000010          ;vector defined, so use it
;
         lda !#scsicmda        ;default if no alternate
;
.0000010 sta irq_pcx,s         ;reroute foreground code &...
         stz ivscsi            ;invalidate alternate vector
         shorta                ;8 bit .A
         lda irq_srx,s         ;get status register
         and #sr_bdm|sr_car_i  ;clear C & D flags
         ora #sr_amw|sr_ixw    ;set m & x flags
         sta irq_srx,s         ;change stack copy
;
;---------------------------------------
;
;FUTURE IRQ PROCESSING
;
iirq0500 ;!!! DO NOT DELETE THIS LABEL !!!
;
;================================================================================
;
;crti: COMMON INTERRUPT RETURN
;
crti     longr
         ply                   ;restore MPU state
         plx
         pla
         pld
         plb
         rti
;
;================================================================================
;
;inmi: NON-MASKABLE INTERRUPT SERVICE ROUTINE
;
inmi     longr                 ;16 bit registers
         phb                   ;save DB
         phd                   ;save DP
         pha
         phx
         phy
;
;
;NMI REGISTER STACK FRAME
;
nmi_yrx  =1                    ;.Y
nmi_xrx  =nmi_yrx+s_word       ;.X
nmi_arx  =nmi_xrx+s_word       ;.C
nmi_dpx  =nmi_arx+s_word       ;DP
nmi_dbx  =nmi_dpx+s_mpudpx     ;DB
nmi_srx  =nmi_dbx+s_mpudbx     ;SR
nmi_pcx  =nmi_srx+s_mpusrx     ;PC
nmi_pbx  =nmi_pcx+s_mpupcx     ;PB
;
;
         jmp (ivnmi)           ;NMI indirect vector
;
inmia    longa
         lda nmi_pcx,s         ;interrupted address
         cmp !#mm_rom          ;anywhere in ROM?
         bcs crti              ;yes, ignore NMI
;
         jmp monbrk            ;goto M/L monitor
;
	.end