;================================================================================
;
;dpyramct: DISPLAY POST RAM COUNT
;
dpyramct php
         longr
         phy
         phx
         pha
         ldx mpptr01           ;copy...
         ldy mpptr01+s_word    ;current byte count to...
         stx faca              ;primary...
         sty faca              ;primary accumulator
         shortx
         ldx #ex2_x10          ;2^10 factor
;
.0000010 lsr faca+s_word       ;convert count to...
         ror faca              ;kilobytes
         dex
         bne .0000010
;
         shortr
         ldx #memctcol
         ldy #memctrow
         jsr plot              ;position cursor
         pea kwrkbuf
         jsr ibinasc           ;convert count to...
         pea kwrkbuf           ;decimal string &...
         jsr sprint            ;print it
         longr
         pla                   ;restore MPU state
         plx
         ply
         plp
         rts
;
;===============================================================================
;
;drawrect: DRAW GRAPHIC RECTANGLE ON CONSOLE
;
;	
;	Calling syntax: pea #width            ;width in columns (1)
;	                pea #height           ;height in rows (1)
;	                pea #tlcol            ;top left corner column (2)
;	                pea #tlrow            ;top left corner row (2)
;	                jsr drawrect          ;draw rectangle (3)
;	                bcs error             ;parameter error
;
;	Exit registers: .A: used
;	                .B: used
;	                .X: used
;	                .Y: used
;	                DB: entry value
;	                DP: entry value
;	                PB: entry value
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    |||||||+> 0: okay
;	                    |||||||      1: parameter error
;	                    +++++++> entry value
;
;	Notes: 1) Width & height parameters include the corners & must be at
;	          least 2.
;
;	       2) Column/row coordinates are zero-based.
;
;	       3) Upon successfully drawing the rectangle, this function will
;	          position the cursor at 1,1 relative to the rectangle's top
;	          left corner.
;	
;
drawrect clc                   ;no initial error
         php
;
;
;STACK DEFINITIONS
;
.sfbase  .set 1                ;base stack index
.sfidx   .set .sfbase          ;workspace index
;
;> workspace stack frame start <
.hlinlen =.sfidx               ;horiz line length
.sfidx   .= .sfidx+s_byte
.vlinlen =.sfidx               ;vert line length
.sfidx   .= .sfidx+s_byte
.rcol    =.sfidx               ;right column
.sfidx   .= .sfidx+s_byte
.brow    =.sfidx               ;bottom row
.sfidx   .= .sfidx+s_byte
;> workspace stack frame end <
;
.s_wsf   =.sfidx-.sfbase       ;workspace stack frame size
.sfbase  .= .sfidx
;
;> MPU register stack frame start <
.reg_sr  =.sfidx               ;SR
.sfidx   .= .sfidx+s_mpusrx
.reg_pc  =.sfidx               ;PC
.sfidx   .= .sfidx+s_mpupcx
;> MPU register stack frame end <
;
.s_rsf   =.sfidx-.sfbase       ;register stack frame size
.sfbase  .= .sfidx
;
;> parameter stack frame start <
.tlrow   =.sfidx               ;TL corner row
.sfidx   .= .sfidx+s_word
.tlcol   =.sfidx               ;TL corner column
.sfidx   .= .sfidx+s_word
.height  =.sfidx               ;height
.sfidx   .= .sfidx+s_word
.width   =.sfidx               ;width
.sfidx   .= .sfidx+s_word
;> parameter stack frame end <
;
.s_psf   =.sfidx-.sfbase       ;parameter stack frame size
;
;
         longa
         cld                   ;binary arithmetic
         sec
         tsc
         sbcw .s_wsf           ;create fugacious workspace
         tcs
         shortr
;
;
;	compute rectangle details...
;
.setup   sec
         ldas .width           ;rectangle width
         sbc #2                ;account for corners
         bcc .setup01          ;width < 2, error
;
         stas .hlinlen         ;set horiz line length
         adcs .tlcol           ;TL corner column
         stas .rcol            ;set right column
         sec
         ldas .height          ;rectangle height
         sbc #2                ;account for corners
         bcs .setup02          ;height > 1, okay
;
.setup01 ldas .reg_sr          ;SR return value
         ora #sr_car           ;set carry, ...
         stas .reg_sr          ;rewrite SR &...
         bra .done             ;exit
;
.setup02 stas .vlinlen         ;set vert line length
         adcs .tlrow           ;TL corner row
         stas .brow            ;set bottom row
;
;	
;	Generate Rectangle
;	
;
.main    ldas .tlcol           ;TL corner column
         tax
         ldas .tlrow           ;TL corner row
         tay
         jsr plot              ;position cursor
;
;
;	draw top border...
;
         pea dc_g2
         jsr sprint            ;print TL corner
         ldas .hlinlen
         tax
         jsr linehg            ;draw horizontal line
         pea dc_g3
         jsr sprint            ;print TR corner
;
;
;	draw left border...
;
         ldas .tlcol           ;TL corner column
         tax
         ldas .tlrow           ;TL corner row
         tay
         iny                   ;down a row
         jsr plot              ;position cursor
         ldas .vlinlen         ;vert line length
         tax
         jsr linevg            ;draw vertical line
;
;
;	draw right border...
;
         ldas .rcol            ;right column
         tax
         ldas .tlrow           ;top row
         tay
         iny                   ;down a row
         jsr plot              ;position cursor
         ldas .vlinlen         ;vert line length
         tax
         jsr linevg            ;draw vertical line
;
;
;	draw bottom border...
;
         ldas .tlcol           ;left column
         tax
         ldas .brow            ;bottom row
         tay
         jsr plot              ;position to BL corner
         pea dc_g4
         jsr sprint            ;print BL corner
         ldas .hlinlen
         tax
         jsr linehg            ;draw horizontal line
         pea dc_g5
         jsr sprint            ;print BR corner
;
;
;	position to 1,1 relative to rectangle...
;
         ldas .tlcol           ;TL corner column
         tax
         ldas .tlrow           ;TL corner row
         tay
         inx                   ;over a column &...
         iny                   ;down a row
         jsr plot              ;position cursor
;
;	
;	Common Return
;	
;
.done    longr
         clc
         tsc
         adcw .s_wsf
         tcs
         adcw .s_rsf
         tax
         adcw .s_psf
         tay
         ldaw .s_rsf-1
         mvp 0,0
         tya
         tcs
         plp
         rts
;
;================================================================================
;
;scsihdrd: DISPLAY SCSI ENUMERATION HEADER
;
;scsihdrd pea sd_bannr          ;enumeration titles
;         jsr sprint
;         ldx #seghllen
;
;================================================================================
;
;linehg: PRINT GRAPHIC HORIZONTAL LINE
;
;	
;	Preparatory Ops: .X: line length
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry values
;	
;
linehg   php                   ;save MPU state
         longr
         phy
         phx
         pha
         txy
         beq .done01           ;zero length, just exit
;
         shortr
         pea dc_bg
         jsr sprint            ;select graphics mode
         lda #t_hline          ;horizontal line char
         ldx #siocha           ;the default channel
;
.0000010 jsr sioput            ;on console
         dey                   ;step count
         bne .0000010
;
         pea dc_eg
         jsr sprint            ;select text mode
;
.done    longr
;
.done01  pla                   ;restore MPU state
         plx
         ply
         plp
         rts
;
;================================================================================
;
;linevg: PRINT GRAPHIC VERTICAL LINE
;
;	
;	Preparatory Ops: .X: line length
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry value
;	
;
linevg   php                   ;save MPU state
         longr
         phy
         phx
         pha
         shortr
         txy
         beq .done             ;zero length, just exit
;
         pea dc_bg
         jsr sprint            ;select graphics mode
;         
.0000010 phx                   ;save count
         ldx #siocha           ;SIO channel
         ldy #0
;
.0000020 lda si_vline,y        ;vertical line sequence
         beq .0000030          ;done with it
;
         jsr sioput            ;print vertical line
         iny
         bpl .0000020          ;next
;
.0000030 plx                   ;recover count
         dex                   ;repeat?
         bne .0000010          ;yes
; 
         printf dc_eg          ;select text mode
;
.done    longr
         pla                   ;restore MPU state
         plx
         ply
         plp
         rts
;
;================================================================================
;
;plot: SET CURSOR POSITION
;
;	
;	Calling syntax: ldx column            ;absolute column
;	                ldy row               ;absolute row
;	                jsr plot
;
;	Exit registers: .A: entry value
;	                .B: entry value
;	                .X: entry value
;	                .Y: entry value
;	                DB: entry value
;	                DP: entry value
;	                PB: entry value
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    ++++++++> entry value
;
;	Notes: 1) DB must be set to kernel's data bank.
;	       2) DP must point to the kernel's direct page shadow area.
;	       3) The I/O block must be in context.
;	
;
plot     php                   ;save MPU state
         longr
         phy
         phx
         pha                   ;protect
;
;
.reg_c   =1                    ;.C
.reg_x   =.reg_c+s_word        ;.X (col)
.reg_y   =.reg_x+s_word        ;.Y (row)
;
;
         shortr
         printf dc_plot        ;positioning preamble
         clc
         tya                   ;get row
         txy                   ;protect column
         adc #t_cpbase         ;convert &...
         ldx #siocha           ;send to...
         jsr sioput            ;console
         tya                   ;column
         adc #t_cpbase
         jsr sioput
         longr
         pla                   ;restore MPU state
         plx
         ply
         plp
         rts
;
;===============================================================================
;
;sprint: PRINT NULL-TERMINATED CHARACTER STRING
;
;	
;	Calling syntax: pea #<string_addr>    ;pointer to string (1)
;	                jsr sprint
;	                bcs error             ;string too long
;
;	Exit registers: .A: entry value
;	                .B: entry value
;	                .X: entry value
;	                .Y: entry value
;	                DB: entry value
;	                DP: entry value
;	                PB: entry value
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    |||||||+> 0: okay
;	                    |||||||      1: string too long
;	                    +++++++> entry value
;
;	Notes: 1) Any method that pushes a word may be used to set the string's
;	          address  PEA is illustrative.
;	       2) Maximum permissible string length including the terminator is
;	          32,767 bytes.
;	       3) This function will block as long as the channel's SIO queue
;	          is full.
;	       5) The entry value of DB determines from which bank the string
;	          will sourced.
;	       6) DP must point to the kernel's direct page shadow area.
;	       7) The I/O block must be in context.
;	       8) DO NOT JUMP OR BRANCH INTO THIS FUNCTION!
;	
;
sprint   clc                   ;no initial error
         php                   ;save MPU state
         longr
         phb
         phy
         phx
         pha
;
;
;STACK DEFINITIONS
;
;
.sfbase  .set 1                ;base stack index
.sfidx   .set .sfbase          ;workspace index
;
;> MPU register stack frame start <
.reg_c   =.sfidx               ;.C
.sfidx   .= .sfidx+s_word
.reg_x   =.sfidx               ;.X
.sfidx   .= .sfidx+s_word
.reg_y   =.sfidx               ;.Y
.sfidx   .= .sfidx+s_word
.reg_db  =.sfidx               ;DB
.sfidx   .= .sfidx+s_mpudbx
.reg_sr  =.sfidx               ;SR
.sfidx   .= .sfidx+s_mpusrx
.reg_pc  =.sfidx               ;PC
.sfidx   .= .sfidx+s_mpupcx
;> MPU register stack frame end <
;
.s_rsf   =.sfidx-.sfbase       ;register stack frame size
.sfbase  .= .sfidx
;
;> parameter stack frame start <
.src     =.sfidx               ;string address
.sfidx   .= .sfidx+s_word
;> parameter stack frame end <
;
.s_psf   =.sfidx-.sfbase       ;parameter stack frame size
;
;
         cld
         shortr
         ldx #siocha           ;console channel
         longx
         ldyw 0
;
.main    ldasi .src            ;get from string
         beq .done             ;end of string
;
         jsr sioput            ;output datum
         bcs .main010          ;error!
;
         iny
         bpl .main             ;get next
;
.main010 ldas .reg_sr          ;set carry...
         ora #sr_car           ;to flag...
         stas .reg_sr          ;an error
;
.done    longr
         clc
         tsc
         adcw .s_rsf
         tax
         adcw .s_psf
         tay
         ldaw .s_rsf-1
         mvp 0,0               ;realign stack
         tya
         tcs
         pla                   ;restore MPU state
         plx
         ply
         plb
         plp
         rts
;
;================================================================================
;
;setautoi: SET UP RTC NVRAM AUTOINCREMENT
;
;setautoi lda #d11aimsk
;         tsb crb_rtc           ;enable NVRAM autoincrement
;         rts
;
	.end