;================================================================================
;
;getcha: GET DATUM FROM TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           ||||||+> entry value
;	           |||||+> 0
;	           |||++> entry value
;	           ||+> 1
;	           ++> entry value
;
;	Example: JSR GETCHA
;	         BCS NODATUM
;	         STA DATUM
;	
;
getcha   clc                   ;assume datum is available
         php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldx !#nxpchana        ;channel index
         bra getchcom          ;goto common function
;
;================================================================================
;
;getchb: GET DATUM FROM TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           ||||||+> entry value
;	           |||||+> 0
;	           |||++> entry value
;	           ||+> 1
;	           ++> entry value
;
;	Example: JSR GETCHB
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchb   clc                   ;assume datum is available
         php                   ;save MPU state
         longr
         phx                   ;save
         ldx !#nxpchanb        ;channel B index
;
;================================================================================
;
;getchcom: GET DATUM FROM TIA-232 CHANNEL (common code)
;
;	
;	Do not directly call this function, as the stack will become unbalanced.
;	Only use the GETCHx functions, where x is the channel ID.
;	
;
getchcom phy
         pha
         phd
;
;
.dpreg   =1                    ;DP
.areg    =.dpreg+s_mpudpx      ;.C
.yreg    =.areg+s_word         ;.Y
.xreg    =.yreg+s_word         ;.X
.sreg    =.xreg+s_word         ;SR
;
;
         pea #kerneldp
         pld                   ;set kernel direct page
         shortr
         txa                   ;channel index...
         asl                   ;becomes...
         tax                   ;channel offset
         lda tiagetrx,x        ;RxQ 'get' pointer
         cmp tiaputrx,x        ;RxQ 'put' pointer
         beq .nodata           ;RxQ is empty
;
         tay                   ;save 'get' pointer
         lda (tiagetrx,x)      ;get oldest datum &...
         sta .areg,s           ;return to caller
         tya                   ;recover 'get' pointer
         inc                   ;bump it &...
         and #m_tiaptr         ;mask to queue size
         ora tiaqpmsk,x        ;align to queue base addr
         sta tiagetrx,x        ;save new pointer
;
.done    longr
         pld                   ;restore MPU state
         pla
         ply
         plx
         plp
         rts
;
.nodata  lda .sreg,s           ;SR stack copy
         ora #sr_car           ;set carry &...
         sta .sreg,s           ;rewrite
         bra .done
;
;================================================================================
;
;getchc: GET DATUM FROM TIA-232 CHANNEL C
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           ||||||+> entry value
;	           |||||+> 0
;	           |||++> entry value
;	           ||+> 1
;	           ++> entry value
;
;	Example: JSR GETCHC
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchc   clc                   ;assume datum is available
         php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldx !#nxpchanc        ;channel index
         bra getchcom          ;goto common function
;
;================================================================================
;
;getchd: GET DATUM FROM TIA-232 CHANNEL D
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           ||||||+> entry value
;	           |||||+> 0
;	           |||++> entry value
;	           ||+> 1
;	           ++> entry value
;
;	Example: JSR GETCHD
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchd   clc                   ;assume datum is available
         php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldx !#nxpchand        ;channel index
         bra getchcom          ;goto common function
;
;================================================================================
;
;putchd: WRITE DATUM TO TIA-232 CHANNEL D
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHD
;
;	Notes: 1) This function will block if the CFIFO is full.
;	
;
putchd   cli                   ;IRQs must be enabled
         php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldx !#nxpchand        ;channel D index
         bra putchcom
;
;================================================================================
;
;putchc: WRITE DATUM TO TIA-232 CHANNEL C
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHC
;
;	Notes: 1) This function will block if the CFIFO is full.
;	
;
putchc   cli                   ;IRQs must be enabled
         php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldx !#nxpchanc        ;channel C index
         bra putchcom
;
;================================================================================
;
;putchb: WRITE DATUM TO TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHB
;
;	Notes: 1) This function will block if the CFIFO is full.
;	
;
putchb   cli                   ;IRQs must be enabled
         php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldx !#nxpchanb        ;channel B index
         bra putchcom
;
;================================================================================
;
;putcha: WRITE DATUM TO TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHA
;
;	Notes: 1) This function will block if the CFIFO is full.
;	
;
putcha   cli                   ;IRQs must be enabled
         php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldx !#nxpchana       ;channel A index
;
;================================================================================
;
;putchcom: WRITE DATUM TO TIA-232 CHANNEL (common code)
;
;	
;	Do not directly call this function, as the stack will become unbalanced.
;	Only use the PUTCHx functions, where x is the comm port ID.
;	
;
putchcom phy                   ;preserve...
         pha                   ;remaining registers
         phd
;
;
.dpreg   =1                    ;DP
.areg    =.dpreg+s_mpudpx      ;.C
.yreg    =.areg+s_word         ;.Y
.xreg    =.yreg+s_word         ;.X
.sreg    =.xreg+s_word         ;SR
;
;
         pea #kerneldp
         pld                   ;set kernel direct page
         txa                   ;get channel index
         asl                   ;becomes...
         tax                   ;channel offset
         shortr
         lda tiaputtx,x        ;TxQ "put" index
         inc                   ;bump it &...
         and #m_tiaptr         ;mask to queue size
         ora tiaqpmsk,x        ;align to queue base addr
;
.main    cmp tiagettx,x        ;room in TxQ?
         beq .fifoful          ;no, wait
;
         tay                   ;save queue pointer
         lda .areg,s           ;get datum &...
         sta (tiaputtx,x)      ;put in queue
         sty tiaputtx,x        ;save new queue pointer
         lda tiatstab,x        ;channel's TxD status mask
         trb tiatxst           ;transmitter running?
         bne .enabltx          ;no, enable it
;
.done    longr
         pld                   ;restore MPU status
         pla
         ply
         plx
         plp
         clc
         rts
;
;
;	enable transmitter...
;
.enabltx lda #nxpcrtxe
         sta (tiacr,x)
         bra .done
;
;
;	block for TxQ space...
;
.fifoful wai                   ;sleep &...
         bra .main             ;try again
;
;================================================================================
;
;getnvr: GET DATA FROM NVRAM
;
;	
;	Preparatory Ops: .A: number of bytes to get (0=256)
;	                 .X: storage location LSB
;	                 .Y: storage location MSB
;
;	Returned Values: entry values
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> not defined
;
;	Example: lda #20        ;bytes to get
;	         ldx #<buf      ;storage LSB
;	         ldy #>buf      ;storage MSB
;	         jsr getnvr
;	
;
getnvr   shortr
         pha
         phy
         phx
;
;
.dest    .= 1                  ;destination address
;
;
         tax                   ;bytes to fetch
         jsr setautoi          ;set up autoincrement
         ldy #0
         sty io_rtc+wr_nvra    ;set starting address
;
.0000010 lda io_rtc+wr_nvrd    ;fetch from RTC NVRAM
         sta (.dest,s),y       ;store in RAM
         iny
         dex
         bne .0000010
;
         bra putnvraa          ;done
;
;================================================================================
;
;putnvr: PUT DATA IN NVRAM
;
;	
;	Preparatory Ops: .A: number of bytes to put (0=256)
;	                 .X: source location LSB
;	                 .Y: source location MSB
;
;	Returned Values: entry values
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> not defined
;
;	Calling Example: lda #20        ;bytes to store
;	                 ldx #<buf      ;source LSB
;	                 ldy #>buf      ;source MSB
;	                 jsr putnvr
;	
;
putnvr   pha
         phy
         phx
;
;
.src     =1
;
;
         tax                   ;bytes to store
         jsr setautoi          ;set up autoincrement
         ldy #0
         sty io_rtc+wr_nvra    ;set starting address
;
.0000010 lda (.src,s),y        ;fetch from RAM
         sta io_rtc+wr_nvrd    ;store in RTC NVRAM
         iny
         dex
         bne .0000010
;
putnvraa plx
         ply
         pla
         rts
;
;================================================================================
;
;chanbctl: TIA-232 CHANNEL B RECEIVER CONTROL
;
;	
;	Preparatory Ops : see programming example
;
;	Register Returns: .A: used
;	                  .B: entry value
;	                  .X: entry value
;	                  .Y: entry value
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> entry value
;	           ||||||+> 0
;	           |||+++> entry values
;	           ||+> 1
;	           |+> entry value
;	           +> undefined
;
;	Examples: clc              ;disable
;	          jsr chanbctl
;
;	          sec              ;enable
;	          jsr chanbctl
;	
;
chanbctl shorta
         bcc disabchb          ;disable
;
;================================================================================
;
;enabchb: ENABLE DUART CHANNEL B RECEIVER
;
enabchb  lda #nxpcrrsa|nxpcrrxe ;assert RTS & enable RxD
         .byte bitabs
;
;================================================================================
;
;disabchb: DISABLE DUART CHANNEL B RECEIVER
;
disabchb lda #nxpcrrsd|nxpcrrxd;disable RxD & deassert RTS
;
;================================================================================
;
;chbctl: DUART CHANNEL B CONTROL
;
chbctl   sta io_quart+nx_crb   ;write to command register
         wai
         rts
;
;================================================================================
;
;setautoi: SET UP RTC NVRAM AUTOINCREMENT
;
setautoi lda #d11aimsk
         tsb io_rtc+wr_crb     ;enable NVRAM autoincrement
         rts
;
	.end