;================================================================================
;
;hrst: SYSTEM RESET & POST
;
hrst     sei                   ;no IRQs during 1st stage POST
         sec                   ;select 'C02 mode to...
         xce                   ;default MPU state
         clc
         xce                   ;select '816 mode
;
;
;	destructively test critical RAM areas...
;
         lda #testpata         ;1st test pattern
         ldx #0
;
.0000010 sta mm_ram,x          ;write direct page
         sta absolute,x        ;write kernel page
         sta hwstackb,x        ;write stack
         inx
         bne .0000010          ;next
;
         ldy #0
;
.0000020 inx                   ;waste time to see if...
         bne .0000020          ;RAM holds
;
         iny
         bne .0000020
;
.0000030 cmp mm_ram,x          ;check for proper pattern
         beq .0000050          ;we're good so far
;
.0000040 stp                   ;fatal, halt MPU...
;
;	
;	In the event testing fails the only thing that can be done is to
;	halt the MPU.   It isn't possible to report anything to the user
;	because I/O isn't operable this early in the testing.
;	
;
.0000050 cmp absolute,x        ;check kernel page
         bne .0000040          ;error
;
         cmp hwstackb,x        ;check stack
         bne .0000040          ;error
;
         inx
         bne .0000030          ;next location
;
         cmp #testpatb         ;1st or 2nd test done?
         beq .0000060          ;2nd
;
         lda #testpatb         ;2nd test pattern
         bra .0000010          ;run 2nd test
;
.0000060 stz mm_ram,x          ;clear direct page
         stz absolute,x        ;clear vector page
         inx
         bne .0000060
;
;
;	set up stack & indirect vectors...
;
         longa
         lda !#hwstack         ;top of stack
         tcs                   ;set it
         shorta
         ldx #s_vectab-1       ;vector table size
;
.0000065 lda ivectab,x         ;copy vectors
         sta ivectors,x
         dex
         bpl .0000065
;
;
;	set up TIA-232 DP pointer array...
;
         ldx #tiaipend-tiaiptab-1
;
.0000070 lda tiaiptab,x        ;copy data table to...
         sta tiazpptr,x        ;DP pointer array
         dex
         bpl .0000070
;
;
;	configure RTC...
;
         ldy #s_d11tab-s_word  ;RTC setup table index
;
.0000080 ldx d11tab,y          ;get register
         lda d11tab+s_byte,y   ;get parameter
         sta io_rtc,x          ;write to register
         decy s_word
         bpl .0000080          ;next
;
         lda io_rtc+wr_mon     ;ensure that RTC...
         and #dlleomsk         ;oscillator is running
         ora #d11sqmsk         ;also disable...
         sta io_rtc+wr_mon     ;square wave output
;
;
;	configure DUARTs...
;
         ldy #s_nxptab-s_word  ;main setup table index
;         
.0000090 ldx nxpsutab,y        ;get register
         lda nxpsutab+s_byte,y ;get parameter
         sta io_sioa,x         ;write to DUART #1
         sta io_siob,x         ;write to DUART #2
         decy s_word
         bpl .0000090          ;next register
;
         ldy #s_nxptax-s_word  ;auxiliary setup table index
;
.0000100 ldx nxpsxtab,y        ;register
         lda nxpsxtab+s_byte,y ;parameter
         sta io_sioa,x         ;write to register
         decy s_word
         bpl .0000100          ;next register
;
         lda #nxpairq|nxpbirq  ;enable DUART #2...
         sta io_siob+nx_imr    ;channel IRQs
;
;
;	prepare for POST...
;
         lda #hz               ;initialize...
         sta jiffct            ;jiffy IRQ counter
         lda io_quart+nx_scta  ;start...
         cli                   ;jiffy IRQs
         wai                   ;wait for 1st IRQ &...
         printf si_tmint       ;initialize console
         jsr td2sec            ;recovery time
;
;
;	display POST banner...
;
         drawrect postxdim,postydim,postrecx,postrecy
         plot postsepx,postsepy
         printf dc_g6          ;print left tee
         drawhlin psghllen
         printf dc_g7          ;print right tee
         printf si_bannr       ;"POC SBC..."
         jsr td2sec
;
;
;	non-destructively size RAM...
;
         printf si_memt        ;"Sizing Memory..."
         longx                 ;use 16 bit index
         ldx !#rambot           ;start of page 2
;
hrst01   lda mm_ram,x          ;get byte &...
         xba                   ;protect it
         lda #testpata         ;1st test pattern
         sta mm_ram,x          ;write it
         .rept 10              ;decay time
             nop
         .endr
         cmp mm_ram,x          ;check location
         bne hrst05            ;failed
;
         lda #testpatb         ;2nd test pattern
         sta mm_ram,x
         .rept 10              ;decay time
             nop
         .endr
         cmp mm_ram,x
         bne hrst05            ;failed
;
         xba                   ;recover & restore...
         sta mm_ram,x          ;old content
         .rept 10              ;decay time
             nop
         .endr
         cmp mm_ram,x          ;verify
         bne hrst05            ;failed
;
         inx                   ;next location
         txa                   ;test LSB of next location
         bne .0000010          ;not time to update POST
;
         jsr dpyramct          ;update RAM count
;
.0000010 cpx #hwstackb         ;reach bottom of stack?
         bcc hrst01            ;no, keep testing
;
         ldx !#mm_io            ;include MPU stack in...
         jsr dpyramct          ;final RAM count, since...
         shortr                ;it was already tested
         jsr td2sec
         printf si_clver       ;clear version row
;
;
;	configure SCSI subsystem & attempt ISL...
;
         jsr scsiinit          ;configure SCSI subsystem
         bcs hrst04            ;host adapter unresponsive
;
         jsr scsihdrd          ;display enumeration header
         jsr td2sec
         jsr scsienum          ;enumerate devices
         bcc isl               ;ready, attempt ISL
;
hrst02   pea #sd_nodev         ;"no SCSI devices..."
;
hrst03   jsr sprint            ;print status messages
;
hrst04   jmp mon               ;start up M/L monitor
;
;
;	memory test error...
;
hrst05   jsr dpyramct          ;try to display error point...
         shortr
         printf si_ramer       ;"Sizing Error..."
;
.0000010 wai
         bra .0000010          ;loop MPU forever
;
;================================================================================
;
;INITIAL SYSTEM LOAD
;
isl      jsr td2sec
         shorta
         longx
         lda #bootsid          ;default boot device...
;
;	
;	Eventually we'll use the boot device defined in NVRAM.
;	
;
         ldx !#cdblkget        ;CDB address
         ldy !#bootblk         ;boot block buffer
         jsr scsicmd           ;get boot block
         bcs hrst04            ;error, go to monitor
;
;
;	check MBR for validity...
;
         ldx #s_islmag-1       ;magic number size
;
.0000010 lda bootblk+mb_magic,x;MBR magic number
         cmp magicisl,x        ;BIOS magic number
         bne .0000020          ;no match, abort ISL
;
         dex
         bpl .0000010          ;next
;
         jmp (bootblk)         ;run master boot code
;
.0000020 pea #si_noos          ;"disk not bootable"
         bra hrst03            ;abort ISL attempt
;
	.end