;================================================================================
;
;MACHINE LANGUAGE MONITOR  COLD START
;
mon      shortr
         pea #kerneldp
         pld                   ;set kernel direct page
         ldx #vopsflag-reg_pbx
;
.0000010 stz reg_pbx,x         ;clear working storage
         dex
         bpl .0000010
;
;
;	initialize shadow register storage...
;
         lda #srinit
         sta reg_srx           ;status register
         lda #kerneldb
         pha
         plb                   ;set kernel data bank
         sta reg_dbx           ;set DB shadow register
         longa                 ;16 bit .A
         lda !#hwstack         ;top of hardware stack
         tcs                   ;set SP
         tdc
         sta reg_dpx           ;set DP shadow register
;
;
;	announce presence...
;
         pea #mm_entry          ;"*MON"
         bra moncom
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR  SOFTWARE INTERRUPT INTERCEPT
;
;	
;	This is the entry point taken when a BRK instruction is executed.  It is
;	assumed that the BRK  handler has pushed the registers to the stack that
;	are not automatically pushed by the MPU in response to BRK.
;
;	COP is pointed here for now.
;	
;
monbrk   longr                 ;store 16 bit registers
         ply                   ;recover registers
         plx
         pla
         sta reg_ax            ;.C
         stx reg_xx            ;.X
         sty reg_yx            ;.Y
         pea #kerneldp
         pld                   ;set kernel direct page
         shortx                ;8 bit index registers
         pla                   ;get DP &...
         sta reg_dpx           ;store
         plx                   ;get DB &...
         stx reg_dbx           ;store
         plx                   ;get SR &...
         stx reg_srx           ;store
         pla                   ;get PC &...
         sta reg_pcx           ;store
         plx                   ;get PB &...
         stx reg_pbx           ;store
         pea #mm_brk           ;"*BRK"
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR  COMMON ENTRY POINT
;
;	
;	This entry point is common to both the cold start & software interrupt
;	intercept startups.  It prints either *MON or *BRK, stores the current
;	stack pointer, sets up the status register & falls through to the reg-
;	ister display.
;	
;
moncom   cli                   ;make sure IRQs are enabled
         jsr sprint            ;print entry banner
         longa
         tsc                   ;get SP &...
         sta reg_spx           ;store
         rep #%11111110        ;clear SR
         sec                   ;see next
;
;================================================================================
;
;DISPLAY MPU REGISTERS
;
monreg   bcs .0000010          ;okay to proceed
;
         jmp monerr            ;error if called with a parm
;
.0000010 printf mm_regs        ;display register heading
;
;
;	display program bank & counter...
;
         shortr
         jsr dpypbr            ;display PB
         jsr printspc          ;inter-field space
         longa
         lda reg_pcx
         shorta
         jsr dpyhexw           ;display PC
         ldx #2
         jsr multspc           ;inter-field spacing
;
;
;	display SR in bitwise fashion...
;
         ldx reg_srx           ;SR
         ldy #s_bibyte         ;bits in a byte
;
.0000020 txa                   ;remaining SR bits
         asl                   ;grab one of them
         tax                   ;save remainder
         lda #0                ;a clear bit but...
         adc #'0'              ;adjust if set &...
         jsr putcha            ;print
         dey                   ;bit processed
         bne .0000020          ;do another
;
;
;	display .C, .X, .Y, SP & DP...
;
.0000030 jsr printspc          ;spacing
         longa
         lda reg_ax,y          ;get register value
         shorta
         jsr dpyhexw           ;convert & display
         incy s_word
         cpy #reg_dbx-reg_ax
         bcc .0000030          ;next
;
;
;	display DB & IRQ vector...
;
         jsr printspc          ;more spacing
         lda reg_dbx           ;get DB &...
         jsr dpyhex            ;display it
         jsr printspc          ;more spacing
         longa
         lda ivirq             ;IRQ vector
         shorta
         jsr dpyhexw           ;convert & display
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR  COMMAND EXECUTIVE
;	
monce    shorta
         lda #0                ;default buffer index
;
moncea   longx
         ldx reg_spx           ;rebalance the...
         txs                   ;stack
         shortr
         sta ibufidx           ;(re)set buffer index
         printf mm_prmpt       ;display input prompt
;
monceb   jsr input             ;wait for input
;
.0000010 jsr getcharc          ;read from buffer
         beq monce             ;terminator, just loop
;
         cmp #a_blank
         beq .0000010          ;strip leading blanks
;
         ldx #n_mpctab-1       ;number of primary commands
;
.0000020 cmp mpctab,x          ;search primary command list
         bne .0000040
;
         txa                   ;get index
         asl                   ;double for offset
         tax
         longa
         lda mpcextab,x        ;command address -1
;
.0000030 pha                   ;prime the stack
         shorta
         jmp getparm           ;evaluate parm & execute command
;
.0000040 dex
         bpl .0000020          ;continue searching primary commands
;
         ldx #n_radix-1        ;number of radices
;
.0000050 cmp radxtab,x         ;search conversion command list
         bne .0000060
;
         jmp monenv            ;convert & display parameter
;
.0000060 dex
         bpl .0000050
;
         cmp #scsipre          ;SCSI command preamble?
         bne monerraa          ;no, unknown command
;
.0000070 jsr getcharc          ;read next char from buffer
         beq monerraa          ;no char  error
;
         cmp #a_blank
         beq .0000070          ;ignore blanks
;
         ldx #n_spctab-1       ;number of SCSI commands
;
.0000080 cmp spctab,x          ;search command table
         bne .0000090
;
         txa                   ;get index
         asl                   ;double for offset
         tax
         longa
         lda spcextab,x        ;command address -1
         bra .0000030          ;eval parm & execute command
;
.0000090 dex
         bpl .0000080
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR  COMMON ERROR HANDLER
;
monerr   shortr                ;8 bit registers
;
monerraa jsr dpyerr            ;indicate an error
         bra monce             ;return to input loop
;
;================================================================================
;
;MACHINE LANGUAGE MONITOR  CLEAR CONSOLE SCREEN
;
;	
;	syntax: z
;	
;
moncls   bcc monerr            ;no parameters allowed
;
         printf dc_cs          ;clear console screen
         lda #'.'              ;prompt
         jsr putcha            ;print it
         stz ibufidx           ;reset buffer index
         bra monceb            ;return to input loop
;
	.end
