;================================================================================
;
;calccnt: COMPUTE BYTE COUNT FROM ADDRESS RANGE
;
calccnt  jsr clrfacb           ;clear accumulator
         longa
         sec
         lda addrb             ;ending address
         sbc addra             ;starting address
         sta facb              ;byte count
         shorta
         lda addrb+s_word      ;handle banks
         sbc addra+s_word
         sta facb+s_word
         rts
;
;================================================================================
;
;clroper: CLEAR OPERAND
;
clroper  ldx #s_oper-1
;
.0000010 stz operand,x
         dex
         bpl .0000010
;
         stz eopsize
         rts
;
;================================================================================
;
;cpfwsup: FOWARD COPY MEMORY SETUP
;
cpfwsup  longr
         ldx !#opc_mvn         ;"move next" opcode
         bra cpsup
;         
;================================================================================
;
;cprvsup: REVERSE COPY MEMORY SETUP
;
cprvsup  longr
         ldx !#opc_mvp         ;"move previous" opcode
;         
;================================================================================
;
;cpsup: COPY MEMORY SETUP COMMON CODE
;
cpsup    pha                   ;save banks
         txa                   ;protect...
         xba                   ;opcode
         shorta
         ldx !#cpcodeee-cpcode-1
;
.0000010 lda cpcode,x          ;transfer copy code to...
         sta mcftwork,x        ;to workspace
         dex
         bpl .0000010
;
         xba                   ;recover opcode &...
         sta mcftopc           ;set it
         longa
         pla                   ;get banks &...
         sta mcftbnk           ;set them
         rts
;
;================================================================================
;
;decdcnt: DECREMENT DUMP COUNT
;	
;	Preparatory Ops: bytes to process in FACB
;	                 bytes processed in CHARCNT
;
;	Returned Values: .A: used
;	                 .X: entry value
;	                 .Y: entry value
;	                 .C: 1 = count = zero
;	
;
decdcnt  longa
         lda facb+s_word       ;count MSW
         and !#%11111111       ;get rid of MSB
         sec
         ora facb              ;count LSW
         beq .0000020          ;zero, just exit
;
         lda facb
         sbc charcnt           ;bytes processed
         sta facb
         shorta
         lda facb+s_word
         sbc #0                ;handle borrow
         bcc .0000010          ;underflow
;
         sta facb+s_word
         clc                   ;count > 0
         rts
;
.0000010 sec
;
.0000020 shorta
         rts
;
;================================================================================
;
;enddest: GET 2ND & 3RD ADDRESSES FOR COMPARE & TRANSFER
;
enddest  jsr facasize          ;check start...
         cmp #s_dword          ;for range
         bcs .0000010          ;out of range  error
;
         jsr facaddra          ;store start
         jsr getparm           ;get end
         bcs .0000010          ;not entered  error
;
         jsr facasize          ;check end...
         cmp #s_dword          ;for range
         bcs .0000010          ;out of range  error
;
         jsr facaddrb          ;store end
         jsr getparm           ;get destination
         bcs .0000010          ;not entered  error
;
         jsr facasize          ;check destination...
         cmp #s_dword          ;for range
         bcc facaoper          ;store dest address
;
.0000010 rts                   ;exit w/error
;
;================================================================================
;
;facaddra: COPY FACA TO ADDRA
;
facaddra ldx #s_wword-1
;
.0000010 lda faca,x
         sta addra,x
         dex
         bpl .0000010
;
         rts
;
;================================================================================
;
;facaddrb: COPY FACA TO ADDRB
;
facaddrb ldx #s_wword-1
;
.0000010 lda faca,x
         sta addrb,x
         dex
         bpl .0000010
;
         rts
;
;================================================================================
;
;facaoper: COPY FACA TO OPERAND
;
facaoper ldx #s_oper-1
;
.0000010 lda faca,x
         sta operand,x
         dex
         bpl .0000010
;
         rts
;
;================================================================================
;
;facasize: REPORT OPERAND SIZE IN FACA
;
;	
;	Preparatory Ops: operand in FACA
;
;	Returned Values: .A: s_byte  (1)
;	                     s_word  (2)
;	                     s_xword (3)
;	                     s_dword (4)
;	                 .X: entry value
;	                 .Y: entry value
;
;	Notes: 1) This function will always report
;	          a non-zero result.
;	
;
facasize shortr
         ldx #s_dword-1
;
.0000010 lda faca,x            ;get byte
         bne .0000020          ;done
;
         dex
         bne .0000010          ;next byte
;
.0000020 inx                   ;count=index+1
         txa
         rts
;
;================================================================================
;
;getparm: EVALUATE A PARAMETER
;
;	
;	Preparatory Ops: null-terminated input in IBUFFER
;
;	Returned Values: .A: chars in converted parameter
;	                 .X: used
;	                 .Y: entry value
;	                 .C: 1 = no parameter entered
;	
;
getparmr dec ibufidx           ;reread previous char
;
getparm  phy                   ;preserve
         jsr ascbin            ;convert parameter to binary
         bcs .0000040          ;conversion error
;
         jsr getcharr          ;reread last char
         bne .0000010          ;not end-of-input
;
         dec ibufidx           ;reindex to terminator
         lda charcnt           ;get chars processed so far
         beq .0000030          ;none
;
         bne .0000020          ;some
;
.0000010 cmp #a_blank          ;recognized delimiter
         beq .0000020          ;end of parameter
;
         cmp #','              ;recognized delimiter
         bne .0000040          ;unknown delimter
;
.0000020 clc
         .byte bitzp           ;skip SEC below
;
.0000030 sec
         ply                   ;restore
         lda charcnt           ;get count
         rts                   ;done
;
.0000040 jmp monerr            ;abort w/error
;
;================================================================================
;
;nxtaddra: TEST & INCREMENT WORKING ADDRESS 'A'
;
;	
;	Calling syntax: JSR NXTADDRA
;
;	Exit registers: .A: used
;	                .B: used
;	                .X: entry value
;	                .Y: entry value
;	                DB: entry value
;	                DP: entry value
;	                PB: entry value
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    |||||||+> 0: ADDRA < ADDRB
;	                    |||||||      1: ADDRA >= ADDRB
;	                    ||||||+> undefined
;	                    |||+++> entry value
;	                    ||+> 1
;	                    ++> undefined
;	
;
nxtaddra shorta
         lda addra+s_word      ;bits 16-23
         cmp addrb+s_word
         bcc incaddra          ;increment
;
         bne .0000010          ;don't increment
;
         longa
         lda addra             ;bits 0-15
         cmp addrb             ;condition flags
         shorta
         bcc incaddra          ;increment
;
.0000010 rts
;
;================================================================================
;
;getbyte: GET A BYTE FROM MEMORY
;
getbyte  lda [addra]           ;get a byte & fall through
;
;================================================================================
;
;incaddra: INCREMENT WORKING ADDRESS 'A'
;
;	
;	Calling syntax: JSR INCADDRA
;
;	Exit registers: .A: entry value
;	                .B: entry value
;	                .X: entry value
;	                .Y: entry value
;	                DB: entry value
;	                DP: entry value
;	                PB: entry value
;	                SR: NVmxDIZC
;	                    ||||||||
;	                    ++++++++> entry value
;	
;
incaddra php
         longa
         inc addra             ;bump bits 0-15
         bne .0000010
;
         shorta
         inc addra+s_word      ;bump bits 16-23
;
.0000010 plp
         rts
;
;================================================================================
;
;instdata: GET INSTRUCTION SIZE & ADDRESSING MODE DATA
;
;	
;	Preparatory Ops: .X: 65C816 opcode
;
;	Returned Values: .A: mode flags
;	                 .X: operand size
;	                 .Y: mode index
;	
;
instdata shortr
         lda mnetabam,x        ;addressing mode data
         pha                   ;save mode flag bits
         pha                   ;save size data
         and #amodmask         ;extract mode index &...
         tay                   ;save
         pla                   ;recover data
         and #opsmask          ;mask mode fields &...
         .rept n_opslsr        ;extract operand size
           lsr
         .endr
         tax                   ;operand size
         pla                   ;recover mode flags
         and #vopsmask         ;discard mode & size fields
         rts
;
;================================================================================
;
;offtarg: CONVERT BRANCH OFFSET TO TARGET ADDRESS
;
;	
;	Preparatory Ops:    ADDRA: base address
;	                 INSTSIZE: instruction size
;	                  OPERAND: offset
;
;	Returned Values:  OPERAND: target address (L/H)
;	                       .A: used
;	                       .X: entry value
;                              .Y: entry value
;	
;
offtarg  longa
         lda addra             ;base address
         shorta
         lsr instsize          ;bit 0 will be set if...
         bcs .0000010          ;a long branch
;
         bit operand           ;short forward or backward?
         bpl .0000010          ;forward
;
         xba                   ;expose address MSB
         dec                   ;back a page
         xba                   ;expose address LSB
;
.0000010 longa
         clc
         adc operand           ;calculate target address
         sta operand           ;new operand
         shorta
         lda #s_wword
         sta instsize          ;effective instruction size
         rts
;
;================================================================================
;
;setxaddr: SET EXECUTION ADDRESS
;
setxaddr bcs .0000010          ;no address given
;
         jsr facasize          ;check address...
         cmp #s_dword          ;range
         bcs .0000020          ;out of range
;
         longa
         lda faca              ;execution address
         sta reg_pcx           ;set new PC value
         shorta
         lda faca+s_word
         sta reg_pbx           ;set new PB value
;
.0000010 clc                   ;no error
;
.0000020 rts
;
;================================================================================
;
;targoff: CONVERT BRANCH TARGET ADDRESS TO BRANCH OFFSET                   
;
;	
;	Preparatory Ops:   ADDRA: instruction address
;	                 OPERAND: target address
;
;	Returned Values: OPERAND: computed offset
;	                      .A: effective operand size
;	                      .X: entry value
;	                      .Y: entry value
;	                      .C: 1 = branch out of range
;
;	Execution notes: ADDRB is set to the branch base
;	                 address.
;	
;
targoff  stz instsize+s_byte   ;always zero
         lda instsize          ;instruction size will tell...
         lsr                   ;if long or short branch
;
;
.btype   =facc+5               ;branch type flag
;
;
         ror .btype            ;set branch type...
;
;	x0000000
;	|
;	+> 0: short
;	             1: long
;
         longa
         clc
         lda addra             ;instruction address
         adc instsize          ;instruction size
         sta addrb             ;base address
         sec
         lda operand           ;target address
         sbc addrb             ;base address
         sta operand           ;offset
         shorta
         bcc .0000040          ;backward branch
;
         bit .btype            ;check branch range
         bmi .0000020          ;long
;
;
;	process short forward branch...
;
         xba                   ;offset MSB should be zero
         bne .0000060          ;it isn't  out of range
;
         xba                   ;offset LSB should be $00-$7F
         bmi .0000060          ;it isn't  out of range
;
.0000010 lda #s_byte           ;final instruction size
         clc                   ;branch in range
         rts
;
;
;	process long forward branch...
;
.0000020 xba                   ;offset MSB should be positive
         bmi .0000060          ;it isn't  branch out of range
;
.0000030 lda #s_word
         clc
         rts
;
;
;	process backward branch...
;
.0000040 bit .btype            ;long or short?
         bmi .0000050          ;long
;
;
;	process short backward branch...
;
         xba                   ;offset MSB should be negative
         bpl .0000060          ;it isn't  out of range
;
         eor #%11111111        ;complement offset MSB 2s 
         bne .0000060          ;out of range
;
         xba                   ;offset LSB should be $80-$FF
         bmi .0000010          ;it is  branch in range
;
         bra .0000060          ;branch out of range
;
;
;	process long backward branch...
;
.0000050 xba                   ;offset MSB should be negative
         bmi .0000030          ;it is  branch in range
;
.0000060 sec                   ;range error
         rts
;
;================================================================================
;
;getcharr: GET PREVIOUS INPUT BUFFER CHARACTER
;
getcharr dec ibufidx           ;move back a char
;
;================================================================================
;
;getchar: GET A CHARACTER FROM INPUT BUFFER
;
;	
;	Preparatory Ops : none
;
;	Register Returns: .A: character or <NUL>
;	                  .B: entry value
;	                  .X: entry value
;	                  .Y: entry value
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> entry value
;	           ||||||+> 1: <NUL> gotten
;	           ++++++> entry value
;	
;
getchar  phx
         phy
         php                   ;save register sizes
         shortr                ;force 8 bits
         ldx ibufidx           ;buffer index
         lda ibuffer,x         ;get char
         inc ibufidx           ;bump index
         plp                   ;restore register widths
         ply
         plx
         pha                   ;condition...
         pla                   ;.Z
         rts
;
;================================================================================
;
;getpat: GET PATTERN FOR MEMORY CHANGE or SEARCH
;
;	
;	Preparatory Ops: Null-terminated pattern in IBUFFER.
;
;	Returned Values: .A: used
;	                 .X: used
;	                 .Y: pattern length if entered
;	                 .C: 0 = pattern valid
;	                     1 = exception
;	                 .N  0 = no pattern entered
;	                     1 = evaluation error
;
;	Notes: 1) If pattern is preceded by "'" the following
;	          characters are interpreted as ASCII.
;	       2) A maximum of S_AUXBUF bytes or chars is
;	          accepted.  Excess input will be discarded.
;	
;
getpat   stz status            ;clear pattern type indicator
         ldy #0                ;pattern index
         jsr getcharr          ;get last char
         beq .0000070          ;EOS
;
         ldx ibufidx           ;current buffer index
         jsr getcharw          ;get next
         beq .0000070          ;EOS
;
         cmp #'''
         bne .0000010          ;not ASCII input
;
         ror status            ;condition flag
         bra .0000030          ;balance of input is ASCII
;
.0000010 stx ibufidx           ;restore buffer index
;
.0000020 jsr getparm           ;evaluate numeric pattern
         bcs .0000060          ;done w/pattern
;
         jsr facasize          ;size
         cmp #s_word
         bcs .0000070          ;not a byte  error
;
         lda faca              ;get byte &...
         bra .0000040          ;store
;
.0000030 jsr getchar           ;get ASCII char
         beq .0000060          ;done w/pattern
;
.0000040 cpy #s_auxbuf         ;pattern buffer full?
         beq .0000050          ;yes
;
         sta auxbuf,y          ;store pattern
         iny
         bit status
         bpl .0000020          ;get next numeric value
;
         bra .0000030          ;get next ASCII char
;
.0000050 jsr alert             ;excess input
;
.0000060 sty auxbufix          ;save pattern size
         tya                   ;condition .Z
         clc                   ;pattern valid
         rts
;
;
;	no pattern entered...
;
.0000070 rep #%10000000
         sec
         rts
;
;
;	evaluation error...
;
.0000080 sep #%10000001
         rts
;
;================================================================================
;
;getcharw: GET FROM INPUT BUFFER, DISCARDING WHITESPACE
;
;	
;	Preparatory Ops: Null-terminated input in IBUFFER.
;
;	Returned Values: .A: char or null
;	                 .X: entry value
;	                 .Y: entry value
;	                 .Z: 1 = null terminator detected
;
;	Notes: Whitespace is defined as a blank ($20) or a
;	       horizontal tab ($09).
;	
;
getcharw jsr getchar           ;get from buffer
         beq .0000010          ;EOI
;
         cmp #' '
         beq getcharw          ;discard whitespace
;
         cmp #a_ht             ;also whitespace
         beq getcharw
;
.0000010 clc
         rts  
;
;================================================================================
;
;input: INTERACTIVE INPUT FROM CONSOLE CHANNEL
;
;	
;	Preparatory Ops: Zero IBUFIDX, or load IBUFFER with default
;	                 input & set IBUFIDX to the number of chars
;	                 loaded into the buffer.
;
;	Returned Values: .A: used
;	                 .X: characters entered
;	                 .Y: used
;
;	Example: STZ IBUFIDX
;	         JSR INPUT
;
;	Notes: Input is collected in IBUFFER & is null-terminated.
;	       IBUFIDX is reset to zero upon exit.
;	
;
input    ldx ibufidx
         stz ibuffer,x         ;be sure buffer is terminated
         jsr dpyibuf           ;print default input if any
         printf dc_cn          ;enable cursor
         ldx ibufidx           ;starting buffer index
;
;
;	main input loop...
;
.0000010 jsr getcha            ;poll for input
         bcs .0000060          ;nothing
;
         cmp #a_del            ;above ASCII range?
         bcs .0000030          ;yes, not allowed
;
         cmp #a_ht             ;horizontal tab?
         bne .0000020          ;no
;
         lda #a_blank          ;replace <HT> w/blank
;
.0000020 cmp #a_blank          ;control char?
         bcc .0000040          ;yes
;
;
;	process QWERTY character...
;
         cpx #s_ibuf           ;room in buffer?
         bcs .0000030          ;no
;
         sta ibuffer,x         ;store char
         inx                   ;bump index
         .byte bitabs          ;echo char
;
.0000030 lda #a_bel            ;alert user
         jsr putcha
         bra .0000010          ;get some more
;
;
;	process carriage return...
;
.0000040 cmp #a_cr             ;carriage return?
         bne .0000050          ;no
;
         phx                   ;protect input count
         printf dc_co          ;cursor off
         plx                   ;recover input count
         stz ibuffer,x         ;terminate input
         stz ibufidx           ;reset buffer index
         rts                   ;done
;
;
;	process backspace...
;
.0000050 cmp #a_bs             ;backspace?
         bne .0000010          ;no
;
         txa
         beq .0000010          ;no input, ignore <BS>
;
         dex                   ;1 less char
         phx                   ;preserve count
         jsr gendbs            ;destructive backspace
         plx                   ;restore count
         bra .0000010          ;get more input
;
;
;	waiting loop...
;
.0000060 wai                   ;wait for datum
         bra .0000010
;
;================================================================================
;
;lodbnk: LOAD SOURCE & DESTINATION BANKS
;
lodbnk   shorta
         lda operand+s_word    ;destination bank
         xba                   ;make it MSB
         lda addra+s_word      ;source bank is LSB
         rts
;         
;================================================================================
;
;getcharc: GET A CHARACTER FROM INPUT BUFFER & CONVERT CASE
;
;	
;	Preparatory Ops: Null-terminated input in IBUFFER.
;
;	Returned Values: .A: char or null
;	                 .X: entry value
;	                 .Y: entry value
;	                 .Z: 1 = null terminator detected
;	
;
getcharc jsr getchar           ;get from buffer
;
;================================================================================
;
;toupper: FORCE CHARACTER TO UPPER CASE
;
;	
;	Preparatory Ops : .A: character to convert
;
;	Register Returns: .A: converted character
;	                  .B: entry value
;	                  .X: entry value
;	                  .Y: entry value
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           ++++++++> entry value
;
;	Notes: 1) Accumulator must be 8 bits on entry.
;	       2) This subroutine has no effect on
;	          characters that are not alpha.
;	
;
toupper  php                   ;protect flags
         cmp #a_asclcl         ;check char range
         bcc .0000010          ;not LC alpha
;
         cmp #a_asclch+s_byte
         bcs .0000010          ;not LC alpha
;
         and #a_lctouc         ;change to UC
;
.0000010 plp                   ;restore flags
         rts
;
;================================================================================
;
;teststop: TEST FOR STOP KEY
;
;	
;	Preparatory Ops: none
;
;	Returned Values: .A: detected keypress, if any
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;	                 DB: entry value
;	                 DP: entry value
;	                 PB: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: normal key detected
;	           |||||||      1: <STOP> detected
;	           +++++++> undefined
;
;	Example: jsr teststop
;	         bcs stopped
;
;	Notes: The symbol STOPKEY defines the ASCII
;	       value of the "stop key."
;	
;
teststop jsr getcha            ;poll console
         bcs .0000010          ;no input
;
         cmp #stopkey          ;stop key pressed?
         beq .0000020          ;yes
;
.0000010 clc
;
.0000020 rts
;
;================================================================================
;
;cpcode: COPY MEMORY CODE
;
;	
;	This code is transfered to workspace when a
;	copy or fill operation is to be performed.
;	
;
cpcode   phb                   ;must preserve data bank
         .rept s_mvinst        ;placeholder  do not delete
           nop
         .endr
         plb                   ;restore data bank
         jmp monce             ;return to command executive
cpcodeee =*                    ;placeholder  do not delete
;
	.end
