;================================================================================
;
;getidlun: GET SCSI ID & LUN
;
;	
;	Preparatory Ops : NONE
;
;	Register Returns: .A: used
;	                  .B: used
;	                  .X: LUN shifted to LUN bit field
;	                  .Y: entry value
;
;	Other returns   : FLIMFLAG: xx000000
;	                            ||
;	                            |+> 0: 21 bit LBAs
;	                            |            1: 32 bit LBAs
;	                            +> 0: block device
;	                                         1: stream device
;	                 
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: okay
;	           |||||||      1: ID or LUN out of range
;	           ||||||+> undefined
;	           |||||+> undefined
;	           ||||+> undefined
;	           |||+> 1
;	           ||+> 1
;	           |+> undefined
;	           +> undefined
;	
;
getidlun jsr facasize          ;check parameter range
         cmp #s_word           ;should be a byte
         bcs .0000020          ;too big  error
;
         lda faca              ;SCSI ID
         sta operand           ;save it
         longx
         ldx !#auxbuf          ;workspace
         jsr scsiparm          ;get device info
         bcs .0000020          ;ID out of range
;
         shortr
         sec
         bit auxbuf            ;device enumerated?
         bpl .0000020          ;no
;
         lda auxbuf+o_sdtype   ;get type field
         tax
         and #%10000000        ;isolate LBA flag bit
         lsr                   ;shift &...
         sta flimflag          ;store
         txa                   ;get type again
         and #sdtmask          ;extract type
         cmp #sd_seq           ;sequential stream device?
         bne .0000010          ;no
;
         lda #%10000000        ;yes
         tsb flimflag
;         
.0000010 jsr getparm           ;get LUN
         bcs .0000020          ;missing LUN  error
;
         jsr facasize          ;check LUN range
         cmp #s_word           ;should be a byte
         bcs .0000020          ;too big  error
;
         lda faca              ;LUN
         cmp #n_scsilu         ;should be 0-7
         bcs .0000020          ;error
;
         .rept n_lunshf        ;shift to LUN field
           asl
         .endr
         tax
;
.0000020 shortr
         rts
;
;================================================================================
;
;setupcdb: SET UP SCSI COMMAND DESCRIPTOR BLOCK
;
;	
;	Preparatory Ops : .A: 8 bit SCSI opcode
;	                  .X: shifted LUN
;
;	Register Returns: .A: used
;	                  .B: entry value
;	                  .X: used
;	                  .Y: entry value
;	                 
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> undefined
;	           ||||||+> undefined
;	           |||||+> undefined
;	           ||||+> undefined
;	           |||+> 1
;	           ||+> 1
;	           |+> undefined
;	           +> undefined
;
;	Notes: 1) AUXBUF is used as the SCSI CDB.
;	
;
setupcdb phx                   ;protect LUN
         ldx #s_cdbg2-1        ;CDB size
;
.0000010 stz auxbuf,x          ;clear CDB workspace
         dex
         bpl .0000010
;
         plx                   ;recover LUN
         sta auxbuf+so_opcod   ;store opcode in CDB
         stx auxbuf+so_lunla   ;store LUN in CDB
         rts
;
;================================================================================
;
;SET UP SCSI READ/WRITE PARAMETERS
;
;	
;	Preparatory Ops:  CDB initialized by SETUPCDB, &
;	                  device & LBA flags conditioned
;
;	Register Returns: .A: used
;	                  .B: used
;	                  .X: used
;	                  .Y: used
;	                 
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: okay
;	           |||||||      1: parameter error
;	           ||||+++> undefined
;	           ||++> 1
;	           ++> undefined
;
;	Notes: 1) If the target is a block device & is 32 bit
;	          capable, a group 2 CDB will be set up.
;	
;
setuprwb jsr getparm           ;get LBA
         bcs .0000020          ;missing LBA
;
         bit flimflag          ;check device type
         longa
         bpl .0000030          ;block (e.g., disk)
;
         lda faca              ;stream device's...
         ora faca+s_word       ;LBA...
         beq .0000050          ;must be zero
;
.0000010 sec                   ;abort w/error
;
.0000020 rts
;
.0000030 longa
         lda faca              ;LBA LSW
         xba                   ;reverse endianess
         bvc .0000040          ;use 21 bit LBA
;
         sta auxbuf+so_lbaml   ;use 32 bit LBA
         lda faca+s_word       ;LBA MSW
         xba                   ;reverse endianess
         sta auxbuf+so_lbamb   ;store into CDB
         bra .0000050
;
.0000040 sta auxbuf+so_lbam    ;use 21 bit LBA
         lda faca+s_word       ;LBA LSB of MSW
         and !#lunmaskr        ;mask LUN bits
         cmp faca+s_word       ;any change?
         bne .0000010          ;yes, bogus LBA
;
         shorta                ;discard MSB of MSW
         tsb auxbuf+so_lunla   ;combine w/LUN
;
.0000050 shorta
         jsr getparm           ;get number of blocks/bytes
         bcs .0000020          ;missing parameter
;
         jsr facasize          ;allowable block range...
         cmp #s_xword          ;0-65535
         bcs setuprex          ;too many blocks/bytes
;
         longa
         lda faca              ;number of blocks/bytes LSW
         xba                   ;reverse endianess
         shorta
         bit flimflag          ;block or stream device?
         longa
         bmi .0000060          ;stream
;
         bvc .0000070          ;short block addressing
;
         sta auxbuf+so_xfrlh   ;long block addressing
         bra .0000080
;
.0000060 sta auxbuf+so_xfrib   ;stream device addressing
         bra .0000080
;
.0000070 shorta                ;short block addressing
         jsr facasize          ;recheck block range
         cmp #s_word           ;0-255
         bcs setuprex          ;too many blocks
;
         lda faca
         beq .0000010          ;zero = 256: invalid
;
         sta auxbuf+so_xfrl
;
.0000080 shorta
;
;
;	get buffer address...
;
setuprAA jsr getparm           ;get address
         bcs setuprex          ;none entered
;
         jsr facasize          ;check...
         cmp #s_wword          ;range
         bcs setuprex          ;out of range
;
         jsr facaddra          ;save address
         jsr getparm           ;check for extra input
         ror                   ;capture carry
         eor #%10000000        ;invert
         asl                   ;put back to carry
;
setuprex rts
;
	.end
