;================================================================================
;
;adjloadr: ADJUST S-RECORD LOAD ADDRESS
;
adjloadr lda pageoff           ;page offset
         xba                   ;move to MSB
         lda #0                ;even page boundary
         longa
         clc
         adc loadaddr          ;record load address
         sta loadaddr          ;new load address
         shorta
         rts
;
;================================================================================
;
;clrchanb: CLEAR EXCESS INPUT FROM AUXILIARY CHANNEL
;
clrchanb wai                   ;wait for an IRQ
         jsr getchb            ;read from channel
         bcc clrchanb          ;keep clearing
;
         rts
;
;================================================================================
;
;getchanb: GET DATUM FROM AUXILIARY CHANNEL W/STOP TEST
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or null
;	                 .B: used
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: .A is valid
;	           |||||||      1: user aborted
;	           ||||||+> 0: datum in .A
;	           ||||||       1: EOR detected
;	           ||||++> undefined
;	           ||++> 1
;	           ++> undefined
;
;	Notes: 1) This sub will loop until a datum is gotten or the stop key
;	          is detected on the console channel.
;
;	       2) If <CR>, <LF> or the <CRLF> combination is encountered in
;	          the input stream, $00 will be substituted.  <LFCR> is not
;	          a synonym for <CRLF> & if encountered, may cause process-
;	          ing errors.
;	
;
getchanb shortr                ;8 bit loads
         lda #0
         xba                   ;flush B-accumulator
;
.0000010 jsr teststop          ;check for user abort
         bcs .0000040          ;aborted
;
         jsr getchb            ;get datum
         bcs .0000050          ;none
;
         cmp #a_nul            ;null received?
         beq .0000010          ;yes, discard
;
         cmp #a_lf             ;<LF>?
         beq .0000020          ;yes, EOR delimiter
;
         cmp #a_cr             ;<CR>?
         bne .0000030          ;no
;
         xba                   ;look at previous datum
         cmp #a_cr             ;was it a <CR> as well?
         bne .0000010          ;no, wait for next datum
;
.0000020 lda #0                ;end of record
;
.0000030 clc                   ;valid datum
;
.0000040 rts
;
.0000050 wai                   ;wait for any interrupt &...
         bra .0000010          ;check for data
;
;================================================================================
;
;getcvt: GET & CONVERT S-RECORD BYTE PAIR
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: byte converted from ASCII pair
;	                 .X: entry value
;	                 .Y: entry value
;	                 .C: 0: conversion successful
;	                     1: conversion error or premature EOT
;	
;
getcvt   shortr
         jsr getchar           ;get MSN char
         beq .0000010          ;premature EOT
;
         jsr nybtobin          ;convert to a nybble
         bcs .0000020          ;conversion error
;
         .rept s_bnybbl        ;shift to MSN position
           asl
         .endr
         sta datax             ;store MSN
         jsr getchar           ;get LSN char
         beq .0000010          ;premature EOT
;
         jsr nybtobin
         bcs .0000020          ;error
;
         ora datax             ;combine with MSN
         rts                   ;done
;
.0000010 sec                   ;conversion error
;
.0000020 rts
;
	.end
