;===============================================================================
;
;scsiparm: RETURN SCSI DEVICE PARAMETERS FROM ENUMERATION TABLE
;
;	
;	Preparatory Ops : .A: 8 bit SCSI ID
;	                  .X: workspace address (1)
;
;	Register Returns: .A: used
;	                  .B: used
;	                  .X: entry value
;	                  .Y: used
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;	                 
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: okay
;	           |||||||      1: SCSI ID out of range
;	           ||||||+> undefined
;	           |||||+> entry value
;	           |||++> 0
;	           ||+> 1
;	           ++> undefined
;
;	Notes: 1) This function copies the device table parameters
;	          associated with the SCSI ID passed in .A to work-
;	          space pointed to by .X.  The workspace must be
;	          sized to 8 bytes minimum.
;	       2) This function does not check the logged device
;	          information in any way.
;	
;
scsiparm longr                 ;force 16 bits
         phd                   ;save current direct page
         pha                   ;protect ID
         lda !#kerneldp
         tcd                   ;set kernel direct page
         pla                   ;recover ID
         jsr sslotptr          ;get device table pointer
         bcs .0000020          ;ID out of range  error
;
         phx                   ;save workspace pointer
;
;
.wrkspc  =1                    ;workspace pointer stack offset
;
;
         shortx
         ldy #s_sdslot-1       ;device table slot index
;
.0000010 lda (scsididx),y      ;get from device table &...
         sta (.wrkspc,s),y     ;write to workspace
         dey
         bpl .0000010          ;next
;
         longx
         plx                   ;clean up stack
;
.0000020 pld
         rts
;
	.end
