;===============================================================================
;
;sslotptr: GET SCSI DEVICE TABLE SLOT POINTER
;
;	
;	Preparatory Ops : .A: SCSI ID
;	                  .B: "don't care"will be masked
;
;	Returns         : slot pointer in SCSIDIDX if no error
;
;	Register Returns: .A: entry value
;	                  .B: 0
;	                  .X: entry value
;	                  .Y: entry value
;	                 
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: okay
;	           |||||||      1: SCSI ID out of range
;	           ||||||+> undefined
;	           |||||+> entry value
;	           ||||+> 0
;	           |||+> entry value
;	           ||+> 1
;	           ++> undefined
;	
;
sslotptr cld                   ;ensure binary mode
         longa                 ;16 bit .A
         and !#%11111111       ;get rid of noise
         cmp !#n_scsiid        ;check ID
         bcs .0000010          ;out of range  error
;
;	
;	The following code depends on the fact that the device
;	table slot size has been atomically defined to be 2^n.
;	
;
         pha                   ;save device ID	
         .rept sdslotex        ;multiply ID to get index
           asl
         .endr
         adc !#scdevtab        ;add offset to base
         sta scsididx          ;set up pointer
         pla                   ;recover ID
;
.0000010 shorta
         rts
;
	.end
