	.opt proc65816,caseinsensitive,swapbin
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                                                                                 *
;*                 W65C816S PROOF OF CONCEPT SINGLE-BOARD COMPUTER                 *
;*                                                                                 *
;*                          S Y S T E M   F I R M W A R E                          *
;*  *
;*      Copyright 1991-2020 by BCS Technology Limited.  All rights reserved.      *
;*                                                                                 *
;* Permission is hereby granted to use, copy, modify and distribute this software, *
;* provided this copyright notice remains unaltered in the source code and  proper *
;* attribution is given.  Redistribution, in any form, must be at no charge to the *
;* end user.  This code or any part  thereof, including any derivation, MAY NOT be *
;* incorporated into any package intended for sale unless written permission to do *
;* so has been granted by the copyright holder.                                    *
;*  *
;* THERE IS NO WARRANTY OF ANY KIND WITH THIS SOFTWARE.                            *
;*                                                                                 *
;* While it is believed that all code will perform as intended,  the user  assumes *
;* all risk in connection with the incorporation of this software into any system. *
;*                                                                                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;
;	* * * * * * * * * * * *
;	* VERSION INFORMATION *
;	* * * * * * * * * * * *
;
hardvers .macro                ;hardware version
         .byte "1"             ;major (0, 1, 2 ...)
         .byte "."
         .byte "2"             ;minor (0, 1, 2 ...)
         .byte " "             ;revision (blank, a, b, etc.)
         .endm
;
hwdate   .macro                ;hardware version date
         .byte "2020/01/08"    ;YYYY/MM/DD
         .endm
;
copyyear .macro                ;firmware copyright year
         .byte "2020"          ;YYYY
         .endm
;
softvers .macro                ;software version 
         .byte "1"             ;major
         .byte "."
         .byte "11"            ;minor
         .byte "."
         .byte "4a"            ;revision
         .endm
;
romsnum  .macro                ;ROM identification
         .byte "20201125"      ;YYYYMMDD
         .byte "-"
         .byte "01"            ;sequence (01, 02 ...)
         .endm
;
;
;REVISION TABLE
;
;Ver  Date           Revision Description
;
;1.11 2020/02/17  A) Firmware adapted to POC hardware version 1.2 for baseline
;                    testing.  Also, banner graphics rendition changed from
;                    hardware to software in preparation for eventual test of
;                    ANSI/VT100 console emulation.
;
;     2020/03/01  A) SIO driver ISR changed from polling vQUART registers to
;                    reading & analyzing hardware channel interrupt status.
;                 B) Permanently set SIO circular queue size to 64 datums.
;                 C) Detail change in the SIO driver receiver ISR in how it han-
;                    dles the queue pointer.
;                 D) Moved the top-of-stack to $00BFFF & repositioned the SIO
;                    queues below the stack.
;
;     2020/05/10  A) Activated SIO channels C & D.  Timer B not activated at
;                    this time.  Also, there is no path at this time to communi-
;                    cate with channels C & D.
;
;     2020/05/11  A) Added primitives to communicate through SIO channels C & D.
;                 B) Added jump table entries for testing SIO channels C & D.
;                 C) Revised algorithm for computing SIO queue pointers, as the
;                    existing algorithm was incorrect for 4 channels.
;                 D) Changed use of a horizontal tab for separating address col-
;                    umns in the monitor C & H functions to multiple spaces.
;
;     2020/05/17  A) Changed time delay function so less code is executed in the
;	            timekeeping part of the interrupt handler.
;
;     2020/11/28  A) Syntax changes made to work with assembler version 1.3,
;                    which now natively supports the 65C816 assembly language.
;                    Also, edits were made to recognize % as the binary radix
;                    (.OPT SWAPBIN) & to coerce immediate mode operands to 16
;                    bits where applicable.  Firmware version is designated as
;                    1.11.4a for documentation purposes only.  No code changes
;                    have been made in this revision.
;
;1.10 2017/02/21  A) SIO driver rewritten to use static direct page pointers
;                    to access UART registers & circular queues.
;                 B) Code that sets DB taken out of primitives & ISRs, as it has
;                    no effect on POC V1 circuitry.
;
;     2017/03/15  A) Added "HBA present" flag in direct page & modified SCSICMD
;                    function to immediately abort if this flag is not set.
;                 B) Modified SCSI section of IRQ handler to test "HBA present"
;                    flag & skip over code if HBA isn't present.
;
;     2017/05/25  A) Modified SIO ISR to work strictly by channel indexing.
;
;1.9  2015/04/29  A) Jiffy IRQ is now generated by the C/T in the DUART.
;                 B) Deactivated all RTC IRQs.  No reference is made within this
;                    firmware to the RTC's date & time registers.
;
;     2015/05/04  A) Changed interrupt level for UART receivers.
;
;     2015/12/16  A) Added test for null byte in S-record input stream process-
;                    ing.
;                 B) Added code in BIOS calls that assures the correct value is
;                    in DP regardless of what the caller is using for direct
;                    page.
;                 C) SIO queues enlarged to 256 bytes.
;                 D) Default stack pointer relocated to $00CBFF to accommodate
;                    change C above.
;
;     2016/01/01  A) Changed the way in which DP is handled when the monitor is
;                    entered via a BReaK.
;
;     2016/01/17  A) Detail change in SCSICMD that saved a few instructions.
;
;1.8  2015/04/25  A) Changed interrupt priority so RTC IRQs are serviced before
;                    UART IRQs.
;                 B) Made changes to the UART driver to reflect a more universal
;                    approach to handling both DUARTs & QUARTs.
;                 C) Detail change to time delay counter & free running clock
;                    ISR code to improve long-term accuracy & reduce code size.
;                 D) NXP 28L92 is now the assumed UART.  However, new code will
;                    continue to work with the 26C92.
;                 E) UNIX time feature is now permanent.
;
;     2015/04/26  A) Changed the IRQ priority & compacted the IRQ ISR a bit.
;
;1.7  2015/02/25  A) Added experimental UNIX time feature.  New jump table ent-
;                    ries added to get & set the clock value.
;
;     2015/03/26  A) Removed console clock setting code.
;                 B) Moved CHANBCTL jump table slot to occupy slot previously
;                    assigned to console clock setting entry.
;
;1.6  2014/06/02  A) SIO driver rewritten to use channel indexing instead of
;                    redundant code.  This change is a verification of a method
;                    that will be used in future UART drivers.
;
;     2014/06/28  A) Added SCSIHAID (get SCSI host adapter bus ID) function to
;                    jump table.  Jump table realigned to place SCSIHAID after
;                    SCSICMD.
;                 B) Changed the monitor load function to end after processing
;                    an S9 record.  The terminating <EOT> is no longer required.
;
;     2015/01/19  A) Entering the monitor now initializes the status register so
;                    registers are set to 16 bits.
;
;1.5  2013/10/13  A) Restructured GETUTIM to accept a storage address in .X
;                 B) BASINA renamed to GETCHA & BASINB renamed to GETCHB.
;                 C) BSOUTA renamed to PUTCHA & BSOUTB renamed to PUTCHB.
;                 D) SPRINT removed from jump table.
;                 E) Added CHANBCTL (channel-B control) function to jump table.
;                 F) Added IRQ vector display to monitor register dump function.
;                 G) Reorganized INCLUDE files so that BIOS data tables are all
;                    stored in a common subdirectory.
;                 H) General code compaction.
;
;     2013/10/23  A) ALARM function removed & jump table entry deleted.  Jump
;                    table realigned.
;                 B) SIO driver changed to support 26C92 added features.
;                 C) Console & auxiliary ports now running at 115.2Kbps.
;                 D) Monitor now recognizes PEA #<op> and PEI <dp> as syntacti-
;                    cally correct during assembly.
;
;     2013/10/24  A) Change the way in which I/O device setup parameters are
;                    processed during reset.
;
;     2013/11/04  A) Fixed a problem where the B-accumulator wasn't always being
;                    be copied to shadow storage after return from execution of
;                    a J command.
;
;     2014/01/23  A) Detail change to the POST screen following memory test.
;
;     2014/01/31  A) Changed the way in which the 53C94 clock factor & (re)sel-
;                    ection timeout setup values are computed.  The previous
;                    method was not matching the ANSI T10 recommendations in all
;                    cases.
;                 B) Performed some code compaction.
;
;     2014/04/05  A) SPRINT string pointer parameter now passed through stack.
;                 B) Removed SEI & CLI instructions from the SCSI DMA data-in
;                    code (SSXFRIN).
;
;1.4  2013/06/26  A) SSLOTADR removed from BIOS jump table & converted to an in-
;                    ternal function.
;                 B) New function SCSIPARM (get SCSI device parameters) add to
;                    BIOS jump table.
;                 C) Reorganized BIOS jump table.  SSSCMD renamed to SCSICMD but
;                    otherwise unchanged.
;                 D) Made enhancements to the SCSI enumeration algorithm to red-
;                    uce boot time.
;                 E) Reorganized the SCSI subsystem code sections.
;
;     2013/07/03  A) Changed the way SIO transmission is reenabled when data
;                    is written to TxD buffers.
;
;1.3  2013/05/27  A) Added GETUTIM (get uptime) BIOS call.
;                 B) Enlarged monitor auxiliary buffer to accept up to 32 bytes
;                    in a search pattern.
;                 C) Modified the way in which ABT, BRK, COP & NMI enter the M/L
;                    monitor if not revectored.
;
;1.2  2013/01/02  A) Changed the SCSI "test ready" algorithm to reduce the boot
;                    time.
;                 B) Removed the Ctrl-D soft reset function from the monitor.
;
;1.1  2012/09/10  A) Added tape rewind command to M/L monitor.
;                 B) Detail change in the SCSI driver DMA setup to properly han-
;                    dle block size to bytes conversion when a stream device is
;                    being accessed.  Previously, the conversion was being taken
;                    for any read or write operation that accessed the medium.
;
;     2012/09/11  A) Slight code shrink in the SCSI driver DMA setup.
;
;     2012/09/17  A) Enlarged <N> field in monitor SCSI read/write commands to
;                    accept a 16 bit quantity to suppport READ(10) & WRITE(10).
;
;1.0  2012/09/07  A) First production version.
;
;0.9  2012/09/01  A) Changed all programatic references to the TMSTD subroutine
;                    to 2 consecutive WAI instructions, as they produce the same
;                    effect with slightly less code.
;                 B) Consistent with change 'A', removed "common/time.asm" from
;                    INCLUDEs, as it is no longer used.
;                 C) Changed alarm handler so it no longer uses the programmable
;                    delay counter.
;                 D) Modified time delay function to initialize the milliseconds
;                    counter for a more precise delay when short delay times are
;                    programmed.
;
;0.8  2012/08/11  A) Rewrote SCSI driver to use IRQs instead of polling.
;                 B) All SCSI I/O functions are now handled by DMA channel hand-
;                    shakes, except for command initiation.
;
;     2012/08/17  A) Modified IRQ handler to immediately exit after processing a
;                    SCSI interrupt.
;                 B) Modified NMI handler to ignore NMI if generated while any
;                    code in ROM is being accessed.
;
;0.7  2012/06/20  A) Method used to translate between assembly language & mach-
;                    ine instructions rewritten in order to shrink table sizes &
;                    improve execution speed.
;                 B) 53C94 FIFO count test removed from SCSI command execution,
;                    as this test is conducted during SCSI subsystem setup.
;
;     2012/07/25  A) Modified SCSI enumeration to store the device's capacity in
;                    blocks into the device table instead of the maximum LBA.
;
;     2012/08/05  A) Changed the way the BIOS executes the master boot code.
;                 B) Modified the SCSI driver primitive so the alarm time can be
;                    changed to accommodate slow operations like formatting a
;                    disk.  Default alarm time reduced to 1 sec, which is more
;                    than enough for a device-not-present error to be be return-
;                    ed if a target is unresponsive.
;                 C) Modified the test ready part of SCSI enumeration to make
;                    multiple attempts to get device status before abandoning.
;
;     2012/08/06  A) Disabled the alarm function in the SCSI driver primitive.
;
;0.6  2012/05/31  A) SCSI driver rewritten to use DREQ handshake during data-in
;                    & data-out phases.  PIO continues to be used in all other
;                    phases.
;
;     2012/06/05  A) Shrunk SCSI driver code.
;
;     2012/06/17  A) Replaced JMP absl instruction with JML absl to conform to
;                    WDC recommended assembler syntax.
;                 B) Enlarged SCSI enumeration temporary work area to accommod-
;                    ate additional information retrieved during device inquiry
;                    operation.
;
;0.5  2012/04/28  A) Added a static SCSI device table, which is populated during
;                    the SCSI enumeration process.
;                 B) Added the SSLOTADR function, which returns a pointer to the
;                    device table slot associated with a given SCSI ID.
;                 C) BIOS can load a boot block from the boot device, currently
;                    defined as SCSI ID 0, LUN 0, & execute code therein.
;
;     2012/05/10  A) Modified ISRs so data bank & direct page pointer are pushed
;                    as part of the stack frame.
;                 B) Modified monitor G & J commands to accommodate changes to
;                    ISR stack frame structure.
;
;     2012/05/16  A) Added alarm function to BIOS.
;                 B) Using alarm function to break SCSI controller command loop
;                    if no response occurs.
;
;     2012/05/19  A) Restructured the SCSI controller initialization code to a
;                    smaller & more efficient form.
;                 B) Changed the SCSI controller FIFO bit pattern to one that
;                    exercises more bits.
;
;     2012/05/19  A) Relocated SIO buffers higher in RAM & relocated the top
;                    of the MPU stack below the bottom of the last SIO buf-
;                    fer to prevent a collision between the stack & the buffers.
;                 B) Reduced all SIO buffer sizes to 128 bytes.
;
;0.4  2011/10/01  A) Added SCSI initialization functions to reset.
;                 B) Added BIOS jump table entry for SCSI initialization.
;                 C) Added BIOS jump table entry for SCSI command execution.
;                 D) Added SCSI subsystem commands to monitor.
;                 E) Added SCSI device enumeration.
;
;     2012/03/03  A) Restructured NVRAM to include serial number & other data.
;
;0.3  2011/09/15  A) Reset now uses the native mode stack location instead of
;                    the emulation mode stack.
;                 B) The native mode stack is tested prior to enabling IRQs &
;                    I/O activity.
;
;0.2  2011/07/06  A) Converted SIO from polled to interrupt-driven.
;                 B) Modified RAM testing to test the emulation mode stack bef-
;                    ore configuring I/O or starting IRQs.
;
;0.1  2010/08/01  A) ROM BIOS running in the MPU's native mode.
;                 B) Entire BIOS rewritten to use 16 bit code where possible.
;                 C) Monitor now has all relevant functions, including code ass-
;                    embly & disassembly.
;
;0.0  2009/12/24  A) Original ROM BIOS running in the MPU's emulation mode.
;
;
;
;	        COMMENT ABBREVIATIONS
;	
;	 ACIA   asynchronous communication interface adapter, aka UART
;	  BCD   binary-coded decimal
;	  BIN   binary
;	 BIOS   basic input/output subsystem
;	  BRG   baud rate generator
;	  C/T   counter/timer
;	   CD   compact disk
;	  CDA   compact disk drive
;	  CDB   SCSI command descriptor block
;	CFIFO   circular first-in, first-out structure, aka circular queue
;	  CTS   TIA-232 clear to send
;	  DMA   direct memory access
;	   DP   direct page or page zero, also MPU direct page register
;	  DSR   TIA-232 data set ready
;	  DTR   TIA-232 data terminal ready
;	DUART   dual ACIA or dual UART
;	  EOF   end-of-field
;	  EOI   end-of-input
;	  EOM   end-of-medium
;	  EOR   end-of-record
;	  EOT   end-of-text (not same as ASCII <EOT>)
;	 FIFO   first-in, first-out structure
;	   FS   filesystem (not same as ASCII <FS>)
;	  HBA   SCSI host adapter
;	  HDA   hard disk drive
;	   HZ   frequency in Hertz
;	  IMR   interrupt mask register
;	  ISL   initial system load
;	  ISR   interrupt service routine or interrupt status register
;	  LBA   logical block address
;	  LBO   logical block offset
;	  LSB   least significant byte/bit
;	  LSD   least significant digit
;	 LSDW   least significant double word
;	  LSN   least significant nybble
;	  LSW   least significant word
;	  LUN   SCSI logical unit number
;	  MFU   major phuque-up
;	  MPU   microprocessor
;	  MSB   most significant byte/bit
;	  MSD   most significant digit
;	 MSDW   most significant double word
;	  MSN   most significant nybble
;	  MSW   most-significant word
;	NVRAM   non-volatile RAM
;	   OS   operating system
;	  OSL   operating system load
;	  RAM   random access memory
;	  RHR   DUART receiver holding register, aka FIFO
;	  RTC   real-time clock (see also WDT)
;	  RTS   TIA-232 request to send
;	  RxD   DUART data receive(r)
;	 SCSI   small computer system interface
;	 SDID   SCSI device bus ID
;	  TAM   BAM tag block bitmap
;	  TDM   magnetic tape drive
;	  THR   DUART transmitter holding register, aka FIFO
;	  TxD   DUART data transmit(ter)
;	   TZ   time zone
;	 UART   universal asynchronous receiver/transmitter, aka ACIA
;	  UTC   oordinated universal time, aka Zulu time
;	  WDT   watchdog timer (see also RTC)
;	   WS   whitespace, i.e., blanks & horizontal tabs
;	   ZP   page zero or direct page
;	
;	A word is defined as 16 bits.
;
;	    MPU REGISTER SYMBOLS
;	
;	   .A   8 bit accumulator LSB
;	   .B   8 bit accumulator MSB
;	   .C   16 bit accumulator
;	   .X   X-index
;	   .Y   Y-index
;	   DB   data bank
;	   DP   direct page
;	   PB   program bank
;	   PC   program counter
;	   PCH  program counter MSB
;	   PCL  program counter LSB
;	   SP   stack pointer
;	   SR   MPU status
;	
;
;	    MPU STATUS FLAG SYMBOLS
;	
;	    C   carry
;	    D   decimal mode
;	    I   maskable interrupts
;	    m   accumulator/memory size
;	    N   result negative
;	    V   sign overflow
;	    x   index registers size
;	    Z   result zero
;	
;
         .include "include/816macs_nc.asm"
         .include "include/local/macros.asm"
         .include "include/sys/macros_console.asm"
         .include "include/ascii.asm"
         .include "include/atomic.asm"
         .include "include/calendrics/atomic.asm"
         .include "include/architecture.asm"
         .include "include_hardware/maxim/ds1511_data.asm"
         .include "include_hardware/maxim/ds1511.asm"
         .include "include_hardware/ncr/53cf94_data.asm"
         .include "include_hardware/ncr/53cf94.asm"
         .include "include_hardware/nxp/common.asm"
         .include "include_hardware/nxp/registers.asm"
         .include "include_hardware/nxp/data.asm"
         .include "include_hardware/wdc/65c816.asm"
         .include "include_hardware/bcs/vquart.asm"
         .include "include_kernel/constants.asm"
         .include "include_boot/constants.asm"
         .include "include_kernel/constants_tia232.asm"
         .include "include_scsi/constants.asm"
         .include "include_mon/constants.asm"
         .include "include_kernel/display.asm"
         .include "include_scsi/display.asm"
         .include "include_scsi/masks.asm"
         .include "include_scsi/commands.asm"
         .include "include_scsi/devices.asm"
         .include "include_scsi/keys_driver.asm"
         .include "include_scsi/keys_status.asm"
         .include "include_scsi/offsets_cdb.asm"
         .include "include_scsi/offsets_misc.asm"
         .include "include_scsi/types_device.asm"
         .include "include_storage/atomic.asm"
         .include "include_storage/islmagic.asm"
         .include "include_storage/layout.asm"
         .include "include_storage/boot.asm"
         .include "include_kernel/directpage.asm"
         .include "include_scsi/directpage.asm"
         .include "include_mon/directpage.asm"
         .include "include_kernel/absolute.asm"
         .include "include_scsi/absolute.asm"
         .include "include_mon/absolute.asm"
         .include "include_boot/boot.asm"
         .include "include_kernel/devices.asm"
;================================================================================
;
;BIOS API JUMP TABLE
;
         *=mm_rom
;
         jmp getcha            ;read from SIO channel A
         jmp getchb            ;read from SIO channel B
         jmp getchc            ;read from SIO channel C
         jmp getchd            ;read from SIO channel D
         jmp putcha            ;write to SIO channel A
         jmp putchb            ;write to SIO channel B
         jmp putchc            ;write to SIO channel C
         jmp putchd            ;write to SIO channel D
         jmp chanbctl          ;disable/enable SIO channel B
         jmp getdtr            ;get date & time from RTC
         jmp getnvr            ;get data from NVRAM
         jmp getuxtim          ;get UNIX time
         jmp getsutim          ;get system up time
         jmp hrst              ;system reset
         jmp mon               ;M/L monitor entry
         jmp plot              ;position console cursor
         jmp putdtr            ;write date & time to RTC
         jmp putnvr            ;write data to NVRAM
         jmp scsicmd           ;execute SCSI command
         jmp scsihaid          ;get SCSI host adapter bus ID
         jmp scsiinit          ;initialize SCSI subsystem
         jmp scsiparm          ;get SCSI device parameters
         jmp setuxtim          ;set UNIX time
         jmp utdelay           ;generate user-defined time delay
;
         .include "kernel/reset.asm"
         .include "kernel/primitive.asm"
         .include "kernel/interrupt.asm"
         .include "kernel/time.asm"
         .include "kernel/display.asm"
         .include "scsi/scsiinit.asm"
         .include "scsi/scsienum.asm"
         .include "scsi/enumptst.asm"
         .include "scsi/anlyzcap.asm"
         .include "scsi/ssdstatf.asm"
         .include "scsi/scsiddid.asm"
         .include "scsi/scsidpiq.asm"
         .include "scsi/scsipdid.asm"
         .include "scsi/scsiparm.asm"
         .include "scsi/inquiry.asm"
         .include "scsi/testrdy.asm"
         .include "scsi/readcap.asm"
         .include "scsi/reqsens.asm"
         .include "scsi/startlun.asm"
         .include "scsi/scsicmd.asm"
         .include "scsi/sslotptr.asm"
         .include "mon/816mon.asm"
         .include "mon/cmd_code_assemble.asm"
         .include "mon/cmd_code_disassemble.asm"
         .include "mon/cmd_code_exec.asm"
         .include "mon/cmd_code_load.asm"
         .include "mon/cmd_mem_change.asm"
         .include "mon/cmd_mem_compare.asm"
         .include "mon/cmd_mem_copy.asm"
         .include "mon/cmd_mem_display.asm"
         .include "mon/cmd_mem_fill.asm"
         .include "mon/cmd_mem_search.asm"
         .include "mon/cmd_radix_display.asm"
         .include "mon/cmd_reg_change.asm"
         .include "mon/cmd_scsi.asm"
         .include "mon/cmd_version.asm"
         .include "mon/subs.asm"
         .include "mon/subs_scsi.asm"
         .include "mon/subs_srec.asm"
         .include "common/display.asm"
         .include "common/numeric.asm"
         .include "scsi/cdb.asm"
         .include "scsi/phases.asm"
         .include "tables/scsi.asm"
         .include "tables/sysvecs.asm"
         .include "tables/tia232.asm"
         .include "tables/ds1511.asm"
         .include "tables/ncr53c94.asm"
         .include "tables/nxp_duart.asm"
         .include "storage/magic.asm"
         .include "mon/tables.asm"
         .include "boot/strings.asm"
         .include "scsi/strings.asm"
         .include "mon/strings.asm"
;================================================================================
;
;MPU VECTOR TABLE COLLISION CHECK
;
_txtend_ =*
;
         .if _txtend_ > mm_mpuv
             .error "!!! PROGRAM COLLISION WITH MPU VECTOR TABLE !!!"
         .else
             .dcb mm_mpuv-_txtend_,$ff
         .endif
;
;================================================================================
;
;MPU VECTOR TABLE
;
         .word $ffff           ;undefined
         .word $ffff           ;undefined
v_icop   .word icop            ;coprocessor interrupt
v_ibrk   .word ibrk            ;software interrupt request
v_iabort .word iabort          ;abort interrupt
v_inmi   .word inmi            ;non-maskable interrupt
         .word $ffff           ;undefined
v_iirq   .word iirq            ;hardware interrupt request
;
;
;	65C02 emulation mode vectors...
;
         .word $ffff           ;undefined
         .word $ffff           ;undefined
         .word $ffff           ;undefined
         .word $ffff           ;undefined
         .word $ffff           ;undefined
         .word $ffff           ;undefined
v_hrst   .word hrst            ;hardware reset
         .word $ffff           ;undefined
;
;================================================================================
	.end